#include <netdb.h>
#include <QRegExp>
#include <kglobal.h>
#include <kdebug.h>
#include "addressdecoder.h"

class AddressDecoderPrivate
{
    public:
        AddressDecoderPrivate();
	QRegExp m_inetAddr;
	QRegExp m_inetPort;
	QHash<int,QString> m_families;
	QHash<int,QString> m_types;
	QHash<int,QString> m_protocols;
};

K_GLOBAL_STATIC(AddressDecoderPrivate, decoder)


AddressDecoderPrivate::AddressDecoderPrivate() :  m_inetAddr(" sin_addr=inet_addr\\(\"([\\d.]+)\"\\)\\}"),
    m_inetPort(" sin_port=htons\\(([\\d]+)\\)")
{
    m_families[AF_LOCAL]="UNIX";
    m_families[AF_INET]="INET";
    m_families[AF_INET6]="INET6";
    
    m_types[SOCK_STREAM]="stream";
    m_types[SOCK_DGRAM]="datagram";
    // TODO: more families and socket types
    m_protocols[0]="default";
}

QString AddressDecoder::address(const QStringList& params, int offset) 
{
    if (params[offset]!=" {sa_family=AF_INET") return "(unknown)";
    if (!decoder->m_inetPort.exactMatch(params[offset+1])) return "(unknown)";
    if (!decoder->m_inetAddr.exactMatch(params[offset+2])) return "(unknown)";
    QString ret=decoder->m_inetAddr.cap(1)+":"+decoder->m_inetPort.cap(1);
    kDebug() << "Decoded " << ret;
    return ret;
}

QString AddressDecoder::family(int af)
{
    return decoder->m_families[af];
}

QString AddressDecoder::type(int st)
{
    return decoder->m_types[st];
}

QString AddressDecoder::protocol(int proto) 
{
    if (!decoder->m_protocols.contains(proto)) {
        struct protoent* p=getprotobynumber(proto);
	if (p) decoder->m_protocols[proto]=QString(p->p_name);
	else decoder->m_protocols[proto]="invalid";
	//FIXME: is endprotoend needed here?
    }
    return decoder->m_protocols[proto];
}
