#include <QPainter>
#include <QModelIndex>
#include <kdebug.h>
#include "customitemdelegate.h"
#include "painterfactory.h"
#include "paintableitems.h"
#include "parameterlocator.h"
#include <cassert>

CustomItemDelegate::CustomItemDelegate(QObject *parent) :
        QItemDelegate(parent) {
}

void CustomItemDelegate::paint(QPainter *painter,
        const QStyleOptionViewItem &option, const QModelIndex & index ) const {
    PaintableItems items = index.data().value<PaintableItems>();

    painter->save();
    QHash<QString,QString> item;
    QRect rect = option.rect;
    ParameterLocator* parLocator = ParameterLocator::instance();
    parLocator->clearPaintedParameters(index);

    foreach(item, items.items) {
        QString type = item["type"];
        Painter::Base *p = PainterFactory::getPainter(type);
        if(p) {
            QRect r = p->paint(item,painter,option,rect);

            rect.setX(r.x() + r.width());
            parLocator->addPaintedParameter(index, rect.x());
        }
    }
    painter->restore();
}
