#include "base.h"
#include "filetracker.h"

#include <kdebug.h>
#include <cassert>

using namespace Decoder;

QHash<QString,QString> Base::generateInfo(DataType type, const QString& value, double time) const
{
    QHash<QString,QString> ret;
    switch (type) {
    case Text:
		ret["type"]="text";
		ret["text"]=value;
		break;
    case Filename:
		ret["type"]="filename";
		ret["path"]=value;
		if (value[0]=='"') {
			ret["path"].remove(0,1);
			ret["path"].chop(1);
		    }
		break;
    case File:
	        ret["type"] = "file";
		{
		    int fd=value.toInt();
	    	    ret["fd"] = QString::number(fd);
		    if (fd != -1) {
    			FileOpenInfo fileInfo = FT->lookupFileInfo(time, fd);
    			kDebug() << "Accessing file " << fileInfo.text() << endl;
    			ret["text"] = fileInfo.text();
			ret["extinfo"]=fileInfo.extraInfo();
		    }
		}
		break;
    default:
		assert(false);
    }
    return ret;
}


void Base::addParameter(DataType type, SysCallInfo &sysCall, int offset) const
{
    QHash<QString,QString> param=generateInfo(type,sysCall.parameters()[offset],sysCall.time());
    sysCall.parametersInfo().items.append(param);

}

void Base::addReturnValue(DataType type, SysCallInfo &sysCall) const
{
    QHash<QString,QString> param=generateInfo(type,sysCall.returnValue(),sysCall.time());
    sysCall.returnInfo().items.append(param);

}
