#include "generic.h"
#include "filetracker.h"
#include <cassert>
#include <kdebug.h>
#include <kconfig.h>
#include <kconfiggroup.h>

using namespace Decoder;

void Generic::decode(SysCallInfo &sysCall) {
    kDebug() << "Decoding generic syscall: " << syscallName;
    assert(sysCall.name() == syscallName);
    
    addReturnValue(returnType, sysCall);
    for (int i=0;i<paramTypes.size();i++) 
	addParameter(paramTypes[i], sysCall, i);
}

Generic::Generic(QString filename)
{
    KConfig cfg(filename);
    KConfigGroup group=cfg.group("System call");
    if (!group.exists()) return;
    returnType=toDatatype(group.readEntry("ReturnType"));
    if (returnType==Invalid) return;
    QStringList ptypes=group.readEntry("ParamTypes",QStringList());
    Q_FOREACH( QString ptype, ptypes) {
	DataType dt=toDatatype(ptype);
	if (dt==Invalid) return;
	paramTypes.append(dt);
    }
    // if constructor returned before setting syscall name, decoder is considered invalid
    
    syscallName=group.readEntry("Name");    
}

Generic::DataType Generic::toDatatype(const QString& type) const
{
    if (type=="text") return Text;
    if (type=="file") return File;
    if (type=="filename") return Filename;
    return Invalid;
}

QString Generic::name() const
{
    return syscallName;
}

    
