#include <kglobal.h>
#include <kdebug.h>
#include <kstandarddirs.h>
#include "decoderfactory.h"
#include "decoder/decoders.h"
#include "decoder/generic.h"

class DecoderFactoryPrivate
{
    public:
        DecoderFactoryPrivate();
        QHash<QString,Decoder::Base*> m_decoders;
};

K_GLOBAL_STATIC(DecoderFactoryPrivate, factory)


DecoderFactoryPrivate::DecoderFactoryPrivate()
{
    const QStringList files = KGlobal::dirs()->findAllResources("appdata", "*.syscall");
    m_decoders["open"]=new Decoder::Open();
    m_decoders["socket"]=new Decoder::Socket();
    m_decoders["connect"]=new Decoder::Connect();
    m_decoders["bind"]=new Decoder::Connect();
    m_decoders["accept"]=new Decoder::Accept();
    m_decoders["listen"]=new Decoder::Listen();
    
    QString fileName;
    Q_FOREACH (QString fileName, files) {
	Decoder::Generic* decoder=new Decoder::Generic(fileName);
	kdDebug() << "New decoder for " << decoder->name();
	if (decoder->name().isNull()) delete decoder;
	else m_decoders[decoder->name()]=decoder;
    }

}

Decoder::Base *DecoderFactory::getDecoder(QString syscall) {
    if (!factory->m_decoders.contains(syscall)) {
        return 0;
    } else {
        return factory->m_decoders[syscall];
    }
}
