#ifndef FILETRACKER_H
#define FILETRACKER_H

#include <QString>
#include <QHash>
#include <QVector>

#define FT (FileTracker::instance())

struct sockaddr;
class FileOpenInfo
{
    public:
	enum Type { File, Socket, Pipe, Unknown };
        FileOpenInfo();
	// for files
        FileOpenInfo(double time, int fd, const QString& fileName);
	// for sockets
	FileOpenInfo(double time, int fd, int family, int type, int protocol);
	FileOpenInfo(double time, int fd, const QString& family, const QString& type, const QString& protocol);
	// for accept() call
	FileOpenInfo(double time, const FileOpenInfo& parent, int fd, const QString& remote);
	void setListening(double time);
	void setLocalAddress(const QString& local, double time) { m_local=local; m_time=time; }
	void setRemoteAddress(const QString& remote, double time) { m_remote=remote; m_time=time; }
        double time() const;
        int fd() const;
	QString text() const;
	QString extraInfo() const;
    private:
        double m_time;
        int m_fd;
	// address and state, not used for files
        QString m_local;
	QString m_remote;
	bool m_listening;

	// info about fd, like filename or 'stream socket of AF_INET family'
	QString m_info;
	Type m_type;
};


class FileTracker
{
    public:
        static FileTracker *instance()
        {
            if (m_instance == 0) {
                m_instance = new FileTracker();
            }
            return m_instance;
        };

        ~FileTracker() {
            delete m_instance;
            m_instance = 0;
        }

        void fileOpened(double time, int fd, const QString& fileName);
	void socketListen(double time, int fd);
	void socketAddress(double time, int fd, const QString& addr, bool local);
	void socketAccepted(double time, int parentfd, int fd, const QString& addr);
	void socketCreated(double time, int fd, const QString& family, const QString& type, const QString& protocol);
        FileOpenInfo lookupFileInfo(double time, int fd);
	void populateFromPID(pid_t pid);

    private:
        FileTracker() {}
        void addFileOpenInfo(const FileOpenInfo& fileOpenInfo);

        static FileTracker *m_instance;
        QHash<int, QVector<FileOpenInfo> > m_fileDescriptors;
};

#endif // FILETRACKER_H
