/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    */

#include "stracerunner.h"
#include "parser.h"
#include "systemcallmodel.h"
#include "mainwindow.h"

//KDE include
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>

#include <QStringList>
#include <QTimer>


static const char description[] = "inspektor";

static const char version[] = "0.1";

int main(int argc, char **argv)
{
    KAboutData about("inspektor", 
            0, 
            ki18n("Inspektor"), 
            version, 
            ki18n(description),
            KAboutData::License_GPL);

    about.addAuthor(ki18n("Isaac Clerencia"), 
            KLocalizedString(), 
            "isaac@warp.es");

    about.addAuthor(ki18n("Javier Uruen"), 
            KLocalizedString(), 
            "juruen@warp.es");

    KCmdLineArgs::init(argc, argv, &about);

    KCmdLineOptions options;
    options.add("command <command>", ki18n("Command to trace"));
    options.add("pid <pid>", ki18n("PID of existing process to trace"));

    KCmdLineArgs::addCmdLineOptions(options);

    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    KApplication a;

    SystemCallModel sysCallModel;
    StraceRunner tracer;
    Parser parser(tracer.straceArgs(), &sysCallModel);
    a.connect(&tracer,
              SIGNAL(outputRead(const QStringList&)),
              &parser,
              SLOT(parseLines(const QStringList&)));

     
    if (args->isSet("command")) {
        tracer.setTraceCommand(args->getOption("command"));
        QTimer::singleShot(0, &tracer, SLOT(traceCommand()));	
    }
    else if (args->isSet("pid")) {
	tracer.setPID(args->getOption("pid").toUInt());
        QTimer::singleShot(0, &tracer, SLOT(traceCommand()));	
    }	

    MainWindow* mainWindow=new MainWindow();
    mainWindow->setSysCallModel(&sysCallModel);
    mainWindow->setStraceRunner(&tracer);

    mainWindow->show();

    a.exec();
    
}
