/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    */

#include "parser.h"
#include "decoderfactory.h"
#include "syscallinfo.h"

#include <kdebug.h>

Parser::Parser(const QStringList &straceArgs, SystemCallModel* model)
				: m_sysCallModel(model) 
{
    Q_UNUSED(straceArgs);
    //TODO: get the strace invocation options to know how to parse the line
    //    m_rx = new QRegExp("^([0-9]+).([0-9]+) +([a-z0-9]+)\\((.*)\\) += +(-?[0-9]+) +(([A-Z]+) \\((.*)\\))? +<([0-9]+.[0-9]+)>$");
    m_rx = new QRegExp("^([0-9]+)\\.([0-9]+) ([a-z0-9]+)\\((.*)\\) += ((?:[0-9]+)|(?:0x[0-9a-f]+)|(?:-[0-9]+)) (?:(E[A-Z]+) \\(.*\\) )?<([0-9]+\\.[0-9]+)>");

    QStringList header;
    header << "Line" << "Name" << "Parameters" << "Return" << "Errno" <<
      "Elapsed Time";
    if (model) {
        model->setHeader(header);
    }
}

Parser::~Parser() {

}

void Parser::parseLines(const QStringList &lines) {
    foreach(QString line, lines) {
        parseLine(line);
    }
}

void Parser::parseLine(const QString &line) {
   // kDebug () << "Trying to parse " <<  line;
    if (m_rx->exactMatch(line.trimmed())) {
        kDebug() << "Match!";
        QStringList capts = m_rx->capturedTexts();
        kDebug () << "capts " <<  capts[3];
        SysCallInfo sysCall(capts);
        Decoder::Base *dec = DecoderFactory::getDecoder(sysCall.name());
        if (dec) {
            dec->decode(sysCall);
	} else {
	    sysCall.setDefaultItems();
	}
        addSysCallToModel(sysCall);
    } else {
        kDebug() << "Doesn't match!";
    }
}

void Parser::addSysCallToModel(const SysCallInfo sysCall)
{
    if (!m_sysCallModel) {
        kDebug() << "There's no model to add syscall info";
        return;
    }
    m_sysCallModel->addSystemCall(sysCall);
}
#include "parser.moc"
