/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STRACERUNNER_H
#define STRACERUNNER_H

#include <QStringList>
#include <QObject>



class KProcess;
class QFileSystemWatcher;
class QFile;

class StraceRunner : public  QObject
{
    Q_OBJECT

    public:
        StraceRunner();
        ~StraceRunner();
        QString tracedCommand() const;
        void setTraceCommand(const QString & command,
                             const QStringList & args = QStringList());
        QStringList straceArgs();
	void setPID(pid_t pid);
	pid_t getPID() const;
    
    public slots:
        void traceCommand();
        void stopTrace(); 
        void processFinished(int exitCode);
        void readyRead();

    signals:
        void outputRead(const QStringList& lines);
        void tracing();
        void traceFinished(int exitCode);


    private:
        void prepareTrace();

    private:
        QStringList m_straceArgs;
        QString m_exec;
        QStringList m_args;
        bool    m_running;
        KProcess *m_strace;
        QFile *m_outFile;
        QFileSystemWatcher *m_fileWatcher;
	QString m_partialLine;
	pid_t m_pid;
};

#endif

