/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "syscallinfo.h"

#include <kdebug.h>

SysCallInfo::SysCallInfo(QStringList parts) {
    m_line = parts.join(" ");
    m_time = (parts[1] + "." + parts[2]).toDouble();
    m_name = parts[3];
    m_parameters = parts[4].split(",");
    m_returnValue = parts[5];
    m_errno = parts[6];
    m_elapsedTime = parts[7].toDouble();
}

QString SysCallInfo::line() const {
    return m_line;
}

QString SysCallInfo::name() const {
    return m_name;
}

QStringList SysCallInfo::parameters() const {
    return m_parameters;
}

QString SysCallInfo::returnValue() const {
    return m_returnValue;
}

QString SysCallInfo::errno() const {
    return m_errno;
}

double SysCallInfo::elapsedTime() const {
    return m_elapsedTime;
}

double SysCallInfo::time() const {
    return m_time;
}

PaintableItems &SysCallInfo::parametersInfo() {
    return m_parametersInfo;
}

PaintableItems &SysCallInfo::returnInfo() {
    return m_returnInfo;
}

const PaintableItems &SysCallInfo::parametersInfo() const {
    return m_parametersInfo;
}

const PaintableItems &SysCallInfo::returnInfo() const {
    return m_returnInfo;
}

void SysCallInfo::setDefaultItems() {
    foreach (QString par, m_parameters) {
         QHash<QString,QString> param;
	 param["type"] = "text";
	 param["text"] = par;
         m_parametersInfo.items.append(param);
    }
    QHash<QString,QString> ret;
    ret["type"] = "text";
    ret["text"] = m_returnValue;
    m_returnInfo.items.append(ret);
}
