#include "syscalltable.h"

#include "paintableitems.h"
#include "parameterlocator.h"
#include "painterfactory.h"
#include "kdebug.h"
#include <krun.h>
#include <kurl.h>

#include <QHeaderView>
#include <QToolTip>
#include <QMouseEvent>
#include <QHelpEvent>


SyscallTable::SyscallTable(QWidget *parent = 0) : QTableView(parent) {}

bool SyscallTable::event(QEvent *e)
{
    if (e->type() == QEvent::ToolTip) {
        QHelpEvent *ehelp = static_cast<QHelpEvent*>(e);
        QPoint pos(ehelp->x(), ehelp->y() - horizontalHeader()->height());
        showToolTip(pos);
    }
    return QTableView::event(e);
}

void SyscallTable::showToolTip(const QPoint& pos)
{
    kDebug() << "Trying to show tooltip for " << pos << endl;
    
    QModelIndex index = indexAt(pos);
    if (index.data().canConvert<PaintableItems>()) {
        kDebug() << "custom column( " <<  index.column() << "," 
                 << index.row() << ")" <<  endl;
        int param = ParameterLocator::instance()
                        ->lookupParameterPosition(index, pos.x());
        kDebug() << "Parameter: " << param << endl;
        if (param != -1) {
            PaintableItems items = index.data().value<PaintableItems>();
            QHash<QString,QString> item = items.items[param];
            QString type = item["type"];
            Painter::Base *p = PainterFactory::getPainter(type);
            if(p) {
                QString text = p->toolTip(item);
                QToolTip::showText(mapToGlobal(pos),text,this);
            }
        }
    }
}

void SyscallTable::mousePressEvent(QMouseEvent* event)
{
    QModelIndex index = indexAt(event->pos());
    if (index.isValid() && index.column()==1) {
	QString manName=index.data().toString();
	// if name is something like stat64, try getting manpage for stat instead, as
	// manpages for 64-bit versions often do not exist
	if (manName.endsWith("64")) manName.resize(manName.size()-2);
	new KRun(KUrl(QString("man:/%1(2)").arg(manName)),0);
    }
    QTableView::mousePressEvent(event);
}
