/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <kdebug.h>
#include <QColor>
#include <cassert>

#include "systemcallmodel.h"
#include "paintableitems.h"

const int SystemCallModel::Line = 0;
const int SystemCallModel::Name = 1;
const int SystemCallModel::Parameters = 2;
const int SystemCallModel::ReturnValue = 3;
const int SystemCallModel::Errno = 4;
const int SystemCallModel::ElapsedTime = 5;

SystemCallModel::SystemCallModel() : QAbstractTableModel(0)
{

}

SystemCallModel::~SystemCallModel() 
{

}

int SystemCallModel::rowCount(const QModelIndex & parent)  const
{
   Q_UNUSED(parent);
   return m_sysCallList.count();
}

int SystemCallModel::columnCount(const QModelIndex & parent) const
{
   Q_UNUSED(parent);
   return 6;
}


QVariant SystemCallModel::data (const QModelIndex & index, int role) const
{
    if (role == Qt::TextColorRole and sysCallFailed(index)) {
        return QColor("red");
    }

    if (role != Qt::DisplayRole) {
        return QVariant();
    }

    const SysCallInfo &row = m_sysCallList[index.row()];

    switch (index.column()) {
        case Line:
            return row.line();
            break;

        case Name:
            return row.name();
            break;

        case Parameters:
        {
            QVariant var;
	    var.setValue(row.parametersInfo());
            return var;
            break;
        }
        case ReturnValue:
        {
            QVariant var;
	    var.setValue(row.returnInfo());
            return var;
            break;
        }
        case Errno:
            return row.errno();
            break;

        case ElapsedTime:
            return QString::number(row.elapsedTime());
            break;

        default:
            assert(false);
    }
    return QVariant(); //Keep compiler happy
}

void SystemCallModel::addSystemCall(const SysCallInfo & sysCall) 
{
    kDebug() << "Appending sys call to model";
    m_sysCallList.append(sysCall);
    kDebug() << "Number of rows: " << m_sysCallList.count();
    reset();
}

QVariant SystemCallModel::headerData(int section, 
										 Qt::Orientation orientation, 
										 int role) const
{
	if (role != Qt::DisplayRole) {
		return QVariant();
	}

	if (orientation == Qt::Horizontal) {
		if (m_headerData.count() <= section) {
			return QVariant();
		} else {
			return m_headerData[section];
		}	
	} else {
		return QVariant();
	}

}

void SystemCallModel::setHeader(const QStringList & header) 
{
	m_headerData = header;
	kDebug() << "Header" << header;
}

bool SystemCallModel::sysCallFailed(const QModelIndex& index) const
{
	int row = index.row();

	if (m_sysCallList.count() <= row)  return false;

	if (m_sysCallList[row].returnValue().toInt() < 0) {
		return true;
	} else {
		return false;
	}
}

void SystemCallModel::restart()
{
    m_sysCallList.clear();
    reset();
}

#include "systemcallmodel.moc"
