/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SYSTEMCALLMODEL_H
#define SYSTEMCALLMODEL_H

#include <QAbstractTableModel>
#include <QList>
#include <QStringList>

#include "syscallinfo.h"

class SystemCallModel : public QAbstractTableModel 
{
    Q_OBJECT

    public:
        static const int Line;
        static const int Name;
        static const int Parameters;
        static const int ReturnValue;
        static const int Errno;
        static const int ElapsedTime;

        SystemCallModel();
        ~SystemCallModel();
        int rowCount(const QModelIndex & parent = QModelIndex()) const;
        int columnCount(const QModelIndex & parent = QModelIndex()) const;
        QVariant data (const QModelIndex & index, 
                int role = Qt::DisplayRole ) const;
        QVariant headerData(int section, 
                Qt::Orientation orientation,
                int role) const;
        void setHeader(const QStringList & header);

    public slots:
        void addSystemCall(const SysCallInfo & sysCall);
        void restart();

    private:
        bool sysCallFailed(const QModelIndex& index) const;

    private:
        QList<SysCallInfo> m_sysCallList;
        QStringList m_headerData;

};

#endif

