/*
This code was created by Ian Ventura-Whiting (Fizz) - fizz@titania.co.uk

ODBC Statistics (for QT 4)
Copyright (C) 2006 Ian Ventura-Whiting

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


#include "maindialog.h"
#include "aboutdialog.h"

#include <QLabel>
#include <QGridLayout>
#include <QPushButton>
#include <QTabWidget>
#include <QSpacerItem>
#include <QTableWidget>
#include <QProgressBar>
#include <QSpinBox>
#include <QTimer>
#include <QFile>
#include <QDir>

#include <pwd.h>
#include <uodbc_stats.h>


// -----------------------------------------------------------------------------
// Constructor and Destructor Stuff...

MainDialog::MainDialog(QDialog *parent) : QDialog(parent)
{
	// Variables...
	QStringList headerList;

	// Variables Init...
	procExists = QFile::exists("/proc");
	statsHandle = 0;
	if (procExists)
		headerList << tr("User") << tr("Program") << tr("Process ID") << tr("Environments") << tr("Connections") << tr("Statements") << tr("Descriptors");
	else
		headerList << tr("Process ID") << tr("Environments") << tr("Connections") << tr("Statements") << tr("Descriptors");

	// Main dialog init...
	setWindowTitle(tr("ODBC Statistics"));

	// Create main dialog widgets and configure...
	gridLayout = new QGridLayout(this);
	statsTabs = new QTabWidget(this);
	gridLayout->addWidget(statsTabs, 0, 0, 1, 5);
	label = new QLabel(this);
	label->setText(tr("Refresh Interval:"));
	gridLayout->addWidget(label, 1, 0);
	intervalSpin = new QSpinBox(this);
	intervalSpin->setMinimum(5);
	intervalSpin->setMaximum(900);
	intervalSpin->setValue(7);
	gridLayout->addWidget(intervalSpin, 1, 1);
	spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Fixed);
	gridLayout->addItem(spacer, 1, 2);
	aboutButton = new QPushButton(this);
	aboutButton->setText(tr("&About"));
	gridLayout->addWidget(aboutButton, 1, 3);
	closeButton = new QPushButton(this);
	closeButton->setText(tr("&Close"));
	gridLayout->addWidget(closeButton, 1, 4);

	// Create summary tab widgets and configure...
	tabWidget = new QWidget();
	gridLayout = new QGridLayout(tabWidget);
	label = new QLabel(tabWidget);
	label->setText(tr("Environments:"));
	gridLayout->addWidget(label, 0, 0);
	envProgress = new QProgressBar(tabWidget);
	envProgress->setMinimum(0);
	envProgress->setMaximum(5);
	envProgress->setValue(0);
	envProgress->setTextVisible(FALSE);
	gridLayout->addWidget(envProgress, 0, 1);
	envLabel = new QLabel(tabWidget);
	envLabel->setText("0");
	gridLayout->addWidget(envLabel, 0, 2);
	label = new QLabel(tabWidget);
	label->setText(tr("Connections:"));
	gridLayout->addWidget(label, 1, 0);
	conProgress = new QProgressBar(tabWidget);
	conProgress->setMinimum(0);
	conProgress->setMaximum(5);
	conProgress->setValue(0);
	conProgress->setTextVisible(FALSE);
	gridLayout->addWidget(conProgress, 1, 1);
	conLabel = new QLabel(tabWidget);
	conLabel->setText("0");
	gridLayout->addWidget(conLabel, 1, 2);
	label = new QLabel(tabWidget);
	label->setText(tr("Statements:"));
	gridLayout->addWidget(label, 2, 0);
	staProgress = new QProgressBar(tabWidget);
	staProgress->setMinimum(0);
	staProgress->setMaximum(5);
	staProgress->setValue(0);
	staProgress->setTextVisible(FALSE);
	gridLayout->addWidget(staProgress, 2, 1);
	staLabel = new QLabel(tabWidget);
	staLabel->setText("0");
	gridLayout->addWidget(staLabel, 2, 2);
	label = new QLabel(tabWidget);
	label->setText(tr("Descriptors:"));
	gridLayout->addWidget(label, 3, 0);
	desProgress = new QProgressBar(tabWidget);
	desProgress->setMinimum(0);
	desProgress->setMaximum(5);
	desProgress->setValue(0);
	desProgress->setTextVisible(FALSE);
	gridLayout->addWidget(desProgress, 3, 1);
	desLabel = new QLabel(tabWidget);
	desLabel->setText("0");
	gridLayout->addWidget(desLabel, 3, 2);
	spacer = new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding);
	gridLayout->addItem(spacer, 4, 0, 1, 3);
	statsTabs->addTab(tabWidget, tr("Summary"));

	// Create details tab widget and configure...
	tabWidget = new QWidget();
	gridLayout = new QGridLayout(tabWidget);
	detailTable = new QTableWidget(tabWidget);
	if (procExists)
		detailTable->setColumnCount(7);
	else
		detailTable->setColumnCount(5);
	detailTable->setHorizontalHeaderLabels(headerList);
	detailTable->setShowGrid(TRUE);
	gridLayout->addWidget(detailTable, 0, 0);
	statsTabs->addTab(tabWidget, tr("Details"));

	// Get initial stats...
	load();

	// Set minimum Dialog Size...
	setMinimumSize(sizeHint());

	// Refresh Timer...
	refreshTimer = new QTimer(this);
	refreshTimer->setInterval(intervalSpin->value() * 100);

	// Connect SIGNAL / SLOTS...
	connect(closeButton, SIGNAL(clicked(bool)), this, SLOT(close()));
	connect(aboutButton, SIGNAL(clicked(bool)), this, SLOT(about()));
	connect(refreshTimer, SIGNAL(timeout()), this, SLOT(load()));
	connect(intervalSpin, SIGNAL(valueChanged(int)), this, SLOT(updateTimer(int)));

	// Start timer...
	refreshTimer->start();
}


MainDialog::~MainDialog()
{
	// Cleanup...
	uodbc_close_stats(statsHandle);
}


// -----------------------------------------------------------------
// Slots...


// About dialog...
void MainDialog::about()
{
	AboutDialog aboutDialog(this);
	aboutDialog.exec();
}


// Update the interval timer...
void MainDialog::updateTimer(int newInterval)
{
	refreshTimer->stop();
	refreshTimer->setInterval(newInterval * 100);
	refreshTimer->start();
}


// Clear Detail Table Row...
void MainDialog::clearDetailRow(int row)
{
	// Variables...
	int column;
	QTableWidgetItem *detail;

	// Clear Row...
	for (column = 0; column < detailTable->columnCount(); column++)
	{
		detail = detailTable->item(row, column);
		detail->setText("");
	}
}


// Refresh stats...
void MainDialog::load()
{
	// Variables...
	int progressMaxValue;
	int pidTotal;
	int handleNum;
	int detailColumn;
	int userID;
	QString tempString;
	QString processUser;
	QString processProgram;
	QFile processFile;
	QTableWidgetItem *detail;
	uodbc_stats_retentry statsResult[4];
	uodbc_stats_retentry pidResults[MAXPROCESSES];
	uodbc_stats_retentry handleResult[MAXHANDLES];
	struct passwd* passwdItem;

	// Get ODBC stats handle...
	if (!statsHandle)
		uodbc_open_stats(&statsHandle, UODBC_STATS_READ);

	// Get stats...
	if (statsHandle != 0)
	{
		// ODBC Summary Stats information...
		if (uodbc_get_stats(statsHandle, -1, statsResult, 4) == 4)
		{
			// Get Progress Bar Maximum...
			progressMaxValue = staProgress->maximum();
			if (statsResult[0].value.l_value > progressMaxValue)
				progressMaxValue = statsResult[0].value.l_value;
			if (statsResult[1].value.l_value > progressMaxValue)
				progressMaxValue = statsResult[1].value.l_value;
			if (statsResult[2].value.l_value > progressMaxValue)
				progressMaxValue = statsResult[2].value.l_value;
			if (statsResult[3].value.l_value > progressMaxValue)
				progressMaxValue = statsResult[3].value.l_value;

			// Set Progress Bar Maximum...
			envProgress->setMaximum(progressMaxValue);
			conProgress->setMaximum(progressMaxValue);
			staProgress->setMaximum(progressMaxValue);
			desProgress->setMaximum(progressMaxValue);

			// Set Progress Bar Values...
			envProgress->setValue(statsResult[0].value.l_value);
			conProgress->setValue(statsResult[1].value.l_value);
			staProgress->setValue(statsResult[2].value.l_value);
			desProgress->setValue(statsResult[3].value.l_value);

			// Set Label Values...
			envLabel->setText(QString::number(statsResult[0].value.l_value));
			conLabel->setText(QString::number(statsResult[1].value.l_value));
			staLabel->setText(QString::number(statsResult[2].value.l_value));
			desLabel->setText(QString::number(statsResult[3].value.l_value));
		}

		// Get Detailed Stats Info...
		pidTotal = uodbc_get_stats(statsHandle, 0, pidResults, MAXPROCESSES);
		detailTable->clearContents();
		detailTable->setRowCount(pidTotal);

		// Update stat details table...
		for (int pidNumber = 0; pidNumber < pidTotal; pidNumber++)
        {
			handleNum = uodbc_get_stats(statsHandle, pidResults[pidNumber].value.l_value, handleResult, MAXHANDLES);
			if (handleNum > 0)
			{
				// Init...
				detailColumn = 0;

				if (procExists)
				{
					// Init...
					processUser = "";
					processProgram = "";

					// Configure /proc file...
					tempString = "/proc/";
					tempString.append(QString::number(pidResults[pidNumber].value.l_value));
					QDir::setCurrent(tempString);
					processFile.setFileName("status");

					// Open /proc file...
					if (processFile.open(QIODevice::ReadOnly))
					{
						// Read line from status file...
						tempString = processFile.readLine();

						while ((!tempString.isEmpty()) && ((processProgram.isEmpty()) || (processUser.isEmpty())))
						{
							// If program name...
							if (tempString.contains("Name:") > 0)
								processProgram = tempString.right(tempString.length() - 6);

							// If user ID...
							else if (tempString.contains("Uid:") > 0)
							{
								processUser = tempString.mid(5, (tempString.indexOf((char)9, 6) - 5));
								userID = processUser.toInt();
								passwdItem = getpwuid(userID);
								processUser = passwdItem->pw_name;
							}

							// Read line from status file...
							tempString = processFile.readLine();
						}
						processFile.close();
					}

					// Add User to details table...
					detail = new QTableWidgetItem(processUser);
					detailTable->setItem(pidNumber, detailColumn, detail);
					detailColumn++;

					// Add Program to details table...
					detail = new QTableWidgetItem(processProgram);
					detailTable->setItem(pidNumber, detailColumn, detail);
					detailColumn++;
				}

				// Add Process ID to details table...
				detail = new QTableWidgetItem(QString::number(pidResults[pidNumber].value.l_value));
				detailTable->setItem(pidNumber, detailColumn, detail);
				detailColumn++;

				// Add remaining details...
				for (int handle = 0; handle < MAXHANDLES; handle++)
				{
					detail = new QTableWidgetItem(QString::number(handleResult[handle].value.l_value));
					detailTable->setItem(pidNumber, detailColumn, detail);
					detailColumn++;
				}
			}
			else
				clearDetailRow(pidNumber);
		}
	}
}

