# - Try to find LibShaman
# Once done this will define
#
#  LIBSHAMAN_FOUND - LibShaman was found on the target system
#  LIBSHAMAN_INCLUDE_DIR - LibShaman include directory, to be added with include_directories
#  LIBSHAMAN_LIBRARY - LibShaman's library to link against
#  LIBSHAMAN_BACKEND_NAME - The name of the backend used by LibShaman
#
# Copyright (c) 2009, Dario Freddi, <drf@kde.org>
#
# Redistribution and use is allowed according to the terms of the BSD license.
# For details see the accompanying COPYING-CMAKE-SCRIPTS file.

if (LIBSHAMAN_INCLUDE_DIR AND LIBSHAMAN_LIBRARY)
    set(LIBSHAMAN_FIND_QUIETLY TRUE)
endif (LIBSHAMAN_INCLUDE_DIR AND LIBSHAMAN_LIBRARY)

if (NOT LIBSHAMAN_MIN_VERSION)
  set(LIBSHAMAN_MIN_VERSION "0.60.0")
endif (NOT LIBSHAMAN_MIN_VERSION)

find_path( LIBSHAMAN_INCLUDE_DIR
     NAMES shamanbackend.h
     PATH_SUFFIXES shaman
)

find_file( LIBSHAMAN_VERSION_FILE
     NAMES shamanmacros.h
     PATHS ${LIBSHAMAN_INCLUDE_DIR}
)

set(LIBSHAMAN_VERSION_OK TRUE)
if(LIBSHAMAN_VERSION_FILE)
  file(READ ${LIBSHAMAN_VERSION_FILE} LIBSHAMAN_VERSION_CONTENT)

  # Read and set the name first
  string (REGEX MATCH "LIBSHAMAN_BACKEND_NAME \".*\"\n" LIBSHAMAN_BACKEND_NAME_MATCH "${LIBSHAMAN_VERSION_CONTENT}")
  if(LIBSHAMAN_BACKEND_NAME_MATCH)
    string(REGEX REPLACE "LIBSHAMAN_BACKEND_NAME \"(.*)\"\n" "\\1" LIBSHAMAN_BACKEND_NAME ${LIBSHAMAN_BACKEND_NAME_MATCH})
  endif(LIBSHAMAN_BACKEND_NAME_MATCH)

  # Now check the version
  string (REGEX MATCH "LIBSHAMAN_VERSION_STRING \".*\"\n" LIBSHAMAN_VERSION_MATCH "${LIBSHAMAN_VERSION_CONTENT}")

  if(LIBSHAMAN_VERSION_MATCH)
    string(REGEX REPLACE "LIBSHAMAN_VERSION_STRING \"(.*)\"\n" "\\1" LIBSHAMAN_VERSION ${LIBSHAMAN_VERSION_MATCH})
    if(LIBSHAMAN_VERSION STRLESS "${LIBSHAMAN_MIN_VERSION}")
      set(LIBSHAMAN_VERSION_OK FALSE)
      if(LibShaman_FIND_REQUIRED)
        message(FATAL_ERROR "LibShaman version ${LIBSHAMAN_VERSION} was found, but it is too old. Please install
                             ${LIBSHAMAN_MIN_VERSION} or newer.")
      else(LibShaman_FIND_REQUIRED)
        message(STATUS "LibShaman version ${LIBSHAMAN_VERSION} is too old. Please install ${LIBSHAMAN_MIN_VERSION} or newer.")
      endif(LibShaman_FIND_REQUIRED)
    endif(LIBSHAMAN_VERSION STRLESS "${LIBSHAMAN_MIN_VERSION}")
  endif(LIBSHAMAN_VERSION_MATCH)
elseif(LIBSHAMAN_INCLUDE_DIR)
  # The version is so old that it does not even have the file
  set(LIBSHAMAN_VERSION_OK FALSE)
  if(Shaman_FIND_REQUIRED)
    message(FATAL_ERROR "It looks like LibShaman is too old. Please install Shaman version ${LIBSHAMAN_MIN_VERSION} or newer.")
  else(Shaman_FIND_REQUIRED)
    message(STATUS "It looks like LibShaman is too old. Please install Shaman version ${LIBSHAMAN_MIN_VERSION} or newer.")
  endif(Shaman_FIND_REQUIRED)
endif(LIBSHAMAN_VERSION_FILE)

find_library( LIBSHAMAN_LIBRARY
    NAMES shamancore
)

include(FindPackageHandleStandardArgs)

# handle the QUIETLY and REQUIRED arguments and set LIBSHAMAN_FOUND to TRUE if
# all listed variables are TRUE
find_package_handle_standard_args(LibShaman DEFAULT_MSG LIBSHAMAN_LIBRARY LIBSHAMAN_INCLUDE_DIR LIBSHAMAN_VERSION_OK)

mark_as_advanced(LIBSHAMAN_INCLUDE_DIR LIBSHAMAN_LIBRARY LIBSHAMAN_BACKEND_NAME LIBSHAMAN_VERSION_OK)
