/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>
   Copyright (C) 2010 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "AkabeiBackend.h"

#include <akabeiclientbackend.h>
#include <akabeigroup.h>
#include <akabeidatabase.h>
#include <akabeigroup.h>

#include <kicon.h>
#include <kdebug.h>

#include "AkabeiPackage.h"
#include <shamanbackendpackage.h>
#include <QTimer>
#include "shamanmacros_p.h"


AkabeiBackend::AkabeiBackend()
    : Shaman::BackendInterface(0),
      m_init(true)
{

}

AkabeiBackend::~AkabeiBackend()
{

}

bool AkabeiBackend::aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate)
{
    if (/*(oldstate == Shaman::Backend::Initializing && newstate == Shaman::Backend::Idling) ||*/
        (oldstate == Shaman::Backend::OnTransaction && newstate == Shaman::Backend::Idling)) {
        // Time to reload, guys.
        QTimer::singleShot(0, this, SLOT(reload()));
        // If we're initializing, just don't reject the state transition
        /*if (oldstate != Shaman::Backend::Initializing) {
            // Otherwise do.
            return false;
        }*/
    }
    return Shaman::BackendInterface::aboutToChangeState(oldstate, newstate);
}

void AkabeiBackend::transactionQuestionAnswered(Shaman::TransactionQuestion* question)
{

}

void AkabeiBackend::init()
{
    kDebug();
    //setState(Shaman::Backend::Initializing);
    setCapabilities(/*Transaction capabilities*/
                    Shaman::BackendCapabilities::InstallPackagesCapability | 
                    Shaman::BackendCapabilities::RemovePackagesCapability |
                    Shaman::BackendCapabilities::UpgradePackagesCapability | 
                    Shaman::BackendCapabilities::UpdateRepositoriesCapability |
                    Shaman::BackendCapabilities::InstallPackageFromFileCapability | 
                    Shaman::BackendCapabilities::UpgradeSystemCapability,
                    /*Search capabilities*/
                    Shaman::BackendCapabilities::BasicSearchFromViewCapability | 
                    Shaman::BackendCapabilities::ExtendedSearchFromViewCapability,
                    /*Package Grouping capabilities*/
                    Shaman::BackendCapabilities::GroupPackagesByPackageGroupsCapability |
                    Shaman::BackendCapabilities::GroupPackagesByRepositoryCapability,
                    /*Information capabilities*/
                    Shaman::BackendCapabilities::ShowFileListCapability |
                    Shaman::BackendCapabilities::ShowDependsOnCapability |
                    Shaman::BackendCapabilities::ShowRequiredByCapability |
                    Shaman::BackendCapabilities::ShowConflictsWithCapability |
                    Shaman::BackendCapabilities::ShowProvidesCapability |
                    Shaman::BackendCapabilities::ShowPackageChangelog);

    KAboutPerson lukas(ki18n("Lukas Appelhans"), ki18n("Maintainer"), "l.appelhans@gmx.de", "http://boom1992.wordpress.com");
    KAboutPerson dario(ki18n("Dario Freddi"), ki18n("Maintainer"), "drf@kde.org", "http://drfav.wordpress.com");
    // TODO: Change package icon
    setProperties(i18n("Akabei"), i18n("A backend for the Chakra Project."), KIcon("applications-other"),
                  KIcon("application-x-deb"), QList<KAboutPerson>() << dario << lukas, QStringList());

    connect(AkabeiClient::Backend::instance(), SIGNAL(statusChanged(Akabei::Backend::Status)),
            this, SLOT(backendStatusChanged(Akabei::Backend::Status)));
    connect(AkabeiClient::Backend::instance()->transactionHandler(), SIGNAL(newTransactionMessage(QString)), SLOT(addTransactionMessage(QString)));
    connect(AkabeiClient::Backend::instance()->transactionHandler(), SIGNAL(phaseChanged(AkabeiClient::TransactionHandler::Phase)), SLOT(phaseChanged(AkabeiClient::TransactionHandler::Phase)));
    connect(AkabeiClient::Backend::instance()->transactionHandler(), SIGNAL(packageProgressChanged(Akabei::Package *, int)), SLOT(packageProgressChanged(Akabei::Package *, int)));
    connect(AkabeiClient::Backend::instance()->transactionHandler(), SIGNAL(errorsOccurred(QList<Akabei::Error*>&)), SLOT(errorsOccurred(QList<Akabei::Error*>&)));
    connect(AkabeiClient::Backend::instance()->transactionHandler(), SIGNAL(validationFinished(bool)), SLOT(validationFinished(bool)));
    connect(AkabeiClient::Backend::instance()->transactionHandler(), SIGNAL(finished()), SLOT(transactionFinished()));

    AkabeiClient::Backend::instance()->initialize();
}

void AkabeiBackend::backendStatusChanged(Akabei::Backend::Status status)
{
    kDebug() << status;
    if (status == Akabei::Backend::StatusReady) {
        disconnect(AkabeiClient::Backend::instance(), SIGNAL(statusChanged(Akabei::Backend::Status)),
            this, SLOT(backendStatusChanged(Akabei::Backend::Status)));
        kDebug();
        reload();
        if (m_init) {
            foreach (AkabeiClient::DatabaseHandler * handler, AkabeiClient::Backend::instance()->databases()) {
                connect(handler, SIGNAL(statusChanged(AkabeiClient::DatabaseHandler::Status)), SLOT(databaseStatusChanged(AkabeiClient::DatabaseHandler::Status)));
                m_databases[handler] = false;
            }
            m_databases[AkabeiClient::Backend::instance()->localDatabase()] = false;
            connect(AkabeiClient::Backend::instance()->localDatabase(), SIGNAL(statusChanged(AkabeiClient::DatabaseHandler::Status)), SLOT(databaseStatusChanged(AkabeiClient::DatabaseHandler::Status)));
            m_init = false;
        }
    }
}

bool AkabeiBackend::cancelQueueProcessing()
{
    return false;
}

void AkabeiBackend::processQueue(const Shaman::AbstractPackage::List& packages, const Shaman::ProcessQueueOption::List &options)
{
    /*
        void phaseChanged(AkabeiClient::TransactionHandler::Phase phase); DONE
        void packageProgressChanged(Akabei::Package * pkg, int percent); DONE
        void downloadSpeedChanged(Akabei::Package * pkg, int speed); NEEDS TO BE IMPLEMENTED IN TRANSACTIOHANDLER
        void newTransactionMessage(const QString &message); DONE
        void errorsOccurred(QList<Akabei::Error*>); DONE
        void finished();*/ //TODO: Move the connects to the first time we will process the queue<

    AkabeiClient::Backend::instance()->transactionHandler()->process();
}

void AkabeiBackend::transactionFinished()
{
    setState(Shaman::Backend::Idling);
    AkabeiClient::Backend::instance()->queue()->clean();
}

void AkabeiBackend::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
    qDebug() << "Mark additional targets";
    //TODO: Maybe also already validate the queue + transactions?
    //TODO: Maybe remove not anymore marked packages?!
    
    AkabeiClient::Queue *queue = AkabeiClient::Backend::instance()->queue();
    
    foreach (Shaman::AbstractPackage * package, backend()->queuedPackages()) {
        if (package->type() != Shaman::AbstractPackage::BackendPackageType)
            continue;
        switch (package->action()) {
            case Shaman::AbstractPackage::Install:
            {
                /*Shaman::AbstractPackage::List deps = computeDependencies(m_packages.key(static_cast<Shaman::BackendPackage*>(package)));
                foreach (Shaman::AbstractPackage * pkg, deps) {
                    if (pkg->type() != Shaman::AbstractPackage::BackendPackageType)
                        break;//TODO: Throw an error maybe?
                    pkg->setAction(Shaman::AbstractPackage::TransactionDependencyInstall);
                }*/
                queue->addPackage(m_packages.key(static_cast<Shaman::BackendPackage*>(package)), AkabeiClient::Install);
                break;
            }
            case Shaman::AbstractPackage::Upgrade:
                queue->addPackage(m_packages.key(static_cast<Shaman::BackendPackage*>(package)), AkabeiClient::Update);
                break;
            case Shaman::AbstractPackage::Uninstall:
                queue->addPackage(m_packages.key(static_cast<Shaman::BackendPackage*>(package)), AkabeiClient::Remove);
                break;
            default:
                break;
        };
    }

    AkabeiClient::Backend::instance()->transactionHandler()->validate();
    //TODO: Validate + set other stuff later
    //backend()->queuedPackages()
}

void AkabeiBackend::validationFinished(bool valid)
{
    qDebug() << "Validation finished" << valid;
    if (!valid) {
        //setTransactionError(Shaman::TransactionInterface::UnknownError, "Erreur");//TODO: Proper error message and stuff
        setState(Shaman::Backend::Idling);
        return;
    }
    foreach (AkabeiClient::QueueItem * item, AkabeiClient::Backend::instance()->queue()->items()) {
        if (item->action() == AkabeiClient::Install) {
            foreach (Akabei::Package * dep, item->dependencyTree()) {
                m_packages[dep]->setAction(Shaman::AbstractPackage::TransactionDependencyInstall);//TODO: Or upgrade
            }
        } else if (item->action() == AkabeiClient::Remove) {
            foreach (Akabei::Package * dep, item->requiredByTree()) {
                m_packages[dep]->setAction(Shaman::AbstractPackage::TransactionDependencyUninstall);
            }
        }
        //TODO: Upgrade!
    }
    
    setState(Shaman::Backend::Idling);
}

void AkabeiBackend::phaseChanged(AkabeiClient::TransactionHandler::Phase phase)
{
    switch (phase) {
        //Validating
        case AkabeiClient::TransactionHandler::Downloading:
            setTransactionState(Shaman::TransactionInterface::Downloading);
            break;
        case AkabeiClient::TransactionHandler::RunningPreHooks:
            setTransactionState(Shaman::TransactionInterface::Processing);
            break;
        case AkabeiClient::TransactionHandler::Processing:
            setTransactionState(Shaman::TransactionInterface::Processing);
            break;
        case AkabeiClient::TransactionHandler::RunningScriptlets:
            setTransactionState(Shaman::TransactionInterface::Processing);
            break;
        case AkabeiClient::TransactionHandler::RunningPostHooks:
            setTransactionState(Shaman::TransactionInterface::Processing);//FIXME maybe this has to be Processing?
            break;
        case AkabeiClient::TransactionHandler::Cleaning:
            setTransactionState(Shaman::TransactionInterface::Cleaning);
            break;
        case AkabeiClient::TransactionHandler::Finished:
            setTransactionState(Shaman::TransactionInterface::NoTransaction);
            break;
        default:
            break;
    }
}

void AkabeiBackend::packageProgressChanged(Akabei::Package * pkg, int percent)
{
    qDebug() << "Package progress changed" << pkg->name() << percent;
    //"Split" it into phases TODO Do it in transactionahandler and do better calculation
    setPackageTransactionProgress(m_packages[pkg], percent / (int)AkabeiClient::TransactionHandler::Finished * (int) AkabeiClient::Backend::instance()->transactionHandler()->phase());
    if (AkabeiClient::Backend::instance()->transactionHandler()->phase() == AkabeiClient::TransactionHandler::Downloading)
        setPackageDownloadProgress(m_packages[pkg], pkg->size() / 100 * percent, pkg->size());
}

void AkabeiBackend::errorsOccurred(QList<Akabei::Error*> &errors)
{
    foreach (Akabei::Error * err, errors) {
        Shaman::TransactionInterface::Error error = Shaman::TransactionInterface::UnknownError;
        switch (err->type()) {
            case Akabei::Error::FilesystemConflictError:
                error = Shaman::TransactionInterface::FileConflictError;
                break;
            case Akabei::Error::PackageConflictError:
                error = Shaman::TransactionInterface::DependencyConflictError;
                break;
            case Akabei::Error::DuplicateTargetError:
                error = Shaman::TransactionInterface::FileConflictError;//TODO: Better error type
                break;
            case Akabei::Error::UnresolvableDependenciesError:
                error = Shaman::TransactionInterface::DependenciesUnsatisfiedError;
                break;
            case Akabei::Error::AkabeiInternalError:
                error = Shaman::TransactionInterface::BackendError;
                break;
            default:
                error = Shaman::TransactionInterface::UnknownError;
        };
        setTransactionError(error, err->description());
    }
}

bool AkabeiBackend::cancelRepositoriesUpdate()
{
    return false;
}

void AkabeiBackend::updateRepositories()
{
    foreach (AkabeiClient::DatabaseHandler * h, m_databases.keys())
        m_databases[h] = false;
    AkabeiClient::Backend::instance()->updateDatabase();
    AkabeiClient::Backend::instance()->localDatabase()->update();//Also update the local database, as Shaman thinks it's a real one... needs a fix... FIXME
    setTransactionState(Shaman::TransactionInterface::Processing);
}

void AkabeiBackend::databaseStatusChanged(AkabeiClient::DatabaseHandler::Status status)
{
    kDebug() << status;
    AkabeiClient::DatabaseHandler *h = qobject_cast<AkabeiClient::DatabaseHandler*>(QObject::sender());
    if (!h)
        return;
    
    Shaman::Repository * repo = 0;
    foreach (Shaman::Repository * r, backend()->repositories()) {
        if (r->name() == h->name() || (r->name() == i18n("Local") && h == AkabeiClient::Backend::instance()->localDatabase())) {
            repo = r;
            break;
        }
    }
    if (!repo)
        return;
    switch (status) {
        case AkabeiClient::DatabaseHandler::StatusBare:
            repo->setState(Shaman::Repository::Idling);
            break;
        case AkabeiClient::DatabaseHandler::StatusDownloading:
            repo->setState(Shaman::Repository::Downloading);
            break;
        case AkabeiClient::DatabaseHandler::StatusUpdating:
            repo->setState(Shaman::Repository::Updating);
            break;
        case AkabeiClient::DatabaseHandler::StatusFinished:
            m_databases[h] = true;
            repo->setState(Shaman::Repository::UpdatingFinished);
            break;
        case AkabeiClient::DatabaseHandler::StatusError:
            m_databases[h] = true;
            repo->setState(Shaman::Repository::Error);//TODO:Error handling/message
            break;
    }
    bool allTrue = true;
    foreach (bool b, m_databases.values()) {
        if (!b) {
            allTrue = false;
            break;
        }
    }
    if (allTrue) {
        kDebug() << "Done";
        setState(Shaman::Backend::Idling);
    } else {
        kDebug() << m_databases.key(false)->name();
    }
}

void AkabeiBackend::reload()
{
    kDebug();
    PERFORMANCE_MONITOR_START("Akabei Reload");
    Shaman::Repository::List repos;
    
    foreach (AkabeiClient::DatabaseHandler * db, AkabeiClient::Backend::instance()->databases()) {
        repos.append(new Shaman::Repository(db->name(), db->mirror(), this));
    }
    
    repos.append(new Shaman::Repository(i18n("Local"), KUrl(), this));

    // Create some demo stuff
    /*Shaman::BackendPackage *package = new AkabeiPackage(i18n("Hello Shaman"), this);
    Shaman::Repository *repo = new Shaman::Repository(i18n("Useless stuff"), KUrl(), this);
    Shaman::PackageGroup *group = new Shaman::PackageGroup(i18n("Group 1"), this);

    repo->append(package);
    group->append(package);

    repos.append(repo);
    groups.append(group);
    packages.append(package);

    package = new AkabeiPackage(i18n("Welcome to the fake backend"), this);
    repo = new Shaman::Repository(i18n("Less Useless stuff"), KUrl(), this);
    group = new Shaman::PackageGroup(i18n("Group 2"), this);

    repo->append(package);
    group->append(package);

    repos.append(repo);
    groups.append(group);
    packages.append(package);

    setPackages(packages);*/
    setRepositories(repos);
    //setPackageGroups(groups);
    
    connect(Akabei::Backend::instance(), SIGNAL(queryGroupCompleted(QUuid, QList<Akabei::Group*>)), SLOT(groupQueryComplete(QUuid, QList<Akabei::Group*>)));
    
    Akabei::Backend::instance()->groups();
}

void AkabeiBackend::groupQueryComplete(const QUuid &id, const QList<Akabei::Group*> &result)
{
    kDebug();
    disconnect(Akabei::Backend::instance(), SIGNAL(queryGroupCompleted(QUuid, QList<Akabei::Group*>)), this, SLOT(groupQueryComplete(QUuid, QList<Akabei::Group*>)));
    
    Shaman::PackageGroup::List groups;
    
    foreach (Akabei::Group * group, result) {
        Shaman::PackageGroup * g = new Shaman::PackageGroup(group->name(), this);
        if (!group->iconName().isEmpty())
            g->setIcon(KIcon(group->iconName()));
        groups.append(g);
    }
    
    setPackageGroups(groups);
    
    connect(Akabei::Backend::instance(), SIGNAL(queryPackagesCompleted(QUuid, QList<Akabei::Package*>)), SLOT(packageQueryComplete(QUuid, QList<Akabei::Package*>)));

    Akabei::Backend::instance()->packages();
}

void AkabeiBackend::packageQueryComplete(const QUuid &id, const QList<Akabei::Package*> &result)
{
    kDebug();
    m_packages.clear();
    disconnect(Akabei::Backend::instance(), SIGNAL(queryPackagesCompleted(QUuid, QList<Akabei::Package*>)), this, SLOT(packageQueryComplete(QUuid, QList<Akabei::Package*>)));
    Shaman::AbstractPackage::List packages;
    
    foreach (Akabei::Package *pkg, result) {
        AkabeiPackage * package = new AkabeiPackage(pkg, this);
        packages.append(package);
        m_packages[pkg] = package;
    }
    setPackages(packages);
    setState(Shaman::Backend::Idling);
    kDebug() << "Stop the monitor";
    PERFORMANCE_MONITOR_STOP("Akabei Reload");
}

Shaman::AbstractPackage::List AkabeiBackend::computeProviders(Akabei::Package *pkg)
{
    QStringList deps = pkg->provides();//FIXME: Check provides or rather providers of the package?
    Shaman::AbstractPackage::List returnList;
    foreach (const QString &package, deps) {
        foreach (Shaman::AbstractPackage * p, backend()->packages()) {
            if (package == p->name()) {
                returnList << p;
                break;
            }
        }
    }
    return returnList;
}

Shaman::AbstractPackage::List AkabeiBackend::computeDependencies(Akabei::Package *pkg)
{
    kDebug() << "Compute deps for" << pkg->name() << "got deps" << pkg->dependencies();
    //QStringList deps = pkg->dependencies();
    Shaman::AbstractPackage::List returnList;
    /*foreach (const QString &package, deps) {
        foreach (Shaman::AbstractPackage * p, backend()->packages()) {
            if (package == p->name()) {
                returnList << p;
                break;
            }
        }
    }*/
    Akabei::Package::List deps = pkg->computeDependencies();
    foreach (Akabei::Package * p, deps) {
        kDebug() << "Try" << p->name();
        returnList << m_packages[p];
    }
    return returnList;
}

Shaman::AbstractPackage::List AkabeiBackend::computeRequiredBy(Akabei::Package *pkg)
{ //FIXME: Cleanup!
    Akabei::Package::List deps = pkg->computeRequiredBy();
    Shaman::AbstractPackage::List returnList;
    QStringList handled;
    Akabei::Package::List installedPackages = Akabei::Backend::instance()->localDatabase()->packages();
    foreach (Akabei::Package * package, deps) {
        if (handled.contains(package->name()) && installedPackages.contains(package)) {
            foreach (Shaman::AbstractPackage * pkg, returnList) {
                if (pkg->name() == package->name())
                    returnList.removeAll(pkg);
            }
            returnList << m_packages[package];
        } else if (!handled.contains(package->name())) {
            returnList << m_packages[package];
            handled << package->name();
        }
    }
    return returnList;
}

Shaman::AbstractPackage::List AkabeiBackend::computeConflictsWith(Akabei::Package *pkg)
{
    QStringList deps = pkg->conflictsWith();
    Shaman::AbstractPackage::List returnList;
    foreach (Shaman::AbstractPackage * p, backend()->packages()) {
        foreach (const QString &package, deps) {
            if (package == p->name()) {
                returnList << p;
                break;
            }
        }
    }
    return returnList;
}

Shaman::Repository * AkabeiBackend::repositoryFromDatabase(Akabei::Database * db)
{
    AkabeiClient::DatabaseHandler *dbHandler = 0;
    foreach (AkabeiClient::DatabaseHandler * h, AkabeiClient::Backend::instance()->databases()) {
        if (db == h->database()) {
            //kDebug() << "Found repo" << h->name();
            dbHandler = h;
            break;
        }
    }
    QString name;
    if (dbHandler)
        name = dbHandler->name();
    //kDebug() << name;
    foreach (Shaman::Repository * repo, backend()->repositories()) {
        //kDebug() << repo->name();
        if (name == repo->name() || (db == Akabei::Backend::instance()->localDatabase() && repo->name() == i18n("Local")))
            return repo;
    }
    return 0;
}

Shaman::PackageGroup::List AkabeiBackend::groupsFromAkabeiGroups(QList<Akabei::Group*> groups)
{
    Shaman::PackageGroup::List returnList;
    foreach (Akabei::Group * group, groups) {
        foreach (Shaman::PackageGroup * g, backend()->packageGroups()) {
            if (g->name() == group->name()) {
                returnList << g;
                break;
            }
        }
    }
    return returnList;
}

Q_EXPORT_PLUGIN2(shaman_backend, AkabeiBackend)
