/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>
   Copyright (C) 2010 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef SHAMANAKABEIBACKEND_H
#define SHAMANAKABEIBACKEND_H

#include <shamanbackendinterface.h>
#include <shamanbackendpackage.h>
#include <akabeibackend.h>
#include <akabeidatabasehandler.h>
#include <akabeiclienttransactionhandler.h>

namespace Akabei {
    class Package;
    class Database;
}

class AkabeiBackend : public Shaman::BackendInterface
{
    Q_OBJECT
    Q_INTERFACES(Shaman::BackendInterface)

    public:
        AkabeiBackend();
        virtual ~AkabeiBackend();
        
        Shaman::AbstractPackage::List computeProviders(Akabei::Package *pkg);
        Shaman::AbstractPackage::List computeDependencies(Akabei::Package *pkg);
        Shaman::AbstractPackage::List computeRequiredBy(Akabei::Package *pkg);
        Shaman::AbstractPackage::List computeConflictsWith(Akabei::Package *pkg);
        Shaman::Repository * repositoryFromDatabase(Akabei::Database * db);
        Shaman::PackageGroup::List groupsFromAkabeiGroups(QList<Akabei::Group*> groups);
        
    protected:
        virtual void transactionQuestionAnswered(Shaman::TransactionQuestion* question);
        virtual void init();
    
    public slots:
        virtual bool aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate);
        virtual bool cancelQueueProcessing();
        virtual void processQueue(const Shaman::AbstractPackage::List& packages, const Shaman::ProcessQueueOption::List& options);
        virtual bool cancelRepositoriesUpdate();
        virtual void updateRepositories();
        virtual void reload();
        virtual void markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options);

    private slots:
        void backendStatusChanged(Akabei::Backend::Status status);
        void packageQueryComplete(const QUuid &id, const QList<Akabei::Package*> &result);
        void groupQueryComplete(const QUuid &id, const QList<Akabei::Group*> &result);
        void databaseStatusChanged(AkabeiClient::DatabaseHandler::Status status);
        void phaseChanged(AkabeiClient::TransactionHandler::Phase phase);
        void packageProgressChanged(Akabei::Package * pkg, int percent);
        void errorsOccurred(QList<Akabei::Error*> &errors);
        void validationFinished(bool valid);
        void transactionFinished();
        
    private:
        bool m_init;
        QHash<AkabeiClient::DatabaseHandler*, bool> m_databases;
        QHash<Akabei::Package *, Shaman::BackendPackage*> m_packages;
};

#endif // AKABEIBACKEND_H
