/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>
   Copyright (C) 2010 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "AkabeiPackage.h"
#include "AkabeiBackend.h"
#include <akabeipackage.h>
#include <akabeidatabase.h>
#include <shamanbackendinterface.h>
#include <KDebug>

AkabeiPackage::AkabeiPackage(Akabei::Package * pkg, AkabeiBackend* parent)
    : BackendPackage(pkg->name(), parent),
      m_backend(parent),
      m_pkg(pkg)
{
    setDescription(pkg->description());
    setVersion(QString(pkg->version().toByteArray()));

    //setAvailableVersion(const QString &version);

    setSize(pkg->installedSize());

    setMaintainer(pkg->packager());
    setHomepage(pkg->url());
    setLicense(pkg->license());

        /*void setProviders(const List &packages);

        void setDependencies(const List &packages);

        void setRequiredBy(const List &packages);

        void setConflictsWith(const List &packages);*/

    setFileList(pkg->retrieveFiles());
    setChangelog(pkg->retrieveChangelog());
    setDownloadSize(pkg->size());
    if (Akabei::Backend::instance()->localDatabase()->packages().contains(pkg)) {
        setState(Shaman::AbstractPackage::Installed);
    } else {
        Akabei::Package::List result = Akabei::Backend::instance()->localDatabase()->queryPackages("SELECT * FROM packages WHERE Name LIKE \"" + pkg->name() + "\" ");
        if (!result.isEmpty() && result.first()->version() < pkg->version())
            setState(Shaman::AbstractPackage::Upgradeable);
        else
            setState(Shaman::AbstractPackage::NotInstalled);//TODO: Optimize!!!
    }

    Shaman::Repository * repo = m_backend->repositoryFromDatabase(pkg->database());
    if (repo)
        repo->append(this);
    
    foreach (Shaman::PackageGroup * group, m_backend->groupsFromAkabeiGroups(pkg->groups()))
        group->append(this);
    
    setUnCachedValues(Shaman::AbstractPackage::Providers | Shaman::AbstractPackage::Dependencies | Shaman::AbstractPackage::RequiredBy | Shaman::AbstractPackage::ConflictsWith);
}

AkabeiPackage::~AkabeiPackage()
{

}

void AkabeiPackage::cacheValue(Shaman::BackendPackage::CachedValue value)
{
    switch (value) {
        case Shaman::AbstractPackage::Providers:
            setProviders(m_backend->computeProviders(m_pkg));
            break;
        case Shaman::AbstractPackage::Dependencies:
            setDependencies(m_backend->computeDependencies(m_pkg));
            break;
        case Shaman::AbstractPackage::RequiredBy:
            setRequiredBy(m_backend->computeRequiredBy(m_pkg));
            break;
        case Shaman::AbstractPackage::ConflictsWith:
            setConflictsWith(m_backend->computeConflictsWith(m_pkg));
            break;
        default:
            break;
    }
}

