/* This file is part of the KDE project

   Copyright © 2010 David Palacio <dpalacio@uninorte.edu.co>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef APTBACKEND_H
#define APTBACKEND_H

#include <apt-pkg/pkgcache.h>

#include <kauth.h>
#include <kauthactionreply.h>

#include <shamanbackendinterface.h>
#include <shamanbackendpackage.h>

class pkgSourceList;
//class pkgCache;
class pkgPolicy;

class DebPackage;
class OperationProgress;

using namespace KAuth;

class AptBackend : public Shaman::BackendInterface
{
    Q_OBJECT
    Q_INTERFACES(Shaman::BackendInterface)
    
    enum Options
    {
        FollowRecommends = 1,
        FollowSuggests = 2,
        Purge = 4
    };
    
    public:
        AptBackend();
        ~AptBackend();
    protected:
        void transactionQuestionAnswered(Shaman::TransactionQuestion* question);
        void init();

    public:
        virtual bool aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate);
        bool cancelQueueProcessing();
        bool cancelRepositoriesUpdate();
        void updateRepositories();
        void markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options);
        void processQueue(const Shaman::AbstractPackage::List &packages, const Shaman::ProcessQueueOption::List &options);
        DebPackage *findPackageByVersion(const pkgCache::VerIterator &version);

    protected slots:
        void slotUpdateFinished();
        void slotProcessRequest(const QVariantMap&);
        void slotProcessFinished(ActionReply reply);
        void slotOperationReply(ActionReply reply);
        void reloadingSources(OperationProgress *op);
        void reload();
        
    private:
        QString mHelperID;
        pkgSourceList *mList;
        pkgCache *mCache;
        pkgPolicy *mPolicy;
};

#endif // APTBACKEND_H
