/* This file is part of the KDE project

   Copyright © 2010 David Palacio <dpalacio@uninorte.edu.co>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "DebPackage.h"

#include <QtCore/QFile>

#include <apt-pkg/pkgcache.h>

#include <kdebug.h>

#include <shamanbackendinterface.h>

DebPackage::DebPackage(Shaman::BackendInterface* parent, const QString &name,
                       const pkgCache::VerIterator &version,
                       const pkgCache::VerIterator &cndVer)
    : BackendPackage(name, parent), version(version)
{
    pkgCache::PkgIterator pkgIter = version.ParentPkg();
    pkgCache::Package *package = (pkgCache::Package*)pkgIter;
    //versión antigua que funciona con un PkgIterator como argumento
    //State state = (*package).CurrentState & pkgCache::State::Installed ?
    //              Installed : NotInstalled;
    if(package->CurrentState & pkgCache::State::Installed) //if installed
    {
      if(pkgIter.CurrentVer() == version)
      {
        setState(Installed);
      }
      else if(cndVer == version)
      {
        setState(Upgradeable);
      }
      else
      {
        setState(NotInstalled);
      }
    }
    else
    {
      setState(NotInstalled);
    }
    setIsCandidate(cndVer == version);
    setVersion(version.VerStr());
    //setAvailableVersion("42");

    setUnCachedValues(/*Homepage | License | Maintainer | Size | Providers |*/
                      /*RequiredBy | Dependencies | ConflictsWith |*/ FileList /*| Changelog | CustomValues*/);

    QString packageName = QString(pkgIter.Name());
    if (packageName.endsWith(QLatin1String("-dbg")) || packageName.endsWith(QLatin1String("-dbgsym"))) {
        // It's a debug package!
        setProperties(DebugInformationProperty);
    }
}

DebPackage::~DebPackage()
{
}

void DebPackage::cacheValue(Shaman::BackendPackage::CachedValue value)
{
    Shaman::AbstractPackage::List retlist;
    switch (value) {
        case FileList:
            QStringList fileList;
            QFile infoFile("/var/lib/dpkg/info/" + name() + ".list");

            if (infoFile.open(QFile::ReadOnly)) {
                QTextStream stream(&infoFile);
                QString line;

                do {
                    line = stream.readLine();
                    fileList << line;
                } while (!line.isNull());

                // The first item won't be a file
                fileList.removeFirst();
            }

            setFileList(fileList);
            break;
    }

}

void DebPackage::setIsCandidate(bool is)
{
    mIsCandidate = is;
}

bool DebPackage::isCandidate() const
{
    return mIsCandidate;
}
