/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "AqpmLocalPackage.h"
#include <aqpm/Backend.h>
#include "AqpmPlugin.h"

AqpmLocalPackage::AqpmLocalPackage(const KUrl& path, Aqpm::Package *package, AqpmPlugin* parent)
        : LocalPackage(package->name(), path, parent)
        , m_package(package)
        , m_plugin(parent)
{
    setVersion(package->version());
    setDescription(m_package->desc());
    setState(NotInstalled);
    setSize(m_package->size());
    setUnCachedValues(Homepage | /*License |*/ Maintainer | Size | Providers |
                      RequiredBy | Dependencies | /*ConflictsWith |*/ FileList | Changelog);
}

AqpmLocalPackage::~AqpmLocalPackage()
{

}

void AqpmLocalPackage::cacheValue(Shaman::AbstractPackage::CachedValue value)
{
    Shaman::AbstractPackage::List retlist;
    switch (value) {
        case Homepage:
            setHomepage(m_package->url());
            break;
        case License:
            //setLicense() TBD in Aqpm
            break;
        case Maintainer:
            setMaintainer(m_package->packager());
            break;
        case Size:
            setSize(m_package->size());
            break;
        case Providers:
            foreach (const QString &provider, m_package->providers()) {
                retlist.append(m_plugin->findPackageByName(provider));
            }
            setProviders(retlist);
            break;
        case RequiredBy:
            foreach (Aqpm::Package *pkg, m_package->requiredBy()) {
                retlist.append(m_plugin->findPackageByName(pkg->name()));
            }
            setRequiredBy(retlist);
            break;
        case Dependencies:
            foreach (Aqpm::Package *pkg, m_package->dependsOn()) {
                retlist.append(m_plugin->findPackageByName(pkg->name()));
            }
            setDependencies(retlist);
            break;
        case ConflictsWith:
            break;
        case FileList:
            setFileList(m_package->files());
            break;
        case Changelog:
            setChangelog(m_package->retrieveChangelog());
            break;
        default:
            break;
    }
}

