/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "AqpmPackage.h"
#include <aqpm/Backend.h>
#include "AqpmPlugin.h"
#include "shamanwidgetfactory.h"

#include <KDebug>
#include <KTextBrowser>

using namespace Aqpm;

class LoggedActionsFactory : public Shaman::WidgetFactory
{
    public:
        LoggedActionsFactory(const QString &log) : m_log(log) {}
        ~LoggedActionsFactory() {}

        virtual QWidget* createInstance() const {
            KTextBrowser *tb = new KTextBrowser();
            tb->setText(m_log);
            return tb;
        };

    private:
        QString m_log;
};

AqpmPackage::AqpmPackage(Aqpm::Package *package, const State &state, AqpmPlugin* parent)
        : BackendPackage(package->name(), parent)
        , m_package(package)
        , m_plugin(parent)
{
    setVersion(package->version());
    setDescription(m_package->desc());
    setState(state);
    setDownloadSize(m_package->size());
    setUnCachedValues(Homepage | /*License |*/ Maintainer | Size | Providers |
                      RequiredBy | Dependencies | /*ConflictsWith |*/ FileList | Changelog | CustomValues);
    if (name().endsWith(QLatin1String("-debug"))) {
        // It's a debug package!
        setProperties(DebugInformationProperty);
    }
}

void AqpmPackage::cacheValue(Shaman::BackendPackage::CachedValue value)
{
    Shaman::AbstractPackage::List retlist;
    switch (value) {
        case Homepage:
            setHomepage(m_package->url());
            break;
        case License:
            //setLicense() TBD in Aqpm
            break;
        case Maintainer:
            setMaintainer(m_package->packager());
            break;
        case Size:
            setSize(m_package->size());
            break;
        case Providers:
            foreach (const QString &provider, m_package->providers()) {
                retlist.append(m_plugin->findPackageByName(provider));
            }
            setProviders(retlist);
            break;
        case RequiredBy:
            foreach (Aqpm::Package *pkg, m_package->requiredBy()) {
                retlist.append(m_plugin->findPackageByName(pkg->name()));
            }
            setRequiredBy(retlist);
            break;
        case Dependencies:
            foreach (Aqpm::Package *pkg, m_package->dependsOn()) {
                retlist.append(m_plugin->findPackageByName(pkg->name()));
            }
            setDependencies(retlist);
            break;
        case ConflictsWith:
            break;
        case FileList:
            setFileList(m_package->files());
            break;
        case Changelog:
            setChangelog(m_package->retrieveChangelog());
            break;
        case CustomValues:
            if (!m_package->retrieveLoggedActions().isEmpty()) {
                addCustomValue(i18n("Logged actions"), new LoggedActionsFactory(m_package->retrieveLoggedActions()));
            }
            break;
        default:
            break;
    }
}

