/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef AQPMPLUGIN_H
#define AQPMPLUGIN_H

#include "shamanbackendinterface.h"

#include <aqpm/Globals.h>
#include <aqpm/Package.h>
#include <aqpm/QueueItem.h>
#include <aqpm/Backend.h>

class AqpmPackage;
class KdemodDistUpgrade;


class AqpmPlugin : public Shaman::BackendInterface
{
    Q_OBJECT
    Q_INTERFACES(Shaman::BackendInterface)

    public:
        AqpmPlugin();
        virtual ~AqpmPlugin();

        void init();

    public slots:
        void reload();
        virtual bool aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate);

        void updateRepositories();

        bool cancelRepositoriesUpdate();

        void processQueue(const Shaman::AbstractPackage::List & packages, const Shaman::ProcessQueueOption::List &options);

        bool cancelQueueProcessing();

        void markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options);
        void slotAdditionalTargetsRetrieved(const Aqpm::Targets &targets);

        void transactionQuestionAnswered(Shaman::TransactionQuestion *question);

        virtual Shaman::AbstractPackage* packageFromFile(const KUrl& path);

    private slots:
        void initAlpm();
        /* Handlers from aqpm */
        void slotDbStatusChanged(const QString &repo, int action);

        void slotStreamDlProg(const QString &c, int bytedone, int bytetotal, int speed,
                              int listdone, int listtotal);

        void slotStreamTransProgress(Aqpm::Globals::TransactionProgress event, const QString &pkgname, int percent,
                                     int howmany, int remain);

        void slotStreamTransEvent(Aqpm::Globals::TransactionEvent event, const QVariantMap &args);

        void slotStreamTransQuestion(Aqpm::Globals::TransactionQuestion event, const QVariantMap &args);

        void slotErrorOccurred(Aqpm::Globals::Errors code, const QVariantMap &args);

        void slotLogMessageStreamed(const QString &msg);

        void slotOperationFinished(bool success);

        void setIdle();

    private:
        void populateCacheForPackageName(const QString &name);
        Shaman::AbstractPackage *findPackageByName(const QString &name);
        AqpmPackage *createShamanPackageFrom(Aqpm::Package *package, const Aqpm::Package::List &upgradeable);
        Shaman::TransactionInterface::State m_cachedState;
        QHash<QString, Shaman::AbstractPackage*> m_packageNameCache;
        int m_dbupdateProgress;
        int m_dbupdateWeight;
        QPointer<KdemodDistUpgrade> m_runningDistUpgrade;

        friend class AqpmPackage;
        friend class AqpmLocalPackage;

};

#endif

