/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "FakeBackend.h"

#include <kicon.h>

#include "FakePackage.h"
#include <shamanbackendpackage.h>
#include <QTimer>


FakeBackend::FakeBackend()
    : Shaman::BackendInterface(0)
{

}

FakeBackend::~FakeBackend()
{

}

bool FakeBackend::aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate)
{
    if ((oldstate == Shaman::Backend::Initializing && newstate == Shaman::Backend::Idling) ||
        (oldstate == Shaman::Backend::OnTransaction && newstate == Shaman::Backend::Idling)) {
        // Time to reload, guys.
        QTimer::singleShot(0, this, SLOT(reload()));
        // If we're initializing, just don't reject the state transition
        if (oldstate != Shaman::Backend::Initializing) {
            // Otherwise do.
            return false;
        }
    }
    return Shaman::BackendInterface::aboutToChangeState(oldstate, newstate);
}

void FakeBackend::transactionQuestionAnswered(Shaman::TransactionQuestion* question)
{

}

void FakeBackend::init()
{
    setCapabilities(/*Transaction capabilities*/
                    Shaman::BackendCapabilities::UpdateRepositoriesCapability,
                    /*Search capabilities*/
                    Shaman::BackendCapabilities::BasicSearchFromViewCapability,
                    /*Package Grouping capabilities*/
                    Shaman::BackendCapabilities::GroupPackagesByPackageGroupsCapability |
                    Shaman::BackendCapabilities::GroupPackagesByRepositoryCapability,
                    /*Information capabilities*/
                    0);

    KAboutPerson dario(ki18n("Dario Freddi"), ki18n("Maintainer"), "drf@kde.org", "http://drfav.wordpress.com");
    // TODO: Change package icon
    setProperties(i18n("Fake backend (not working)"), i18n("A backend that does nothing"), KIcon("applications-other"),
                  KIcon("application-x-deb"), QList<KAboutPerson>() << dario, QStringList());

    setState(Shaman::Backend::Idling);
}

bool FakeBackend::cancelQueueProcessing()
{
    return false;
}

void FakeBackend::processQueue(const Shaman::AbstractPackage::List& packages, const Shaman::ProcessQueueOption::List &options)
{
 // Do nothing
}

bool FakeBackend::cancelRepositoriesUpdate()
{
    return false;
}

void FakeBackend::updateRepositories()
{

}

void FakeBackend::reload()
{
    Shaman::Repository::List repos;
    Shaman::AbstractPackage::List packages;
    Shaman::PackageGroup::List groups;

    // Create some demo stuff
    Shaman::BackendPackage *package = new FakePackage(i18n("Hello Shaman"), this);
    Shaman::Repository *repo = new Shaman::Repository(i18n("Useless stuff"), KUrl(), this);
    Shaman::PackageGroup *group = new Shaman::PackageGroup(i18n("Group 1"), this);

    repo->append(package);
    group->append(package);

    repos.append(repo);
    groups.append(group);
    packages.append(package);

    package = new FakePackage(i18n("Welcome to the fake backend"), this);
    repo = new Shaman::Repository(i18n("Less Useless stuff"), KUrl(), this);
    group = new Shaman::PackageGroup(i18n("Group 2"), this);

    repo->append(package);
    group->append(package);

    repos.append(repo);
    groups.append(group);
    packages.append(package);

    setPackages(packages);
    setRepositories(repos);
    setPackageGroups(groups);

    setState(Shaman::Backend::Idling);
}

void FakeBackend::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
 // Do nothing
}

Q_EXPORT_PLUGIN2(shaman_backend, FakeBackend)
