/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PKPackage.h"
#include "PKPlugin.h"
#include "PKPackageListener.h"

using namespace Shaman;

PKPackage::PKPackage(PackageKit::Package * package, PKPlugin * parent)
  : BackendPackage(package->name(), QString(), package->version(), BackendPackage::NotInstalled, parent),
    m_package(package)
{
    if (package->state() == PackageKit::Package::StateInstalled) {
        setState(BackendPackage::Installed);
    }
    setUnCachedValues(Description | Homepage | License | Size | cachedValueForAdditionalCacheValue(DownloadSize) | Providers |
                      RequiredBy | Dependencies | ConflictsWith | FileList);
}

PKPackage::~PKPackage()
{
    m_package->deleteLater();
}

PackageKit::Package * PKPackage::package()
{
    return m_package;
}

void PKPackage::cacheValue(Shaman::AbstractPackage::CachedValue value)
{
    PackageKit::Transaction * transaction;
    switch (value) {
        case Description:
        case Homepage:
        case License:
        case Size:
            // Hack because C++ and switch suck balls.
        case AdditionalValue1:
            if (!m_package->hasDetails()) {
                PKPackageListener::instance()->retrieveDetails(this);
            } else {
                slotDetails();
            }
            break;
        case Providers:
            setProviders(AbstractPackage::List());
            break;
        case RequiredBy:
            PKPackageListener::instance()->retrieveRequiredBy(this);
            break;
        case Dependencies:
            PKPackageListener::instance()->retrieveDependencies(this);
            break;
        case ConflictsWith:
            setConflictsWith(AbstractPackage::List());
            break;
        case FileList:
            PKPackageListener::instance()->retrieveFiles(this);
            break;
        default:
            break;
    }
}

void PKPackage::slotDetails()
{
    if (m_package->hasDetails()) {
        setDescription(m_package->details()->description());
        setHomepage(m_package->details()->license());
        setSize(m_package->details()->size());
        setDownloadSize(m_package->details()->size());
        setLicense(m_package->details()->license());
    } else {
        // Something went wrong, let's try relaunching the query
        cacheValue(Description);
    }
}
