/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "PKPackageListener.h"
#include "PKPackage.h"
#include <transaction.h>
#include "PKPlugin.h"

class PKPackageListenerHelper
{
  public:
    PKPackageListenerHelper() : q(0) {}
    ~PKPackageListenerHelper() { delete q; }
    PKPackageListener *q;
};

K_GLOBAL_STATIC(PKPackageListenerHelper, s_globalPKPackageListener)

PKPackageListener *PKPackageListener::instance()
{
    if (!s_globalPKPackageListener->q) {
        new PKPackageListener;
    }

    return s_globalPKPackageListener->q;
}

PKPackageListener::PKPackageListener(QObject * parent)
  : QObject(parent)
{
    Q_ASSERT(!s_globalPKPackageListener->q);
    s_globalPKPackageListener->q = this;
}

PKPackageListener::~PKPackageListener()
{

}

void PKPackageListener::setPlugin(PKPlugin* plugin)
{
    m_plugin = plugin;
}

void PKPackageListener::files(PackageKit::Package * package, const QStringList& files)
{
    if (package->id() == m_transactionToPackage[sender()]->package()->id()) {
        m_transactionToPackage[sender()]->setFileList(files);
    }
}

void PKPackageListener::newPackage(PackageKit::Package* p)
{
    m_transactionToList[sender()].append(p);
}

void PKPackageListener::retrieveDependencies(PKPackage* package)
{
    PackageKit::Transaction *transaction = PackageKit::Client::instance()->getDepends(package->package(),
                                                                                     PackageKit::Client::NoFilter, false);
    connect(transaction, SIGNAL(package(PackageKit::Package*)), SLOT(newPackage(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus, uint)),
            this, SLOT(slotDependsOn()));
}

void PKPackageListener::retrieveDetails(PKPackage* package)
{
    PackageKit::Transaction *transaction = PackageKit::Client::instance()->getDetails(package->package());
    m_transactionToPackage[transaction] = package;
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus, uint)), this, SLOT(slotDetails()));
}

void PKPackageListener::retrieveFiles(PKPackage* package)
{
    PackageKit::Transaction *transaction = PackageKit::Client::instance()->getFiles(package->package());
    m_transactionToPackage[transaction] = package;
    connect(transaction, SIGNAL(files(PackageKit::Package*,QStringList)),
            this, SLOT(files(PackageKit::Package*,QStringList)));
}

void PKPackageListener::retrieveRequiredBy(PKPackage* package)
{
    PackageKit::Transaction *transaction = PackageKit::Client::instance()->getRequires(package->package(),
                                                                                      PackageKit::Client::NoFilter, false);
    m_transactionToPackage[transaction] = package;
    connect(transaction, SIGNAL(package(PackageKit::Package*)), SLOT(newPackage(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus, uint)), this, SLOT(slotRequiredBy()));
}

void PKPackageListener::slotDependsOn()
{
    m_transactionToPackage[sender()]->setDependencies(m_plugin->pkPackageListToList(m_transactionToList[sender()]));
}

void PKPackageListener::slotDetails()
{
    m_transactionToPackage[sender()]->slotDetails();
}

void PKPackageListener::slotRequiredBy()
{
    m_transactionToPackage[sender()]->setRequiredBy(m_plugin->pkPackageListToList(m_transactionToList[sender()]));
}
