/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef PKPLUGIN_H
#define PKPLUGIN_H

#include <shamanbackendinterface.h>

#include <QVariantList>

#include <PackageKit/packagekit-qt/client.h>
#include <PackageKit/packagekit-qt/package.h>
#include <PackageKit/packagekit-qt/transaction.h>
#include <KIcon>
#include <shamanwidgetfactory.h>

using namespace Shaman;
typedef PackageKit::Package Package;

class EulaWidgetFactory : public Shaman::WidgetFactory
{
    public:
        EulaWidgetFactory(const QString &header, const QString &eula) : WidgetFactory(), m_header(header), m_eula(eula) {}
        virtual ~EulaWidgetFactory() {}

        virtual QWidget* createInstance() const;

    private:
        QString m_header;
        QString m_eula;
};

class PKPlugin : public BackendInterface
{
    Q_OBJECT
    Q_INTERFACES(Shaman::BackendInterface)

    public:
        PKPlugin();
        ~PKPlugin();

        virtual void init();

    public slots:
        virtual void markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options);

        virtual bool aboutToChangeState(Backend::State oldstate, Backend::State newstate);

        virtual void reload();

        virtual void partialReload(const Shaman::AbstractPackage::List& packages);

        virtual void updateRepositories();

        virtual bool cancelRepositoriesUpdate();

        virtual void processQueue(const Shaman::AbstractPackage::List & packages,
                                  const Shaman::ProcessQueueOption::List &options);

        virtual void searchPackages(Shaman::Query *query);

        virtual bool cancelQueueProcessing();
        void slotCurrentPackageChanged(PackageKit::Package*);
        void errorDetected(PackageKit::Client::DaemonError);
        void addPackageToGroup(PackageKit::Package*);
        Shaman::AbstractPackage::List pkPackageListToList(QList<PackageKit::Package*> packages);
        void reportUpdateDatabaseProgress(PackageKit::Transaction::ProgressInfo);
        void reloadPackage(PackageKit::Package*);
        void setUpgradeable(PackageKit::Package*);
        void setIdle();
        void slotSearchFinished();
        void backgroundOperationProgress(PackageKit::Transaction::ProgressInfo info);
        void groupLoadingProgress(PackageKit::Transaction::ProgressInfo);
        void slotRequireRestart(PackageKit::Client::RestartType,Package*);
        void slotEulaRequired(PackageKit::Client::EulaInfo);
        void slotMediaChangeRequired(PackageKit::Transaction::MediaType,const QString&,const QString&);
        void slotMessage(PackageKit::Client::MessageType,const QString&);
        void slotErrorCode(PackageKit::Client::ErrorType,const QString&);
        void slotStateChanged(Shaman::Backend::State state);
        void simulateProgressChanged(PackageKit::Transaction::ProgressInfo);

    private slots:
        void reloadRepositories();
        void reloadPackageGroups();
        void reloadPackages();

        void addRepo(const QString &id, const QString &description, bool enabled);

        void addPackage(PackageKit::Package* p);
        void addSearchPackage(PackageKit::Package* p);
        void simulationPackage(PackageKit::Package* p);
        void reloadNextPackageGroup();

        void setUpdateDatabaseFinished();

        void simulateInstall();
        void simulateUpgrade();
        void simulateRemove();

        void installPackages();
        void upgradePackages();
        void removePackages();

        void reportProgress(PackageKit::Transaction::ProgressInfo info);
        void reportStatus(PackageKit::Transaction::Status status);

        void stopBackgroundTransactions();

    protected:
        virtual void transactionQuestionAnswered(Shaman::TransactionQuestion *question);

    private:
        void checkAndMarkUpdates();
        void waitForTransactionFinished(PackageKit::Transaction *transaction);
        void connectTransaction(PackageKit::Transaction *transaction);

        class Private;
        Private * d;
};

#endif // PKPLUGIN_H
