/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef PKSTRINGSANDICONS_H
#define PKSTRINGSANDICONS_H
#include <QPair>
#include <KIcon>
#include <PackageKit/packagekit-qt/client.h>
#include <klocalizedstring.h>
#include <KDebug>
#include <PackageKit/packagekit-qt/transaction.h>
#include <shamantransactioninterface.h>

namespace PKStringsAndIcons
{
    QPair<QString, KIcon> groupName(PackageKit::Client::Group group)
    {
        switch (group) {
            case PackageKit::Client::GroupAccessibility :
            return qMakePair(i18n("Accessibility"), KIcon("preferences-desktop-accessibility"));
            case PackageKit::Client::GroupAccessories :
            return qMakePair(i18n("Accessories"), KIcon("accessories-text-editor"));
            case PackageKit::Client::GroupAdminTools :
            return qMakePair(i18n("Admin tools"), KIcon("preferences-desktop"));
            case PackageKit::Client::GroupCommunication :
            return qMakePair(i18n("Communication"), KIcon("mail-message"));
            case PackageKit::Client::GroupDesktopGnome :
            return qMakePair(i18n("GNOME desktop"), KIcon("preferences-desktop-theme"));
            case PackageKit::Client::GroupDesktopKde :
            return qMakePair(i18n("KDE desktop"), KIcon("kde"));
            case PackageKit::Client::GroupDesktopOther :
            return qMakePair(i18n("Other desktops"), KIcon("preferences-desktop-theme"));
            case PackageKit::Client::GroupDesktopXfce :
            return qMakePair(i18n("XFCE desktop"), KIcon("preferences-desktop-theme"));
            case PackageKit::Client::GroupEducation :
            return qMakePair(i18n("Education"), KIcon("applications-education"));
            case PackageKit::Client::GroupFonts :
            return qMakePair(i18n("Fonts"), KIcon("application-x-font-ttf"));
            case PackageKit::Client::GroupGames :
            return qMakePair(i18n("Games"), KIcon("applications-games"));
            case PackageKit::Client::GroupGraphics :
            return qMakePair(i18n("Graphics"), KIcon("applications-graphics"));
            case PackageKit::Client::GroupInternet :
            return qMakePair(i18n("Internet"), KIcon("applications-internet"));
            case PackageKit::Client::GroupLegacy :
            return qMakePair(i18n("Legacy"), KIcon(""));
            case PackageKit::Client::GroupLocalization :
            return qMakePair(i18n("Localization"), KIcon("preferences-desktop-locale"));
            case PackageKit::Client::GroupMaps :
            return qMakePair(i18n("Maps"), KIcon(""));
            case PackageKit::Client::GroupMultimedia :
            return qMakePair(i18n("Multimedia"), KIcon("applications-multimedia"));
            case PackageKit::Client::GroupNetwork :
            return qMakePair(i18n("Network"), KIcon("network-wired"));
            case PackageKit::Client::GroupOffice :
            return qMakePair(i18n("Office"), KIcon("applications-office"));
            case PackageKit::Client::GroupOther :
            return qMakePair(i18n("Other"), KIcon("applications-other"));
            case PackageKit::Client::GroupPowerManagement :
            return qMakePair(i18n("Power management"), KIcon("preferences-system-power-management"));
            case PackageKit::Client::GroupProgramming :
            return qMakePair(i18n("Development"), KIcon("applications-development"));
            case PackageKit::Client::GroupPublishing :
            return qMakePair(i18n("Publishing"), KIcon("view-presentation"));
            case PackageKit::Client::GroupRepos :
            return qMakePair(i18n("Software sources"), KIcon("run-build-configure"));
            case PackageKit::Client::GroupScience :
            return qMakePair(i18n("Science"), KIcon("applications-science"));
            case PackageKit::Client::GroupDocumentation :
            return qMakePair(i18n("Documentation"), KIcon("documentation"));
            case PackageKit::Client::GroupElectronics :
            return qMakePair(i18n("Electronics"), KIcon(""));
            //case PackageKit::Client::MetaPackages ://TODO check this one
        //    return i18n("Package collections");
            case PackageKit::Client::GroupSecurity :
            return qMakePair(i18n("Security"), KIcon("security-high"));
            case PackageKit::Client::GroupServers :
            return qMakePair(i18n("Servers"), KIcon("network-server"));
            case PackageKit::Client::GroupSystem :
            return qMakePair(i18n("System"), KIcon("applications-system"));
            case PackageKit::Client::GroupVirtualization :
            return qMakePair(i18n("Virtualization"), KIcon(""));
            case PackageKit::Client::UnknownGroup :
            return qMakePair(i18n("Unknown group"), KIcon("unknown"));
        default :
            kDebug() << "group unrecognised: " << group;
            return qMakePair(QString(), KIcon());
        }
    }

    QString stringForStatus(PackageKit::Transaction::Status status)
    {
        QString msg;
        // Status message
        switch (status) {
        case PackageKit::Transaction::StatusSetup :
            msg = i18nc("transaction state, the daemon is in the process of starting",
                        "Waiting for service to start");
            break;
        case PackageKit::Transaction::StatusWait :
            msg = i18nc("transaction state, the transaction is waiting for another to complete",
                        "Waiting for other tasks");
            break;
        case PackageKit::Transaction::StatusRunning :
            msg = i18nc("transaction state, just started",
                        "Running task");
            break;
        case PackageKit::Transaction::StatusQuery :
            msg = i18nc("transaction state, is querying data",
                        "Querying backend");
            break;
        case PackageKit::Transaction::StatusInfo :
            msg = i18nc("transaction state, getting data from a server",
                        "Getting information");
            break;
        case PackageKit::Transaction::StatusRemove :
            msg = i18nc("transaction state, removing packages",
                        "Removing packages");
            break;
        case PackageKit::Transaction::StatusDownload :
            msg = i18nc("transaction state, downloading package files",
                        "Downloading packages");
            break;
        case PackageKit::Transaction::StatusInstall :
            msg = i18nc("transaction state, installing packages",
                        "Installing packages");
            break;
        case PackageKit::Transaction::StatusRefreshCache :
            msg = i18nc("transaction state, refreshing internal lists",
                        "Refreshing software list");
            break;
        case PackageKit::Transaction::StatusUpdate :
            msg = i18nc("transaction state, installing updates",
                        "Updating packages");
            break;
        case PackageKit::Transaction::StatusCleanup :
            msg = i18nc("transaction state, removing old packages, and cleaning config files",
                        "Cleaning up packages");
            break;
        case PackageKit::Transaction::StatusObsolete :
            msg = i18nc("transaction state, obsoleting old packages",
                        "Obsoleting packages");
            break;
        case PackageKit::Transaction::StatusDepResolve :
            msg = i18nc("transaction state, checking the transaction before we do it",
                        "Resolving dependencies");
            break;
        case PackageKit::Transaction::StatusSigCheck :
            msg = i18nc("transaction state, checking if we have all the security keys for the operation",
                        "Checking signatures");
            break;
        case PackageKit::Transaction::StatusRollback :
            msg = i18nc("transaction state, when we msg = to a previous system state",
                        "Rolling back");
            break;
        case PackageKit::Transaction::StatusTestCommit :
            msg = i18nc("transaction state, when we're doing a test transaction",
                        "Testing changes");
            break;
        case PackageKit::Transaction::StatusCommit :
            msg = i18nc("transaction state, when we're writing to the system package database",
                        "Committing changes");
            break;
        case PackageKit::Transaction::StatusRequest :
            msg = i18nc("transaction state, requesting data from a server",
                        "Requesting data");
            break;
        case PackageKit::Transaction::StatusCancel :
            msg = i18nc("transaction state, in the process of cancelling",
                        "Cancelling");
            break;
        case PackageKit::Transaction::StatusDownloadRepository :
            msg = i18nc("transaction state, downloading metadata",
                        "Downloading repository information");
            break;
        case PackageKit::Transaction::StatusDownloadPackagelist :
            msg = i18nc("transaction state, downloading metadata",
                        "Downloading list of packages");
            break;
        case PackageKit::Transaction::StatusDownloadFilelist :
            msg = i18nc("transaction state, downloading metadata",
                        "Downloading file lists");
            break;
        case PackageKit::Transaction::StatusDownloadChangelog :
            msg = i18nc("transaction state, downloading metadata",
                        "Downloading lists of changes");
            break;
        case PackageKit::Transaction::StatusDownloadGroup :
            msg = i18nc("transaction state, downloading metadata",
                        "Downloading groups");
            break;
        case PackageKit::Transaction::StatusDownloadUpdateinfo :
            msg = i18nc("transaction state, downloading metadata",
                        "Downloading update information");
            break;
        case PackageKit::Transaction::StatusRepackaging :
            msg = i18nc("transaction state, repackaging delta files",
                        "Repackaging files");
            break;
        case PackageKit::Transaction::StatusLoadingCache :
            msg = i18nc("transaction state, loading databases",
                        "Loading cache");
            break;
        case PackageKit::Transaction::StatusScanApplications :
            msg = i18nc("transaction state, scanning for running processes",
                        "Scanning installed applications");
            break;
        case PackageKit::Transaction::StatusGeneratePackageList :
            msg = i18nc("transaction state, generating a list of packages installed on the system",
                        "Generating package lists");
            break;
        case PackageKit::Transaction::StatusWaitingForLock :
            msg = i18nc("transaction state, when we're waiting for the native tools to exit",
                        "Waiting for package manager lock");
            break;
        case PackageKit::Transaction::StatusWaitingForAuth :
            msg = i18nc("waiting for user to type in a password",
                        "Waiting for authentication");
            break;
        case PackageKit::Transaction::StatusScanProcessList :
            msg = i18nc("we are updating the list of processes",
                        "Updating the list of running applications");
            break;
        case PackageKit::Transaction::StatusCheckExecutableFiles :
            msg = i18nc("we are checking executable files in use",
                        "Checking for applications currently in use");
            break;
        case PackageKit::Transaction::StatusCheckLibraries :
            msg = i18nc("we are checking for libraries in use",
                        "Checking for libraries currently in use");
            break;
    /*    case PackageKit::Transaction::StatusCopyFiles :
            msg = i18nc("we are copying package files to prepare to install",
                        "Copying files");
            break;*/
        default:
            break;
        }
        return msg;
    }

    QPair<Shaman::TransactionInterface::Error, QString> stringForError(PackageKit::Client::ErrorType error, const QString
&details)
    {
        QString eTxt;
        Shaman::TransactionInterface::Error tError = Shaman::TransactionInterface::UnknownError;
        switch (error) {
        case PackageKit::Client::ErrorNoNetwork :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("There is no network connection available.\n"
                        "Please check your connection settings and try again");
            break;
        case PackageKit::Client::ErrorNoCache :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The package list needs to be rebuilt.\n"
                        "This should have been done by the backend automatically.");
            break;
        case PackageKit::Client::ErrorOom :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The service that is responsible for handling user requests is out of memory.\n"
                        "Please close some programs or restart your computer.");
            break;
        case PackageKit::Client::ErrorCreateThreadFailed :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("A thread could not be created to service the user request.");
            break;
        case PackageKit::Client::ErrorNotSupported :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The action is not supported by this backend.\n"
                        "Please report a bug as this should not have happened.");
            break;
        case PackageKit::Client::ErrorInternalError :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("A problem that we were not expecting has occurred.\n"
                        "Please report this bug with the error description.");
            break;
        case PackageKit::Client::ErrorGpgFailure :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("A security trust relationship could not be made with the software origin.\n"
                        "Please check your software signature settings. %1", details);
            break;
        case PackageKit::Client::ErrorPackageNotInstalled :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package that is trying to be removed or updated is not already installed.");
            break;
        case PackageKit::Client::ErrorPackageNotFound :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package that is being modified was not found on your system or in any software origin.");
            break;
        case PackageKit::Client::ErrorPackageAlreadyInstalled :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package that is trying to be installed is already installed.");
            break;
        case PackageKit::Client::ErrorPackageDownloadFailed :
            tError = Shaman::TransactionInterface::DownloadFailedError;
            eTxt = details;
            break;
        case PackageKit::Client::ErrorGroupNotFound :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The group type was not found.\n"
                        "Please check your group list and try again.");
            break;
        case PackageKit::Client::ErrorGroupListInvalid :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The group list could not be loaded.\n"
                        "Refreshing your cache may help, although this is normally a software "
                        "origin error.");
            break;
        case PackageKit::Client::ErrorDepResolutionFailed :
            tError = Shaman::TransactionInterface::DependenciesUnsatisfiedError;
            eTxt = details;
            break;
        case PackageKit::Client::ErrorFilterInvalid :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The search filter was not correctly formed.");
            break;
        case PackageKit::Client::ErrorPackageIdInvalid :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The package identifier was not well formed when sent to the system daemon.\n"
                        "This normally indicates an internal bug and should be reported.");
            break;
        case PackageKit::Client::ErrorTransactionError :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = details;
            break;
        case PackageKit::Client::ErrorRepoNotFound :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The remote software origin name was not found.\n"
                        "You may need to enable an item in Software Origins.");
            break;
        case PackageKit::Client::ErrorCannotRemoveSystemPackage :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("Removing a protected system package is not allowed.");
            break;
        case PackageKit::Client::ErrorTransactionCancelled :
            // Pass by
            break;
        case PackageKit::Client::ErrorProcessKill :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The task was canceled successfully and no packages were changed.\n"
                        "The backend did not exit cleanly.");
            break;
        case PackageKit::Client::ErrorFailedConfigParsing :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The native package configuration file could not be opened.\n"
                        "Please make sure your system's configuration is valid.");
            break;
        case PackageKit::Client::ErrorCannotCancel :
            // TODO: Do not report an error here
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The task is not safe to be canceled at this time.");
            break;
        case PackageKit::Client::ErrorCannotInstallSourcePackage :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("Source packages are not normally installed this way.\n"
                        "Check the extension of the file you are trying to install.");
            break;
        case PackageKit::Client::ErrorNoLicenseAgreement :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The license agreement was not agreed to.\n"
                        "To use this software you have to accept the license.");
            break;
        case PackageKit::Client::ErrorFileConflicts :
            tError = Shaman::TransactionInterface::FileConflictError;
            eTxt = details;
            break;
        case PackageKit::Client::ErrorPackageConflicts :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("Multiple packages exist that are not compatible with each other.\n"
                        "This is usually due to mixing packages from different software origins. %1", details);
            break;
        case PackageKit::Client::ErrorRepoNotAvailable :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("There was a (possibly temporary) problem connecting to a software origin.\n"
                        "%1", details);
            break;
        case PackageKit::Client::ErrorFailedInitialization :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("Failed to initialize packaging backend.\n"
                        "This may occur if other packaging tools are being used simultaneously.");
            break;
        case PackageKit::Client::ErrorFailedFinalise :
            // Pass by
            break;
        case PackageKit::Client::ErrorCannotGetLock :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("Cannot get the exclusive lock on the packaging backend.\n"
                        "Please close any other packaging tools that may be open. %1", details);
            break;
        case PackageKit::Client::ErrorNoPackagesToUpdate :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("None of the selected packages could be updated.");
            break;
        case PackageKit::Client::ErrorCannotWriteRepoConfig :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The repository configuration could not be modified.");
            break;
        case PackageKit::Client::ErrorLocalInstallFailed :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("Installing the local file failed. %1"
                        , details);
            break;
        case PackageKit::Client::ErrorBadGpgSignature :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package signature could not be verified.");
            break;
        case PackageKit::Client::ErrorMissingGpgSignature :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package signature was missing and this package is untrusted.\n"
                        "This package was not signed with a GPG key when created.");
            break;
        case PackageKit::Client::ErrorRepoConfigurationError :
            eTxt = i18n("Repository configuration was invalid and could not be read.");
            break;
        case PackageKit::Client::ErrorInvalidPackageFile :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package you are attempting to install is not valid.\n"
                        "The package file could be corrupt, or not a proper package.");
            break;
        case PackageKit::Client::ErrorPackageInstallBlocked :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("Installation of this package was prevented by your packaging system's configuration.");
            break;
        case PackageKit::Client::ErrorPackageCorrupt :
            tError = Shaman::TransactionInterface::DownloadFailedError;
            eTxt = i18n("The package that was downloaded is corrupt and needs to be downloaded again.");
            break;
        case PackageKit::Client::ErrorAllPackagesAlreadyInstalled :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("All of the packages selected for install are already installed on the system.");
            break;
        case PackageKit::Client::ErrorFileNotFound :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The specified file could not be found on the system.\n"
                        "Check that the file still exists and has not been deleted.");
            break;
        case PackageKit::Client::ErrorNoMoreMirrorsToTry :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("Required data could not be found on any of the configured software origins.\n"
                        "There were no more download mirrors that could be tried.");
            break;
        case PackageKit::Client::ErrorNoDistroUpgradeData :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("Required upgrade data could not be found in any of the configured software origins.\n"
                        "The list of distribution upgrades will be unavailable.");
            break;
        case PackageKit::Client::ErrorIncompatibleArchitecture :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package that is trying to be installed is incompatible with this system.");
            break;
        case PackageKit::Client::ErrorNoSpaceOnDevice :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("There is insufficient space on the device.\n"
                        "Free some space on the system disk to perform this operation.");
            break;
        case PackageKit::Client::ErrorMediaChangeRequired :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("Additional media are required to complete the transaction.");
            break;
        case PackageKit::Client::ErrorNotAuthorized :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("You have failed to provide correct authentication.\n"
                        "Please check any passwords or account settings.");
            break;
        case PackageKit::Client::ErrorUpdateNotFound :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The specified update could not be found.\n"
                        "It could have already been installed or no longer available on the remote server.");
            break;
        case PackageKit::Client::ErrorCannotInstallRepoUnsigned :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package could not be installed from untrusted origin.");
            break;
        case PackageKit::Client::ErrorCannotUpdateRepoUnsigned :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("The package could not be updated from untrusted origin.");
            break;
        case PackageKit::Client::ErrorCannotGetFilelist :
        case PackageKit::Client::ErrorCannotGetRequires :
            // Pass by
            break;
        case PackageKit::Client::ErrorCannotDisableRepository :
            tError = Shaman::TransactionInterface::BackendError;
            eTxt = i18n("The specified software origin could not be disabled.");
            break;
        case PackageKit::Client::ErrorRestrictedDownload :
            tError = Shaman::TransactionInterface::DownloadFailedError;
            eTxt = i18n("The download could not be done automatically and should be done manually. %1", details);
            break;
        case PackageKit::Client::ErrorPackageFailedToConfigure :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("One of the selected packages failed to configure correctly. %1"
                        , details);
            break;
        case PackageKit::Client::ErrorPackageFailedToBuild :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("One of the selected packages failed to build correctly. %1"
                        , details);
            break;
        case PackageKit::Client::ErrorPackageFailedToInstall :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("One of the selected packages failed to install correctly. %1"
                        , details);
            break;
        case PackageKit::Client::ErrorPackageFailedToRemove :
            tError = Shaman::TransactionInterface::ProcessingFailedError;
            eTxt = i18n("One of the selected packages failed to be removed correctly. %1"
                        , details);
            break;
        case PackageKit::Client::UnknownErrorType :
            eTxt = i18n("Unknown error, more information available in the detailed report.");
            break;
        default:
            kDebug() << "I cannot handle" << error;
            eTxt = i18n("Unknown error, more information available in the detailed report.");
        }

        return qMakePair(tError, eTxt);
    }

    QString stringForMessage(PackageKit::Client::MessageType type)
    {
        QString txt;
        switch (type) {
        case PackageKit::Client::MessageBrokenMirror :
            txt = i18n("A mirror is possibly broken");
            break;
        case PackageKit::Client::MessageConnectionRefused :
            txt = i18n("The connection was refused");
            break;
        case PackageKit::Client::MessageParameterInvalid :
            txt = i18n("The parameter was invalid");
            break;
        case PackageKit::Client::MessagePriorityInvalid :
            txt = i18n("The priority was invalid");
            break;
        case PackageKit::Client::MessageBackendError :
            txt = i18n("Backend warning");
            break;
        case PackageKit::Client::MessageDaemonError :
            txt = i18n("Daemon warning");
            break;
        case PackageKit::Client::MessageCacheBeingRebuilt :
            txt = i18n("The package list cache is being rebuilt");
            break;
        case PackageKit::Client::MessageUntrustedPackage :
            txt = i18n("An untrusted package was installed");
            break;
        case PackageKit::Client::MessageNewerPackageExists :
            txt = i18n("A newer package exists");
            break;
        case PackageKit::Client::MessageCouldNotFindPackage :
            txt = i18n("Could not find package");
            break;
        case PackageKit::Client::MessageConfigFilesChanged :
            txt = i18n("Configuration files were changed");
            break;
        case PackageKit::Client::MessagePackageAlreadyInstalled :
            txt = i18n("Package is already installed");
            break;
        case PackageKit::Client::MessageAutoremoveIgnored :
            txt = i18n("Automatic cleanup is being ignored");
            break;
        case PackageKit::Client::MessageRepoMetadataDownloadFailed :
            txt = i18n("Software source download failed");
            break;
        case PackageKit::Client::UnknownMessageType :
            break;
        }
        return txt;
    }

    QString daemonErrorToString(PackageKit::Client::DaemonError error)
    {
        QString txt;
        switch (error) {
        case PackageKit::Client::ErrorFailedAuth :
            txt = i18n("You do not have the necessary privileges to perform this action.");
            break;
        case PackageKit::Client::ErrorNoTid :
            txt = i18n("Could not get a transaction id from packagekitd.");
            break;
        case PackageKit::Client::ErrorAlreadyTid :
            txt = i18n("Cannot connect to this transaction id.");
            break;
        case PackageKit::Client::ErrorRoleUnkown :
            txt = i18n("This action is unknown.");
            break;
        case PackageKit::Client::ErrorCannotStartDaemon :
            txt = i18n("The packagekitd service could not be started.");
            break;
        case PackageKit::Client::ErrorInvalidInput :
            txt = i18n("The query is not valid.");
            break;
        case PackageKit::Client::ErrorInvalidFile :
            txt = i18n("The file is not valid.");
            break;
        case PackageKit::Client::ErrorFunctionNotSupported :
            txt = i18n("This function is not yet supported.");
            break;
        case PackageKit::Client::ErrorDaemonUnreachable :
            txt = i18n("Could not talk to packagekitd.");
            break;
        case PackageKit::Client::NoError :
        case PackageKit::Client::ErrorFailed :
        case PackageKit::Client::UnkownError :
            txt = i18n("An unknown error happened.");
            break;
        }
        return txt;
    }

    QIcon iconForMediaType(PackageKit::Transaction::MediaType type) {
        switch (type) {
            case PackageKit::Transaction::MediaDisc:
                return KIcon("drive-removable-media");
            case PackageKit::Transaction::MediaDvd:
                return KIcon("media-optical-dvd");
            default:
                return KIcon("media-optical");
        }
    }
}

#endif
