/* This file is part of the KDE project

   Copyright © 2010 David Palacio <dpalacio@uninorte.edu.co>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef APTBACKEND_H
#define APTBACKEND_H

#include <libqapt/backend.h>

#include <shamanbackendinterface.h>
#include <shamanbackendpackage.h>

class DebPackage;

class AptBackend : public Shaman::BackendInterface
{
    Q_OBJECT
    Q_INTERFACES(Shaman::BackendInterface)

    enum Options
    {
        FollowRecommends = 1,
        FollowSuggests = 2,
        Purge = 4
    };

    public:
        AptBackend();
        ~AptBackend();
    protected:
        void transactionQuestionAnswered(Shaman::TransactionQuestion* question);
        void init();

    public Q_SLOTS:
        virtual bool aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate);
        bool cancelQueueProcessing();
        bool cancelRepositoriesUpdate();
        void updateRepositories();
        Shaman::AbstractPackage *findPackageByName(const QString &name);
        void markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options);
        void refreshAdditionalTargets();
        void processQueue(const Shaman::AbstractPackage::List &packages, const Shaman::ProcessQueueOption::List &options);
        virtual void searchPackages(Shaman::Query *query);

    protected slots:
        void slotErrorOccurred(QApt::ErrorCode error, const QVariantMap &args);
        void slotWorkerEvent(QApt::WorkerEvent event);
        void slotSetTransactionProgress(int percent, int speed, int ETA);
        void slotSetOperationDescription(int flag, const QString& message);
//         void slotProcessRequest(const QVariantMap&);
        void reload();

    private:
        QApt::Backend *m_backend;
};

#endif // APTBACKEND_H
