/* This file is part of the KDE project

   Copyright © 2010 David Palacio <dpalacio@uninorte.edu.co>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "DebPackage.h"

#include <shamanbackendinterface.h>

#include "AptBackend.h"

DebPackage::DebPackage(AptBackend* parent, QApt::Package *package)
    : BackendPackage(package->name(), parent)
    , m_package(package)
    , m_shamanBackend(parent)
{
    setVersion(package->version());
    setAvailableVersion(package->availableVersion());
    setDownloadSize(m_package->downloadSize());

    int state = package->state();
    if (state & QApt::Package::Installed) {
        setState(Shaman::AbstractPackage::Installed);
    } else {
        setState(Shaman::AbstractPackage::NotInstalled);
    }
    if (state & QApt::Package::Upgradeable) {
        setState(Shaman::AbstractPackage::Upgradeable);
    }

    setUnCachedValues(Homepage | License | Maintainer | Size |/* | Providers |*/ Description |
                      RequiredBy | Dependencies | ConflictsWith | FileList/* | Changelog | CustomValues*/);

    if (name().endsWith(QLatin1String("-dbg")) || name().endsWith(QLatin1String("-dbgsym"))) {
        // It's a debug package!
        setProperties(DebugInformationProperty);
    }
}

DebPackage::~DebPackage()
{
}

void DebPackage::cacheValue(Shaman::BackendPackage::CachedValue value)
{
    Shaman::AbstractPackage::List retlist;
    switch (value) {
        case Homepage:
            setDescription(m_package->homepage());
            break;
        case License:
            if (m_package->component() == "main" ||
                m_package->component() == "universe") {
                setLicense(i18n("Open Source"));
            } else if (m_package->component() == "restricted") {
                setLicense(i18n("Proprietary"));
            }
        case Maintainer:
            setDescription(m_package->maintainer());
            break;
        case Size:
            setSize(m_package->availableInstalledSize());
            break;
        case Description:
            setDescription(m_package->longDescription());
            break;
        case RequiredBy:
            foreach (const QString &name, m_package->requiredByList()) {
                retlist.append(m_shamanBackend->findPackageByName(name));
            }
            setRequiredBy(retlist);
        case Dependencies:
            break;
        case ConflictsWith:
            break;
        case FileList:
            setFileList(m_package->installedFilesList());
            break;
        case Changelog:
            break;
        case CustomValues:
            break;
    }

}
