/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "BackendWrapper.h"

#include <KDebug>

using namespace Shaman;

BackendWrapper::BackendWrapper(ScriptBackend * parent)
  : QObject(parent),
    m_backend(parent),
    m_aboutData(0)
{
}

BackendWrapper::~BackendWrapper()
{
}

void BackendWrapper::setBackendClass(Kross::Object::Ptr obj)
{
    m_backend->setBackendClass(obj);
}

void BackendWrapper::reload()
{
    m_backend->reload();
}

void BackendWrapper::setState(BackendWrapper::State state)
{
    m_backend->setState((Shaman::Backend::State)state);
}

void BackendWrapper::setError(BackendWrapper::Error error, const QString &details)
{
    m_backend->setError((Shaman::Backend::Error)error, details);
}

// void BackendWrapper::setCapabilities(BackendWrapper::Capabilities capabilities)
// {
//     emit emitCapabilities((Shaman::Backend::Capabilities)(int)capabilities);
// }

void BackendWrapper::setPackages(QList<QObject*> packages)
{
    Shaman::AbstractPackage::List list;
    foreach (QObject * object, packages) {
        list << qobject_cast<AbstractPackage*>(object);
    }
    m_backend->setPackages(list);
}

void BackendWrapper::setRepositories(QList<QObject*> repos)
{
    Shaman::Repository::List list;
    foreach (QObject * object, repos) {
        list << qobject_cast<Repository*>(object);
    }
    m_backend->setRepositories(list);
}

void BackendWrapper::setPackageGroups(QList<QObject*> groups)
{
    Shaman::PackageGroup::List list;
    foreach (QObject * object, groups) {
        list << qobject_cast<PackageGroup*>(object);
    }
    m_backend->setPackageGroups(list);
}

void BackendWrapper::setTransactionState(BackendWrapper::TransactionInterfaceState state)
{
    m_backend->setTransactionState((Shaman::TransactionInterface::State)state);
}

void BackendWrapper::setTransactionError(BackendWrapper::TransactionInterfaceError error, const QString &details)
{
    m_backend->setTransactionError((Shaman::TransactionInterface::Error)error, details);
}

void BackendWrapper::setPackageTransactionProgress(QObject *package, int percent)
{
    m_backend->setPackageTransactionProgress(qobject_cast<BackendPackage*>(package), percent);
}

void BackendWrapper::setPackageDownloadProgress(QObject *package, qint64 downloadedBytes, qint64 totalBytes)
{
    m_backend->setPackageDownloadProgress(qobject_cast<BackendPackage*>(package), downloadedBytes, totalBytes);
}

QObject* BackendWrapper::manualProgressReporter()
{
    return qobject_cast<QObject*>(m_backend->manualProgressReporter());
}

void BackendWrapper::showOperationProgress(const QString &operation, int progress)
{
    m_backend->showOperationProgress(operation, progress);
}

void BackendWrapper::addTransactionMessage(const QString &message)
{
    m_backend->addTransactionMessage(message);
}

/*int BackendWrapper::newTransactionQuestion(const QString &question,
                                   int possibleAnswers,
                                   const QString &user1Text, const QIcon &user1Icon,
                                   const QString &user2Text, const QIcon &user2Icon,
                                   const QString &user3Text, const QIcon &user3Icon) const
{
    return m_backend->newTransactionQuestion(question, (TransactionInterface::TransactionQuestionAnswers)possibleAnswers, user1Text, user1Icon, user2Text, user2Icon, user3Text, user3Icon);
}*/

void BackendWrapper::customEvent(QEvent *event)
{
    m_backend->customEvent(event);
}

bool BackendWrapper::aboutToChangeState(Backend::State oldstate, Backend::State newstate)
{
    m_backend->aboutToChangeState(oldstate, newstate);
}

void BackendWrapper::setCapabilities(BackendWrapper::TransactionCapabilities transactionCapabilities,
                             BackendWrapper::SearchCapabilities searchCapabilities,
                             BackendWrapper::PackageGroupingCapabilities packageGroupingCapabilities,
                             BackendWrapper::InformationCapabilities informationCapabilities)
{
    m_backend->setCapabilities((BackendCapabilities::TransactionCapabilities)((int)transactionCapabilities), (BackendCapabilities::SearchCapabilities)((int)searchCapabilities), 
                               (BackendCapabilities::PackageGroupingCapabilities)((int)packageGroupingCapabilities), (BackendCapabilities::InformationCapabilities)((int)informationCapabilities));
}

QObject * BackendWrapper::backend()
{
    return qobject_cast<QObject*>(m_backend->backend());
}

void BackendWrapper::setProperties(const QString &name, const QString &description, const QIcon &icon, const QIcon &packageIcon, const QStringList &configurationModules)
{
    m_backend->setProperties(name, description, icon, packageIcon, QList<KAboutPerson>(), configurationModules);
    //TODO:
    // The way to do this is to create an author wrapper class, with a wrappedObject() == KAboutPerson...
}
