/* This file is part of the KDE project

   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef BACKENDWRAPPER_H
#define BACKENDWRAPPER_H

#include "ScriptBackendInterface.h"
#include "shamantransactioninterface.h"

#include <kross/core/object.h>
#include <KAboutData>

using namespace Shaman;

class BackendWrapper : public QObject
{
    Q_OBJECT
    Q_ENUMS(State)
    Q_ENUMS(Error)
    Q_ENUMS(TransactionCapability)
    Q_ENUMS(SearchCapability)
    Q_ENUMS(PackageGroupingCapability)
    Q_ENUMS(InformationCapability)
    Q_ENUMS(TransactionInterfaceState)
    Q_ENUMS(TransactionInterfaceError)
    Q_ENUMS(TransactionQuestionAnswer)
    Q_ENUMS(CachedValue)
    public:
        enum State
        {
            Idling = 0,
            Initializing = 1,
            Reloading = 2,
            Searching = 3,
            MarkingAdditionalTargets = 4,
            WaitingForLock = 5,
            OnTransaction = 6,
            Broken = 7
        };

        enum Error
        {
            NoError = 0,
            InitializationError = 1,
            TransactionError = 2,
            HookError = 3
        };

        enum TransactionCapability {
            UnknownTransactionCapabilities = 0,
            InstallPackagesCapability = 1,
            RemovePackagesCapability = 2,
            UpgradePackagesCapability = 4,
            UpdateRepositoriesCapability = 8,
            UpgradeSystemCapability = 16,
            DistUpgradeCapability = 32,
            InstallPackageFromFileCapability = 64,
        };
        Q_DECLARE_FLAGS(TransactionCapabilities, TransactionCapability)
        Q_FLAGS(TransactionCapabilities)

        enum SearchCapability {
            UnknownSearchCapabilities = 0,
            BasicSearchFromViewCapability = 1,
            ExtendedSearchFromViewCapability = 2,
            BasicSearchFromBackendCapability = 4,
            ExtendedSearchFromBackendCapability = 8
        };
        Q_DECLARE_FLAGS(SearchCapabilities, SearchCapability)
        Q_FLAGS(SearchCapabilities)

        enum PackageGroupingCapability {
            UnknownPackageGroupingCapabilities = 0,
            GroupPackagesByRepositoryCapability = 1,
            GroupPackagesByPackageGroupsCapability = 2
        };
        Q_DECLARE_FLAGS(PackageGroupingCapabilities, PackageGroupingCapability)
        Q_FLAGS(PackageGroupingCapabilities)

        enum InformationCapability {
            UnknownInformationCapabilities = 0,
            ShowFileListCapability = 1,
            ShowDependsOnCapability = 2,
            ShowRequiredByCapability = 4,
            ShowConflictsWithCapability = 8,
            ShowProvidesCapability = 16,
            ShowUpdateType = 32
        };
        Q_DECLARE_FLAGS(InformationCapabilities, InformationCapability)
        Q_FLAGS(InformationCapabilities)

        enum ProcessingOption
        {
            NoOptions = 0,
            ForceProcessing = 1,
            NoHooks = 2
        };
        Q_DECLARE_FLAGS(ProcessingOptions, ProcessingOption)
        Q_FLAGS(ProcessingOptions)

        enum SearchType {
            BasicSearch,
            ExtendedSearch
        };

        enum TransactionInterfaceState
        {
            NoTransaction = 0,
            RunningPreHooks = 1,
            Downloading = 2,
            ProcessingTransaction = 3,
            Cleaning = 4,
            RunningPostHooks = 5
        };

        enum TransactionInterfaceError
        {
            NoTransactionError = 0,
            DownloadFailedError = 1,
            ProcessingFailedError = 2,
            CleanupFailedError = 4,
            FileConflictError = 8,
            DependencyConflictError = 16,
            DependenciesUnsatisfiedError = 32,
            BackendError = 64,
            TransactionHookError = 128,
            UnknownError = 256
        };

        enum TransactionQuestionAnswer {
            UnknownAnswer = 0,
            AnswerYes = 1,
            AnswerNo = 2,
            AnswerCancel = 4,
            AnswerUser1 = 8,
            AnswerUser2 = 16,
            AnswerUser3 = 32
        };
        Q_DECLARE_FLAGS(TransactionQuestionAnswers, TransactionQuestionAnswer)
        Q_FLAGS(TransactionQuestionAnswers)

        enum CachedValue {
            Description = 1,
            Size = 2,
            Maintainer = 4,
            Homepage = 8,
            License = 16,
            Path = 32,
            Providers = 64,
            Dependencies = 128,
            RequiredBy = 256,
            ConflictsWith = 512,
            FileList = 1024
        };

        BackendWrapper(ScriptBackend * parent);
        ~BackendWrapper();

    public slots:
        void setBackendClass(Kross::Object::Ptr obj);

        bool aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate);

        void setState(BackendWrapper::State state);

        void setError(BackendWrapper::Error error, const QString &details);

        void setPackages(QList<QObject*> packages);

        void setRepositories(QList<QObject*> repos);

        void setPackageGroups(QList<QObject*> groups);

        void setTransactionState(BackendWrapper::TransactionInterfaceState state);

        void setTransactionError(BackendWrapper::TransactionInterfaceError error, const QString &details = QString());

        void setPackageTransactionProgress(QObject *package, int percent);

        void setPackageDownloadProgress(QObject *package, qint64 downloadedBytes, qint64 totalBytes);

        QObject *manualProgressReporter();

        void showOperationProgress(const QString &operation, int progress);

        void addTransactionMessage(const QString &message);

        /*int newTransactionQuestion(const QString &question,
                                   int possibleAnswers = 
                                   BackendWrapper::AnswerYes | BackendWrapper::AnswerNo,
                                   const QString &user1Text = QString(), const QIcon &user1Icon = QIcon(),
                                   const QString &user2Text = QString(), const QIcon &user2Icon = QIcon(),
                                   const QString &user3Text = QString(), const QIcon &user3Icon = QIcon()) const;*/

        void customEvent(QEvent *event);

        void setCapabilities(BackendWrapper::TransactionCapabilities transactionCapabilities,
                             BackendWrapper::SearchCapabilities searchCapabilities,
                             BackendWrapper::PackageGroupingCapabilities packageGroupingCapabilities,
                             BackendWrapper::InformationCapabilities informationCapabilities);

        QObject * backend();

        void setProperties(const QString &name, const QString &description,
                           const QIcon &icon, const QIcon &packageIcon,
                           const QStringList &configurationModules);

    private:
        ScriptBackend * m_backend;
        KAboutData * m_aboutData;
}; 

#endif
