/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "ScriptBackendInterface.h"

#include "CreationHandler.h"
#include "BackendWrapper.h"

#include <kross/core/action.h>
#include <kross/core/manager.h>

#include <KDebug>
#include <KAboutData>

class ScriptBackend::Private
{
    public:
        Private()
          : action(0),
            scriptedClass(0),
            creationHandler(0)
        {
        }
        ~Private() {}

        Kross::Action * action;
        Kross::Object::Ptr scriptedClass;
        CreationHandler * creationHandler;
};

ScriptBackend::ScriptBackend(QObject * parent, const QVariantList &)
  : Shaman::BackendInterface(parent),
    d(new Private())
{
}

ScriptBackend::~ScriptBackend()
{
    delete d;
}

void ScriptBackend::init()
{
    setState(Shaman::Backend::Initializing);
    /*setCapabilities(//Transaction capabilities
                    Shaman::BackendCapabilities::InstallPackagesCapability | 
                    Shaman::BackendCapabilities::RemovePackagesCapability |
                    Shaman::BackendCapabilities::UpgradePackagesCapability | 
                    Shaman::BackendCapabilities::UpdateRepositoriesCapability |
                    Shaman::BackendCapabilities::InstallPackageFromFileCapability | 
                    Shaman::BackendCapabilities::UpgradeSystemCapability,
                    //Search capabilities
                    Shaman::BackendCapabilities::BasicSearchFromViewCapability | 
                    Shaman::BackendCapabilities::ExtendedSearchFromViewCapability,
                    //Package Grouping capabilities
                    Shaman::BackendCapabilities::GroupPackagesByPackageGroupsCapability |
                    Shaman::BackendCapabilities::GroupPackagesByRepositoryCapability,
                    //Information capabilities
                    Shaman::BackendCapabilities::ShowRequiredByCapability |
                    Shaman::BackendCapabilities::ShowDependsOnCapability |
                    Shaman::BackendCapabilities::ShowFileListCapability |
                    Shaman::BackendCapabilities::ShowProvidesCapability |
                    Shaman::BackendCapabilities::ShowConflictsWithCapability);*/

    if (d->scriptedClass && d->scriptedClass->methodNames().contains("init")) {
        d->scriptedClass->callMethod("init");
    }
    
    /*KAboutData *about = new KAboutData("scriptbackend", "scriptbackend", ki18n("Scripted Backend"), "1.0.0",
                                       ki18n("still a script")
                                       .subs("alpha"), KAboutData::License_GPL, ki18n("(c) 2009 Lukas Appelhans"),
                                       KLocalizedString(), "http://chakra-project.org");*/

    //about->addAuthor(ki18n("Dario Freddi"), ki18n("Maintainer"), "drf@kde.org", "http://drfav.wordpress.com");
    //setAboutData(about);
    
    //setState(Shaman::Backend::Idling);
}

void ScriptBackend::loadScript(const QString &script)
{
    d->action = new Kross::Action(this, script);

    BackendWrapper * wrapper = new BackendWrapper(this);

    d->action->addObject(wrapper, "BackendInterface");

    d->creationHandler = new CreationHandler(this);
    d->action->addObject(d->creationHandler, "CreationHandler");
    d->action->setFile(script);

    d->action->trigger();
}

void ScriptBackend::setBackendClass(Kross::Object::Ptr obj)
{
    d->scriptedClass = obj;
}

void ScriptBackend::reload()
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("reload"))
        return;
    d->scriptedClass->callMethod("reload");
}

void ScriptBackend::updateRepositories()
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("updateRepositories"))
        return;
    d->scriptedClass->callMethod("updateRepositories");
}

bool ScriptBackend::cancelRepositoriesUpdate()
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("cancelRepositoriesUpdate"))
        return false;
    return d->scriptedClass->callMethod("cancelRepositoriesUpdate").toBool();
}

void ScriptBackend::processQueue(const Shaman::AbstractPackage::List & packages, const Shaman::ProcessQueueOption::List &options)
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("processQueue"))
        return;

    QVariantList list;
    QVariant variant;
    variant.setValue(packages);
    list << variant;
    QVariant opts;
    opts.setValue(options);
    list << opts;

    d->scriptedClass->callMethod("processQueue", list);
}

bool ScriptBackend::cancelQueueProcessing()
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("cancelQueueProcessing"))
        return false;
    return d->scriptedClass->callMethod("cancelQueueProcessing").toBool();
}

void ScriptBackend::transactionQuestionAnswered(Shaman::TransactionQuestion *question)//TODO: Export TransactionInterface::TransactionQuestionAnswer through BackendWrapper to python
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("transactionQuestionAnswered"))
        return;
    QVariant v;
    v.setValue(question);
    d->scriptedClass->callMethod("transactionQuestionAnswered", QVariantList() << v);
}

void ScriptBackend::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("markAdditionalTargetsForQueue"))
        return;
    QVariant v;
    v.setValue(options);
    d->scriptedClass->callMethod("markAdditionalTargetsForQueue", QVariantList() << v);
}

void ScriptBackend::partialReload(const Shaman::BackendPackage::List &packages)
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("partialReload"))
        return;

    //QList<QObject*> pkgs;
    //foreach (Shaman::Package * package, packages)
    //    pkgs << qobject_cast<QObject*>(package);
    
    QVariant v;
    v.setValue(packages);
    
    d->scriptedClass->callMethod("partialReload", QVariantList() << v);
}

void ScriptBackend::searchPackages(const QString &query, Shaman::Backend::SearchType type)
{
    if (!d->scriptedClass || !d->scriptedClass->methodNames().contains("searchPackages"))
        return;
    
    d->scriptedClass->callMethod("partialReload", QVariantList() << QVariant(query) << QVariant(type));
}
