/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef SHAMANABSTRACTPLUGIN_H
#define SHAMANABSTRACTPLUGIN_H

#include "shamanmacros.h"
#include <QtCore/QObject>

#include <KConfigGroup>

namespace Shaman {

class AbstractPluginPrivate;
/** \class AbstractPlugin shamanabstractplugin.h
 * \brief Shaman's Base-Plugin-class
 *
 * This class is the base of all plugins. It gives the plugin access to
 * its KConfigGroup and stores a Plugin-Name in it.
 */
class SHAMAN_EXPORT AbstractPlugin : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(AbstractPlugin)
    Q_DECLARE_PRIVATE(AbstractPlugin)
    public:
        AbstractPlugin(QObject * parent);
        virtual ~AbstractPlugin();

        /**
         * @returns the KConfigGroup where the plugin can store its settings
         * \code
         * config().writeEntry("Example", QString::Number(42));
         * \endcode
         */
        KConfigGroup config() const;

        /**
         * @returns the Plugin-Name
         */
        QString name() const;

    Q_SIGNALS:
        void configChanged();

    protected:
        virtual void init() = 0;

        AbstractPlugin(AbstractPluginPrivate &dd, QObject *parent = 0);
        AbstractPluginPrivate * const d_ptr;

        friend class PluginLoader;
};
}

#endif
