/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanbase_p.h"

#include "shamancoreinterface.h"
#include "shamanbackend.h"
#include "shamanproviderplugin.h"

#include "pluginsystem/PluginLoader_p.h"
#include "pluginsystem/CorePluginLoader_p.h"
#include "pluginsystem/ProviderPluginLoader_p.h"

#include <QThread>

#include <KDebug>
#include "shamancoreinterface_p.h"

namespace Shaman {

class BaseHelper
{
  public:
    BaseHelper() : q(0) {}
    ~BaseHelper() { delete q; }
    Base *q;
};

K_GLOBAL_STATIC(BaseHelper, s_globalBase)

Base *Base::instance()
{
    if (!s_globalBase->q) {
        new Base;
    }

    return s_globalBase->q;
}

Base::Base(QObject * parent)
  : QObject(parent),
    d_ptr(new BasePrivate())
{
    Q_ASSERT(!s_globalBase->q);
    s_globalBase->q = this;
    Q_D(Base);
    d->q_ptr = this;
    KGlobal::locale()->insertCatalog("libshaman");

    //Initialize PluginLoader
    d->pluginLoader = new PluginLoader(d->config, this);
}

Base::~Base()
{
    delete d_ptr;
}

void Base::init(OperationMode mode)
{
    Q_D(Base);
    //Initialize CoreInterface
    d->coreInterface = new CoreInterface(this);
    d->mode = mode;

    // Set up pluginLoader connections
    connect(d->pluginLoader, SIGNAL(backendPluginCreated(Shaman::Backend*)), SLOT(__k__slotBackendCreated(Shaman::Backend*)));
    connect(d->pluginLoader, SIGNAL(pluginsLoaded()), SLOT(__k__slotSetProviderPlugins()));
    connect(d->pluginLoader, SIGNAL(pluginsLoaded()), this, SLOT(__k__initCompleted()));

    // Register the basic loaders
    registerPluginLoader(new CorePluginLoader);
    registerPluginLoader(new ProviderPluginLoader);

    // Load 'em up
    d->pluginLoader->loadPlugins();
}

void Base::registerPluginLoader(AbstractPluginLoader* loader)
{
    Q_D(Base);
    d->pluginLoader->registerPluginLoader(loader);
}

KSharedConfig::Ptr Base::configurationFile()
{
    Q_D(Base);
    return d->config;
}

void Base::setConfigurationFile(KSharedConfig::Ptr cfg)
{
    Q_D(Base);
    d->config = cfg;
}

CoreInterface * Base::core()
{
    Q_D(Base);
    return d->coreInterface;
}

void Base::setConfigurationChanged()
{
    emit configChanged();
}

void BasePrivate::__k__slotBackendCreated(Backend *backend)
{
    Q_Q(Base);
    q->connect(q, SIGNAL(destroyed()), backend, SLOT(deleteLater()));
    coreInterface->d_func()->setBackend(backend);
}

void BasePrivate::__k__slotProviderCreated(Provider *provider)
{
    Q_Q(Base);
    q->connect(q, SIGNAL(destroyed()), provider, SLOT(deleteLater()));
}

void BasePrivate::__k__slotSetProviderPlugins()
{
    coreInterface->d_func()->setProviders(pluginLoader->providers());
}

void BasePrivate::__k__initCompleted()
{
    Q_Q(Base);
    emit q->initCompleted();
    // After doing that, happily start reloading the whole backend
    // Doing this we can start the GUI earlier
    foreach (Shaman::Provider *provider, coreInterface->providers()) {
        provider->reload();
    }
}
}

#include "shamanbase.moc"
