/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamancoreplugin.h"
#include "shamancoreplugin_p.h"
#include "shamanbase.h"
#include "shamancoreinterface.h"
#include "shamantransactioninterface.h"
#include "shamantransactioninterface_p.h"

namespace Shaman {

CorePlugin::CorePlugin(QObject* parent)
        : AbstractPlugin(*(new CorePluginPrivate()) ,parent)
{

}

CorePlugin::CorePlugin(CorePluginPrivate& dd, QObject* parent)
    : AbstractPlugin(dd, parent)
{

}

CorePlugin::~CorePlugin()
{

}

CoreInterface* CorePlugin::core()
{
    return Base::instance()->core();
}

CorePlugin::Hooks CorePlugin::registeredHooks() const
{
    Q_D(const CorePlugin);

    Hooks retvalue = 0;
    QHash<Hook, const char*>::const_iterator i;
    for (i = d->hooks.constBegin(); i != d->hooks.constEnd(); ++i) {
        if (!(retvalue & i.key())) {
            retvalue |= i.key();
        }
    }
    
    return retvalue;
}

void CorePlugin::unregisterHook(CorePlugin::Hook hook)
{
    Q_D(CorePlugin);

    d->hooks.remove(hook);
    emit hookUnregistered(hook);
}

void CorePlugin::registerHook(CorePlugin::Hook hook, const char* member)
{
    Q_D(CorePlugin);

    d->hooks.insert(hook, member);
    emit hookRegistered(hook, member);
}

void CorePlugin::unregisterAllHooks()
{
    Q_D(CorePlugin);

    d->hooks.clear();
}

const char * CorePluginPrivate::memberForHook(CorePlugin::Hook hook)
{
    if (hooks.contains(hook)) {
        return hooks[hook];
    } else {
        return 0;
    }
}

void CorePlugin::hookMessage(const QString& message)
{
    Shaman::Base::instance()->core()->transactionInterface()->d->newTransactionMessageFromHook(message);
}

}

#include "shamancoreplugin.moc"
