/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamandistupgrade_p.h"

namespace Shaman {

DistUpgrade::DistUpgrade(QObject* parent)
    : QObject(parent)
    , d_ptr(new DistUpgradePrivate)
{

}

DistUpgrade::~DistUpgrade()
{

}

DistUpgrade::States DistUpgrade::availableStates() const
{
    Q_D(const DistUpgrade);
    return d->availableStates;
}

QString DistUpgrade::customStateIconOrMovieName(DistUpgrade::State customState) const
{
    Q_D(const DistUpgrade);
    return d->stateIcons[customState];
}

DistUpgrade::ViewType DistUpgrade::customStateViewType(DistUpgrade::State customState) const
{
    Q_D(const DistUpgrade);
    return d->stateViews[customState];
}

QString DistUpgrade::customStateName(DistUpgrade::State customState) const
{
    Q_D(const DistUpgrade);
    return d->stateNames[customState];
}

QString DistUpgrade::description() const
{
    Q_D(const DistUpgrade);
    return d->description;
}

WidgetFactory* DistUpgrade::details() const
{
    Q_D(const DistUpgrade);
    return d->details;
}

QString DistUpgrade::name() const
{
    Q_D(const DistUpgrade);
    return d->name;
}

bool DistUpgrade::needsSetup()
{
    Q_D(const DistUpgrade);
    return d->setup.count() > 0;
}

QList< WidgetFactory* > DistUpgrade::setup() const
{
    Q_D(const DistUpgrade);
    return d->setup;
}

DistUpgrade::State DistUpgrade::state() const
{
    Q_D(const DistUpgrade);
    return d->state;
}

void DistUpgrade::setAvailableStates(DistUpgrade::States states)
{
    Q_D(DistUpgrade);
    d->availableStates = states;
}

void DistUpgrade::setCustomStateIconOrMovieByName(DistUpgrade::State customState, const QString& name)
{
    Q_D(DistUpgrade);
    d->stateIcons[customState] = name;
}

void DistUpgrade::setCustomStateName(DistUpgrade::State customState, const QString& name)
{
    Q_D(DistUpgrade);
    d->stateNames[customState] = name;
}

void DistUpgrade::setCustomStateViewType(DistUpgrade::State customState, DistUpgrade::ViewType type)
{
    Q_D(DistUpgrade);
    d->stateViews[customState] = type;
}

void DistUpgrade::setDescription(const QString& description)
{
    Q_D(DistUpgrade);
    d->description = description;
}

void DistUpgrade::setDetails(WidgetFactory* details)
{
    Q_D(DistUpgrade);
    d->details = details;
}

void DistUpgrade::setName(const QString& name)
{
    Q_D(DistUpgrade);
    d->name = name;
}

void DistUpgrade::setSetup(WidgetFactory* setup)
{
    Q_D(DistUpgrade);
    d->setup = QList<WidgetFactory*>() << setup;
}

void DistUpgrade::setSetup(QList< WidgetFactory* > setup)
{
    Q_D(DistUpgrade);
    d->setup = setup;
}

void DistUpgrade::setState(DistUpgrade::State state)
{
    Q_D(DistUpgrade);
    d->state = state;
}

}

#include "shamandistupgrade.moc"
