/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamanmacros.h"
#include "shamanmacros_p.h"
#include <KDebug>

namespace Shaman {

unsigned int versionMajor()
{
    return LIBSHAMAN_VERSION_MAJOR;
}

unsigned int versionMinor()
{
    return LIBSHAMAN_VERSION_MINOR;
}

unsigned int versionRelease()
{
    return LIBSHAMAN_VERSION_RELEASE;
}

const char *versionString()
{
    return LIBSHAMAN_VERSION_STRING;
}

////////////////

class PerformanceMonitorHelper
{
  public:
    PerformanceMonitorHelper() : q(0) {}
    ~PerformanceMonitorHelper() { delete q; }
    PerformanceMonitor *q;
};

K_GLOBAL_STATIC(PerformanceMonitorHelper, s_globalPerformanceMonitor)

PerformanceMonitor *PerformanceMonitor::instance()
{
    if (!s_globalPerformanceMonitor->q) {
        new PerformanceMonitor;
    }

    return s_globalPerformanceMonitor->q;
}

PerformanceMonitor::PerformanceMonitor()
{
    Q_ASSERT(!s_globalPerformanceMonitor->q);
    s_globalPerformanceMonitor->q = this;
}

PerformanceMonitor::~PerformanceMonitor()
{
}

void PerformanceMonitor::start(const QString &name)
{
    QTime t;
    kDebug() << "PERFORMANCE MONITOR --- Start" << name;
    t.start();
    m_times.insert(name, t);
}

void PerformanceMonitor::stop(const QString &name)
{
    kDebug() << "PERFORMANCE MONITOR --- " << name << " finished in " << m_times[name].elapsed() << " milliseconds";
    m_times.remove(name);
}

}
