/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamanmetapackage_p.h"

namespace Shaman {

MetaPackage::MetaPackage(const QString& name, QObject* parent)
        : AbstractPackage(*(new MetaPackagePrivate(name, this)), parent)
{
}

MetaPackage::~MetaPackage()
{
    foreach (AbstractPackage *package, packages()) {
        package->d_func()->metapackages.removeOne(this);
    }
}

void MetaPackagePrivate::__k__rootPackageChanged(Shaman::AbstractPackage *package)
{
    Q_Q(MetaPackage);
    if (package) {
        //q->setDescription(package->description());
        q->setVersion(package->version());
        q->setAvailableVersion(package->availableVersion());
        q->setState(package->state());
    }
}

void MetaPackage::cacheValue(Shaman::AbstractPackage::CachedValue value)
{
    Q_D(MetaPackage);
    if (d->root) {
        d->root->cacheValue(value);
    } else {
        // Hmmm... this really should not happen
    }
}

QList< Shaman::Repository* > MetaPackage::repositories()
{
    Q_D(MetaPackage);
    if (d->root) {
        return d->root->repositories();
    } else {
        return QList< Shaman::Repository* >();
    }
}

QList< Shaman::PackageGroup* > MetaPackage::packageGroups()
{
    Q_D(MetaPackage);
    if (d->root) {
        return d->root->packageGroups();
    } else {
        return QList< Shaman::PackageGroup* >();
    }
}

QIcon MetaPackage::icon() const
{
    Q_D(const MetaPackage);
    if (d->root) {
        return d->root->icon();
    } else {
        return Shaman::AbstractPackage::icon();
    }
}

void MetaPackage::addPackage(AbstractPackage* package)
{
    Q_D(MetaPackage);
    d->packages.append(package);
    package->d_func()->metapackages.append(this);
    //emit packagesChanged(packages());
    // If the root package is empty, set this package as the root one
    if (!d->root) {
        d->root = package;
        //emit rootPackageChanged(d->root);
        d->__k__rootPackageChanged(d->root);
    }
}

AbstractPackage::List MetaPackage::packages() const
{
    Q_D(const MetaPackage);
    return d->packages;
}

AbstractPackage* MetaPackage::rootPackage()
{
    Q_D(MetaPackage);
    return d->root;
}

void MetaPackage::removePackage(AbstractPackage* package)
{
    Q_D(MetaPackage);
    d->packages.removeOne(package);
    package->d_func()->metapackages.removeOne(this);
    //emit packagesChanged(packages());
    // If this package was the root one, remove it from the root and set as root the first package available, if any
    if (d->root == package) {
        if (!d->packages.isEmpty()) {
            d->root = d->packages.first();
        } else {
            d->root = 0;
        }
        //emit rootPackageChanged(d->root);
        d->__k__rootPackageChanged(d->root);
    }
}

void MetaPackage::setRootPackage(AbstractPackage* package)
{
    Q_D(MetaPackage);
    // If this package is not in the list, skip that out
    if (!d->packages.contains(package)) {
        return;
    }
    d->root = package;
    //emit rootPackageChanged(d->root);
    d->__k__rootPackageChanged(d->root);
}

}

//#include "shamanmetapackage.moc"
