/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANPACKAGEGROUP_H
#define SHAMANPACKAGEGROUP_H

#include "shamanabstractpackage.h"
#include "shamanmacros.h"
#include <QObject>
#include <QString>

namespace Shaman {
/** \class PackageGroup shamanpackagegroup.h
 * \brief General PackageGroup-class.
 *
 * This class is the implementation of a Package-Group in Shaman.
 * Basically it's a Shaman::Package::List and does not need to be reimplemented.
 * 
 * \code
 * Package Group * group = new PackageGroup("Package-Group", this);
 * group->append(package);
 * \endcode
 * 
 * This initializes an empty Package-Group and adds the Package package to it afterwards.
 */
class SHAMAN_EXPORT PackageGroup : public QObject, public Shaman::AbstractPackage::List
{
    Q_OBJECT
    Q_PROPERTY(QString name READ name WRITE setName)
//     Q_PROPERTY(QIcon icon READ icon WRITE setIcon)
    public:
        typedef QList<PackageGroup*> List;
        PackageGroup(const QString &name, QObject *parent);
        PackageGroup(const QString &name, const QIcon &icon, QObject *parent);
        virtual ~PackageGroup();

    public slots:
        /**
         * Sets the name of the Package-Group
         * @param name the new Package-Group-Name
         */
        void setName(const QString &name);

        /**
         * @returns the name of the Package-Group
         */
        QString name() const;

        QIcon icon() const;

        void setIcon(const QIcon &icon);

    private:
        class Private;
        Private *d;
};
}

#endif
