/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamanprovider_p.h"
#include "shamanproviderplugin_p.h"

namespace Shaman
{

Provider::Provider(Shaman::ProviderPlugin* plugin, QObject* parent)
    : QObject(parent)
    , d_ptr(new ProviderPrivate(plugin))
{
    Q_D(Provider);
    d->q_ptr = this;
    // Inject the plugin
    plugin->d_func()->provider = this;
}

Provider::~Provider()
{

}

Provider::Capabilities Provider::capabilities() const
{
    Q_D(const Provider);
    return d->capabilities;
}

AbstractPackage::List Provider::packages() const
{
    Q_D(const Provider);
    return d->packages;
}

Repository::List Provider::repositories() const
{
    Q_D(const Provider);
    return d->repos;
}

PackageGroup::List Provider::packageGroups() const
{
    Q_D(const Provider);
    return d->groups;
}

QSet< AbstractPackage* > Provider::availablePackages() const
{
    Q_D(const Provider);
    QSet< AbstractPackage* > retset;
    retset.unite(d->plugin->d_func()->searchCache);
    retset.unite(d->packages.toSet());
    return retset;
}

AbstractPackage::List Provider::queue() const
{
    AbstractPackage::List retlist;
    // Let's find out
    foreach (AbstractPackage * const package, availablePackages()) {
        if (package->action() == AbstractPackage::Install ||
            package->action() == AbstractPackage::Uninstall ||
            package->action() == AbstractPackage::Upgrade) {
            retlist.append(package);
        }
    }

    return retlist;
}

Backend::State Provider::state() const
{
    Q_D(const Provider);
    return d->state;
}

void Provider::cancelProcessing()
{
    Q_D(Provider);
    d->plugin->cancelProcessing();
}

void Provider::markAdditionalTargetsForQueue()
{
    Q_D(Provider);
    d->plugin->setState(Shaman::Backend::MarkingAdditionalTargets);
    d->plugin->markAdditionalTargetsForQueue();
}

void Provider::process()
{
    Q_D(Provider);
    d->plugin->setState(Shaman::Backend::OnTransaction);
    d->plugin->process();
}

void Provider::reload()
{
    Q_D(Provider);
    d->plugin->setState(Shaman::Backend::Reloading);
    d->plugin->reload();
}

}

#include "shamanprovider.moc"
