/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef PROVIDER_H
#define PROVIDER_H

#include <QtCore/QObject>
#include <QtCore/QPair>

#include "shamanproviderpackage.h"
#include "shamanmacros.h"
#include "shamanbackend.h"

namespace Shaman {

class ProviderPlugin;
class ProviderPrivate;
class SHAMAN_EXPORT Provider : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(Provider)
    Q_DECLARE_PRIVATE(Provider)
    public:
        ~Provider();

        enum Capabilities {
            NoCapabilities = 0,
            ExposePackagesCapability = 1,
            RetrieveAdditionalPackagesOnSearchCapability = 2,
        };

        typedef QList<Provider*> List;

        Backend::State state() const;
        Capabilities capabilities() const;

        AbstractPackage::List queue() const;

        void markAdditionalTargetsForQueue();
        void process();
        void cancelProcessing();
        void reload();

        AbstractPackage::List packages() const;
        QSet< AbstractPackage* > availablePackages() const;
        Repository::List repositories() const;
        PackageGroup::List packageGroups() const;

    Q_SIGNALS:
        void availablePackagesChanged(Shaman::AbstractPackage::List packages);
        void packagesChanged(Shaman::AbstractPackage::List packages);
        void repositoriesChanged(Shaman::Repository::List repos);
        void packageGroupsChanged(Shaman::PackageGroup::List packageGroups);
        void additionalTargetsRetrieved(const Shaman::AbstractPackage::List &additionalProvider);
        void stateChanged(Shaman::Backend::State state);
        //void errorTriggered(Shaman::Provider::Error error, const QString &details);
        //void newMessage(const QString &message, Shaman::Provider::MessageLevel level);
        void packageProgressChanged(Shaman::ProviderPackage * package, int progress);
        void progressChanged(int progress);

    private:
        explicit Provider(ProviderPlugin *plugin, QObject *parent = 0);

        ProviderPrivate * const d_ptr;

        friend class PluginLoader;
        friend class ProviderPlugin;
        friend class CoreInterface;
        friend class ProviderPluginLoader;
};

}

#endif // PROVIDER_H
