/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanproviderpackage_p.h"

#include "shamanproviderplugin.h"

namespace Shaman {

ProviderPackage::ProviderPackage(const QString &name, ProviderPlugin * provider)
  : AbstractPackage(*(new ProviderPackagePrivate(name, provider)), provider)
{
}

ProviderPackage::~ProviderPackage()
{
}

AbstractPackage * ProviderPackage::package()
{
    Q_D(ProviderPackage);
    return d->package;
}

Provider * ProviderPackage::provider()
{
    Q_D(ProviderPackage);
    return d->provider->provider();
}

void ProviderPackage::setPackage(AbstractPackage * package)
{
    Q_D(ProviderPackage);
    d->package = package;
}

QList< PackageGroup* > ProviderPackage::packageGroups()
{
    PackageGroup::List list;
    foreach (PackageGroup * group, provider()->packageGroups())
    {
        if (group->contains(this)) {
            list.append(group);
        }
    }
    return list;
}

QList< Repository* > ProviderPackage::repositories()
{
    Repository::List list;
    foreach (Repository * group, provider()->repositories())
    {
        if (group->contains(this)) {
            list.append(group);
        }
    }
    return list;
}

}
