/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANPROVIDERPACKAGE_H
#define SHAMANPROVIDERPACKAGE_H

#include "shamanabstractpackage.h"
#include "shamanmacros.h"

namespace Shaman {

class Provider;
class ProviderPlugin;
class ProviderPackagePrivate;
class SHAMAN_EXPORT ProviderPackage : public AbstractPackage
{
    Q_DECLARE_PRIVATE(ProviderPackage)
    public:
        typedef QList<ProviderPackage*> List;
        ProviderPackage(const QString &name, ProviderPlugin * provider);
        ~ProviderPackage();

        /**
         * @returns the package built out of this ProviderPackage, 0 if no package has been built
         */
        AbstractPackage * package();

        /**
         * @returns the plugin which provides this ProviderPackage
         */
        Provider * provider();

        /**
         * @returns a list of all PackageGroups which are containing this package
         */
        virtual QList<Shaman::PackageGroup*> packageGroups();

        /**
         * @returns a list of all Repositories which are containing this package
         */
        virtual QList<Shaman::Repository*> repositories();

        /**
         * Sets the package built out of this ProviderPackage
         * @param package the package built out of this ProviderPackage
         */
        void setPackage(AbstractPackage * package);
};
}

#endif
