/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanproviderplugin_p.h"
#include "shamanprovider_p.h"
#include "shamanbase.h"
#include "shamancoreinterface.h"
#include "shamancoreinterface_p.h"

namespace Shaman {

ProviderPlugin::ProviderPlugin(QObject * parent)
  : AbstractPlugin(*(new ProviderPluginPrivate), parent)
{
}

ProviderPlugin::~ProviderPlugin()
{
}

void ProviderPlugin::setState(Backend::State state)
{
    Q_D(ProviderPlugin);
    if (state == d->provider->d_ptr->state) {
        return;
    }

    d->provider->d_ptr->state = state;
}

void ProviderPlugin::reload()
{
    setState(Shaman::Backend::Idling);
}

void ProviderPlugin::setAdditionalTargetsForQueue(const Shaman::AbstractPackage::List& packages)
{
    Q_D(ProviderPlugin);
    emit d->provider->additionalTargetsRetrieved(packages);
    setState(Backend::Idling);
}

void ProviderPlugin::setPackageProgress(AbstractPackage* package, int progress)
{

}

Provider* ProviderPlugin::provider() const
{
    Q_D(const ProviderPlugin);
    return d->provider;
}

void ProviderPlugin::setSearchResult(const QUuid& search, const Shaman::AbstractPackage::List& result)
{
    Q_D(ProviderPlugin);
    if (d->provider->capabilities() & Provider::RetrieveAdditionalPackagesOnSearchCapability) {
        // Then we need to populate the cache as well
        d->searchCache = d->searchCache.unite(result.toSet());
        emit d->provider->availablePackagesChanged(d->provider->availablePackages().toList());
    }
    Shaman::Base::instance()->core()->d_func()->queryCompleted(this, search, result);
    setState(Backend::Idling);
}

QSet< AbstractPackage* > ProviderPlugin::searchCache() const
{
    Q_D(const ProviderPlugin);
    return d->searchCache;
}

void ProviderPlugin::setCapabilities(Provider::Capabilities capabilities)
{
    Q_D(ProviderPlugin);
    d->provider->d_func()->capabilities = capabilities;
}

void ProviderPlugin::setPackages(const Shaman::AbstractPackage::List& packages)
{
    Q_D(ProviderPlugin);
    d->provider->d_func()->packages = packages;
    emit d->provider->packagesChanged(packages);
    emit d->provider->availablePackagesChanged(d->provider->availablePackages().toList());
}

void ProviderPlugin::setRepositories(const Shaman::Repository::List& repos)
{
    Q_D(ProviderPlugin);
    d->provider->d_func()->repos = repos;
    emit d->provider->repositoriesChanged(repos);
}

void ProviderPlugin::setPackageGroups(const Shaman::PackageGroup::List& groups)
{
    Q_D(ProviderPlugin);
    d->provider->d_func()->groups = groups;
    emit d->provider->packageGroupsChanged(groups);
}

}

#include "shamanproviderplugin.moc"
