/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANPROVIDERPLUGIN_H
#define SHAMANPROVIDERPLUGIN_H

#include "shamanabstractplugin.h"

#include "shamanabstractpackage.h"
#include "shamanmacros.h"
#include "shamanprovider.h"
#include <QtCore/QSet>

namespace Shaman {

class Query;
class Backend;
class ProviderPluginPrivate;
class SHAMAN_EXPORT ProviderPlugin : public AbstractPlugin
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(ProviderPlugin)
    Q_DISABLE_COPY(ProviderPlugin)

    friend class PluginLoader;
    friend class Provider;

    public:
        ProviderPlugin(QObject * parent);
        ~ProviderPlugin();

        Shaman::Provider *provider() const;

        virtual void markAdditionalTargetsForQueue() = 0;
        virtual void process() = 0;
        virtual void cancelProcessing() = 0;
        virtual void reload();

        virtual void search(Query *search) = 0;

        QSet< AbstractPackage* > searchCache() const;

    protected:
        void setState(Backend::State state);
        //void setError(Provider::Error error, const QString &details);
        //void addMessage(const QString &message, Provider::MessageLevel level);
        void setPackageProgress(Shaman::AbstractPackage * package, int progress);
        void setSearchResult(const QUuid &searchUuid, const Shaman::AbstractPackage::List &result);
        void setAdditionalTargetsForQueue(const Shaman::AbstractPackage::List &packages);
        void setPackages(const Shaman::AbstractPackage::List &packages);
        void setRepositories(const Shaman::Repository::List &repos);
        void setPackageGroups(const Shaman::PackageGroup::List &groups);

        void setCapabilities(Shaman::Provider::Capabilities capabilities);

};
}

#endif
