/* This file is part of the KDE project

   Copyright (C) 2010 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamanquery_p.h"

namespace Shaman {

class QueryPoolHelper
{
  public:
    QueryPoolHelper() : q(0) {}
    ~QueryPoolHelper() { delete q; }
    QueryPool *q;
};

K_GLOBAL_STATIC(QueryPoolHelper, s_globalQueryPool)

QueryPool *QueryPool::instance()
{
    if (!s_globalQueryPool->q) {
        new QueryPool;
    }

    return s_globalQueryPool->q;
}

QueryPool::QueryPool()
{
    Q_ASSERT(!s_globalQueryPool->q);
    s_globalQueryPool->q = this;
}

QueryPool::~QueryPool()
{

}

Query *QueryPool::fromUuid(const QUuid &uuid)
{
    if (queries.contains(uuid)) {
        return queries[uuid];
    }

    return 0;
}

/////////////

void QueryPrivate::setState(Query::State newstate)
{
    Q_Q(Query);
    if (state == newstate) {
        return;
    }

    state = newstate;
    emit q->stateChanged(newstate);

    if (newstate == Query::Completed) {
        emit q->completed(metapackages);
    }
}

//////////

Query::Query(const QRegExp& search, Type type, AbstractPackage::Properties matchingProperties, Targets targets, QObject* parent)
        : QObject(parent)
        , d_ptr(new QueryPrivate(this))
{
    Q_D(Query);
    d->search = search;
    d->type = type;
    d->matchingProperties = matchingProperties;
    d->targets = targets;

    QueryPool::instance()->queries.insert(d->uid, this);
}

Query::~Query()
{
    Q_D(Query);
    QueryPool::instance()->queries.remove(d->uid);
}

QUuid Query::id() const
{
    Q_D(const Query);
    return d->uid;
}

AbstractPackage::Properties Query::matchingProperties() const
{
    Q_D(const Query);
    return d->matchingProperties;
}

QRegExp Query::query() const
{
    Q_D(const Query);
    return d->search;
}

QList< MetaPackage* > Query::result() const
{
    Q_D(const Query);
    return d->metapackages;
}

Query::State Query::state() const
{
    Q_D(const Query);
    return d->state;
}

Query::Targets Query::targets() const
{
    Q_D(const Query);
    return d->targets;
}

}

#include "shamanquery.moc"
