/* This file is part of the KDE project

   Copyright (C) 2010 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef SHAMANQUERY_H
#define SHAMANQUERY_H

#include "shamanmetapackage.h"
#include "shamanmacros.h"

#include <QObject>
#include <QtCore/QRegExp>
#include <QtCore/QUuid>

namespace Shaman {

class QueryPrivate;
class SHAMAN_EXPORT Query : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(Query)
    Q_DECLARE_PRIVATE(Query)
    public:
        enum Type {
            BasicSearch = 0,
            ExtendedSearch = 1,
            FileSearch = 2,
            ExtendedRelationalSearch = 3,
            FileRelationalSearch = 4
        };
        enum State {
            Ready = 0,
            Running = 1,
            Completed = 2
        };
        enum Target {
            None = 0,
            BackendTarget = 1,
            ProvidersTarget = 2,
            AllTargets = 4
        };
        Q_DECLARE_FLAGS(Targets, Target)

        explicit Query(const QRegExp &search, Type type = BasicSearch,
                       AbstractPackage::Properties matchingProperties = AbstractPackage::NoProperties,
                       Targets = AllTargets, QObject* parent = 0);
        virtual ~Query();

        QUuid id() const;
        QRegExp query() const;
        AbstractPackage::Properties matchingProperties() const;
        Targets targets() const;

        QList<MetaPackage*> result() const;
        State state() const;

    Q_SIGNALS:
        void stateChanged(State newstate);
        void completed(const Shaman::MetaPackage::List &result);

    private:
        QueryPrivate * const d_ptr;

        friend class CoreInterface;
        friend class CoreInterfacePrivate;
};

}

Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::Query::Targets)

#endif // SHAMANQUERY_H
