/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanrepository.h"
#include <KIcon>

namespace Shaman {
class Repository::Private
{
    public:
        Private(const KUrl &u)
          : url(u),
            state(Repository::Idling)
        {}
        ~Private()
        {}

        KUrl url;
        State state;
};

Repository::Repository(const QString &name, const KUrl &url, QObject *parent)
  : PackageGroup(name, KIcon("repository"), parent),
    d(new Private(url))
{
    qRegisterMetaType<State>("Repository::State");
    qRegisterMetaType<Repository*>();
    qRegisterMetaType<Shaman::Repository::List>();
}

Repository::Repository(const QString& name, const QIcon& icon, const KUrl& url, QObject* parent)
    : PackageGroup(name, icon, parent)
    , d(new Private(url))
{
    qRegisterMetaType<State>("Repository::State");
    qRegisterMetaType<Repository*>();
    qRegisterMetaType<Shaman::Repository::List>();
}

Repository::~Repository()
{
}

void Repository::setUrl(const KUrl &url)
{
    d->url = url;
}

KUrl Repository::url() const
{
    return d->url;
}

void Repository::setState(State state)
{
    if (state != UpdatingFinished) {
        d->state = state;
    } else {
        d->state = Idling;
    }
    emit stateChanged(state);
}

Repository::State Repository::state()
{
    return d->state;
}

}

#include "shamanrepository.moc"
