/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef TRANSACTIONINTERFACE_P
#define TRANSACTIONINTERFACE_P

#include "shamantransactioninterface.h"

namespace Shaman {

    enum AutomaticComputation {
            NoComputation = 0,
            TotalDownloadedSize = 1,
            DownloadSpeed = 2,
            RemainingTime = 4,
            TransactionProgress = 8
        };
        Q_DECLARE_FLAGS(AutomaticComputations, AutomaticComputation)

class TransactionInterface::Private
{
    public:
        Private()
          : state(TransactionInterface::NoTransaction),
            error(TransactionInterface::NoError),
            downloadSpeed(-1)
        {}
        ~Private()
        {}

        Q_DECLARE_PUBLIC(TransactionInterface)
        TransactionInterface *q_ptr;

        State state;
        Error error;
        QString errorDetails;
        Shaman::AbstractPackage::List packages;
        qint64 downloadSpeed;
        QTime elapsedTime;
        QTime downloadTime;
        QTime remainingTime;
        qint64 totalDownloadSize;
        qint64 totalDownloaded;
        QHash<Shaman::AbstractPackage*, qint64> downloadStatus;
        QList<Shaman::AbstractPackage*> transactionPackages;
        int packagePercentageInfluence;
        int transactionProgress;
        bool downloadReliable;
        QList< QPair<QString, Shaman::TransactionInterface::MessageLevel> > transactionMessages;
        QList<Shaman::TransactionQuestion*> transactionQuestions;
        AutomaticComputations automaticComputations;

        void reset();
        void setPackages(const Shaman::AbstractPackage::List &packages);
        void setTransactionState(Shaman::TransactionInterface::State state);
        void setTransactionError(Shaman::TransactionInterface::Error error, const QString &details);
        void setPackageTransactionProgress(Shaman::AbstractPackage *package, int percent);
        void setPackageDownloadProgress(Shaman::AbstractPackage *package, qint64 downloadedBytes, qint64 totalBytes);
        void newTransactionMessageFromBackend(const QString &message);
        void newTransactionMessageFromHook(const QString &message);
        void newTransactionQuestion(Shaman::TransactionQuestion *question);
        void setTransactionProgress(int percent);
        void setTotalDownloadedSize(qint64 downloadedBytes);
        void setTotalDownloadSize(qint64 totalBytes);
        void setDownloadSpeed(qint64 bytesPerSecond);
        void setRemainingTime(const QTime &dueToFinishAt);
};

}

Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::AutomaticComputations)

#endif
