/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamantransactionquestion_p.h"
#include "shamanbase.h"
#include "shamancoreinterface.h"
#include "shamanbackend_p.h"
#include "shamanbackendinterface.h"

namespace Shaman
{

TransactionQuestion::TransactionQuestion(WidgetFactory* factory, TransactionQuestion::Answers possibleAnswers)
    : d_ptr(new TransactionQuestionPrivate)
{
    Q_D(TransactionQuestion);

    d->possibleAnswers = possibleAnswers;
    d->factory = factory;
}

TransactionQuestion::TransactionQuestion(const QMetaObject& metaobject, TransactionQuestion::Answers possibleAnswers)
    : d_ptr(new TransactionQuestionPrivate)
{
    Q_D(TransactionQuestion);

    d->possibleAnswers = possibleAnswers;
    d->factory = new MetaObjectWidgetFactory(metaobject);
}

TransactionQuestion::~TransactionQuestion()
{
    if (!d_func()->answerWidget.isNull()) {
        d_func()->answerWidget.data()->deleteLater();
    }
    delete d_ptr;
}

TransactionQuestion::Answers TransactionQuestion::possibleAnswers() const
{
    Q_D(const TransactionQuestion);
    return d->possibleAnswers;
}

TransactionQuestion::Answer TransactionQuestion::answer() const
{
    Q_D(const TransactionQuestion);
    return d->answer;
}

void TransactionQuestion::setAnswer(TransactionQuestion::Answer answer)
{
    Q_D(TransactionQuestion);
    d->answer = answer;

    // Now deliver the good (or bad) news to the backend
    Shaman::Base::instance()->core()->backend()->d->interface->transactionQuestionAnswered(this);
}

void TransactionQuestion::setUser1Icon(const QIcon& icon)
{
    Q_D(TransactionQuestion);
    d->user1Icon = icon;
}
 
void TransactionQuestion::setUser1Text(const QString& text)
{
    Q_D(TransactionQuestion);
    d->user1Text = text;
}

void TransactionQuestion::setUser2Icon(const QIcon& icon)
{
    Q_D(TransactionQuestion);
    d->user2Icon = icon;
}

void TransactionQuestion::setUser2Text(const QString& text)
{
    Q_D(TransactionQuestion);
    d->user2Text = text;
}

void TransactionQuestion::setUser3Icon(const QIcon& icon)
{
    Q_D(TransactionQuestion);
    d->user3Icon = icon;
}

void TransactionQuestion::setUser3Text(const QString& text)
{
    Q_D(TransactionQuestion);
    d->user3Text = text;
}

QIcon TransactionQuestion::user1Icon() const
{
    Q_D(const TransactionQuestion);
    return d->user1Icon;
}

QString TransactionQuestion::user1Text() const
{
    Q_D(const TransactionQuestion);
    return d->user1Text;
}

QIcon TransactionQuestion::user2Icon() const
{
    Q_D(const TransactionQuestion);
    return d->user2Icon;
}

QString TransactionQuestion::user2Text() const
{
    Q_D(const TransactionQuestion);
    return d->user2Text;
}

QIcon TransactionQuestion::user3Icon() const
{
    Q_D(const TransactionQuestion);
    return d->user3Icon;
}

QString TransactionQuestion::user3Text() const
{
    Q_D(const TransactionQuestion);
    return d->user3Text;
}

QWidget* TransactionQuestion::answerWidget() const
{
    Q_D(const TransactionQuestion);
    return d->answerWidget.data();
}

void TransactionQuestion::setAnswerWidget(QWidget* widget)
{
    Q_D(TransactionQuestion);
    d->answerWidget = widget;
}

WidgetFactory* TransactionQuestion::questionWidgetFactory() const
{
    Q_D(const TransactionQuestion);
    return d->factory;
}

QUuid TransactionQuestion::id() const
{
    Q_D(const TransactionQuestion);
    return d->id;
}

template<class T> T* TransactionQuestion::answerWidgetByName(const QString& name)
{
    Q_D(TransactionQuestion);
    T *retval = 0;
    if (!d->answerWidget) {
        return retval;
    }

    retval = d->answerWidget->findChild<T*>(name);

    return retval;
}

/////

StringTransactionQuestion::StringTransactionQuestion(const QString& text, TransactionQuestion::Answers possibleAnswers,
                                                     const QIcon& icon, bool modal)
       : TransactionQuestion(new LabelIconWidgetFactory(text, icon), possibleAnswers)
       , d(new Private(modal))
{
    d->text = text;
    d->icon = icon;
}

StringTransactionQuestion::~StringTransactionQuestion()
{
    delete d;
}

void StringTransactionQuestion::setModal(bool modal)
{
    d->modal = modal;
}

bool StringTransactionQuestion::modal() const
{
    return d->modal;
}

QString StringTransactionQuestion::text() const
{
    return d->text;
}

QIcon StringTransactionQuestion::icon() const
{
    return d->icon;
}

}
