/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef SHAMANTRANSACTIONQUESTION_P_H
#define SHAMANTRANSACTIONQUESTION_P_H

#include "shamantransactionquestion.h"
#include "shamanwidgetfactory.h"
#include <QBoxLayout>
#include <QLabel>
#include <QPointer>

namespace Shaman
{

class MetaObjectWidgetFactory : public WidgetFactory
{
    public:
        MetaObjectWidgetFactory(const QMetaObject &metaobject) : m_metaObject(metaobject) {}
        virtual ~MetaObjectWidgetFactory() {}

        virtual QWidget* createInstance() const {
            return qobject_cast<QWidget*>(m_metaObject.newInstance());
        }

    private:
        QMetaObject m_metaObject;
};

class LabelIconWidgetFactory : public WidgetFactory
{
    public:
        LabelIconWidgetFactory(const QString &text, const QIcon &icon) : m_text(text), m_icon(icon) {}
        virtual ~LabelIconWidgetFactory() {}

        virtual QWidget* createInstance() const {
            QWidget *retwidget = new QWidget;
            QHBoxLayout *h = new QHBoxLayout;
            QVBoxLayout *v = new QVBoxLayout;

            QLabel *plabel = new QLabel;
            QLabel *label = new QLabel(m_text);
            v->addStretch();
            v->addWidget(label);
            v->addStretch();

            plabel->setPixmap(m_icon.pixmap(64));
            h->addWidget(plabel);
            h->addLayout(v);

            retwidget->setLayout(h);

            return retwidget;
        }

    private:
        QString m_text;
        QIcon m_icon;
};

class TransactionQuestionPrivate
{
    public:
        TransactionQuestionPrivate() : id(QUuid::createUuid()) {}

        WidgetFactory *factory;
        TransactionQuestion::Answer answer;
        QPointer<QWidget> answerWidget;

        QString user1Text;
        QIcon user1Icon;

        QString user2Text;
        QIcon user2Icon;

        QString user3Text;
        QIcon user3Icon;

        QUuid id;
        TransactionQuestion::Answers possibleAnswers;
};

class StringTransactionQuestion::Private
{
    public:
        Private(bool m) : modal(m) {}

        bool modal;
        QString text;
        QIcon icon;
};

}

#endif
