/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "TransactionLoggerConfig.h"
#include "ui_TransactionLoggerConfig.h"

#include <shamanmacros.h>

SHAMAN_PLUGIN_CONFIG_EXPORT(TransactionLoggerConfig)

TransactionLoggerConfig::TransactionLoggerConfig(QWidget* parent, const QVariantList& args)
        : PluginConfig(ShamanFactory::componentData(), parent, args)
{
    setButtons(Apply);

    m_ui = new Ui::TransactionLoggerConfig;
    m_ui->setupUi(this);
    m_ui->sizeSpin->setSpecialValueText(i18n("Unlimited"));

    // KCModule changed signals

    connect(m_ui->logTransactionBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->logRepoBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->compressBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));

    connect(m_ui->overwriteRadio, SIGNAL(toggled(bool)), this, SIGNAL(changed(bool)));
    connect(m_ui->archiveRadio, SIGNAL(toggled(bool)), this, SIGNAL(changed(bool)));

    connect(m_ui->sizeSpin, SIGNAL(valueChanged(int)), this, SLOT(changed()));

    connect(m_ui->archiveUrl, SIGNAL(textChanged(QString)), this, SLOT(changed()));

    // Appearance signals
    connect(m_ui->archiveRadio, SIGNAL(toggled(bool)), m_ui->archiveUrl, SLOT(setEnabled(bool)));
    connect(m_ui->archiveRadio, SIGNAL(toggled(bool)), m_ui->compressBox, SLOT(setEnabled(bool)));
    connect(m_ui->archiveRadio, SIGNAL(toggled(bool)), m_ui->archiveLabel, SLOT(setEnabled(bool)));

    connect(m_ui->sizeSpin, SIGNAL(valueChanged(int)), this, SLOT(sizeValueChanged(int)));
}

TransactionLoggerConfig::~TransactionLoggerConfig()
{

}

void TransactionLoggerConfig::load()
{
    m_ui->logTransactionBox->setChecked(pluginConfigGroup().readEntry("logTransactions", true));
    m_ui->logRepoBox->setChecked(pluginConfigGroup().readEntry("logRepoUpdates", false));
    m_ui->sizeSpin->setValue(pluginConfigGroup().readEntry("logSize", 100));
    if (pluginConfigGroup().readEntry("sizeExceededAction", "overwrite") == "overwrite") {
        m_ui->overwriteRadio->setChecked(true);
    } else {
        m_ui->archiveRadio->setChecked(true);
    }
    m_ui->archiveUrl->setUrl(KUrl::fromPath(pluginConfigGroup().readEntry("archivePath", QString())));
    m_ui->compressBox->setChecked(pluginConfigGroup().readEntry("archiveCompress", true));
}

void TransactionLoggerConfig::save()
{
    pluginConfigGroup().writeEntry("logTransactions", m_ui->logTransactionBox->isChecked());
    pluginConfigGroup().writeEntry("logRepoUpdates", m_ui->logRepoBox->isChecked());
    pluginConfigGroup().writeEntry("logSize", m_ui->sizeSpin->value());
    if (m_ui->overwriteRadio->isChecked()) {
        pluginConfigGroup().writeEntry("sizeExceededAction", "overwrite");
    } else {
        pluginConfigGroup().writeEntry("sizeExceededAction", "archive");
    }
    QString url = m_ui->archiveUrl->url().pathOrUrl(KUrl::AddTrailingSlash);
    pluginConfigGroup().writeEntry("archivePath", url);
    pluginConfigGroup().readEntry("archiveCompress", m_ui->compressBox->isChecked());

    // Write
    pluginConfigGroup().config()->sync();
}

void TransactionLoggerConfig::sizeValueChanged(int value)
{
    if (value > 0) {
        m_ui->archiveRadio->setEnabled(true);
        m_ui->overwriteRadio->setEnabled(true);

        if (m_ui->archiveRadio->isChecked()) {
            m_ui->archiveLabel->setEnabled(true);
            m_ui->archiveUrl->setEnabled(true);
            m_ui->compressBox->setEnabled(true);
        }
    } else {
        m_ui->archiveRadio->setEnabled(false);
        m_ui->overwriteRadio->setEnabled(false);
        m_ui->archiveLabel->setEnabled(false);
        m_ui->archiveUrl->setEnabled(false);
        m_ui->compressBox->setEnabled(false);
    }
}

