/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "FavouriteSearchModel.h"

#include <KIcon>

FavouriteSearchModel::FavouriteSearchModel(const KConfigGroup &config, QObject *parent)
  : QAbstractItemModel(parent),
    m_config(config)
{
    foreach (const QString &key, m_config.keyList())
    {
        Search search;
        search.search = key;
        int type = m_config.readEntry(key).toInt();
        search.type = (SearchModel::Type) type;
        m_searches.append(search);
    }
}

FavouriteSearchModel::~FavouriteSearchModel()
{
    foreach (const Search &search, m_searches)
    {
        m_config.writeEntry(search.search, QString::number(search.type));
    }
    m_config.sync();
    m_config.parent().sync();
}

void FavouriteSearchModel::addFavouriteSearch(const QString &string, SearchModel::Type type)
{
    beginInsertRows(QModelIndex(), m_searches.count(), m_searches.count() + 1);
    Search search;
    search.search = string;
    search.type = type;
    m_searches.append(search);
    endInsertRows();
}

int FavouriteSearchModel::columnCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return 1;
}

QVariant FavouriteSearchModel::data(const QModelIndex & index, int role) const
{
    if (role == Qt::DisplayRole)
    {
        return m_searches.at(index.row()).search;
    }
    else if (role == Qt::DecorationRole) 
    {
        return KIcon("rating").pixmap(16, 16);
    }
    return QVariant();
}

QModelIndex FavouriteSearchModel::index(int row, int column, const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    if (row >= 0 && row < m_searches.count())
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex FavouriteSearchModel::parent(const QModelIndex & index) const
{
    Q_UNUSED(index);
    return QModelIndex();
}

int FavouriteSearchModel::rowCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return m_searches.count();
}

FavouriteSearchModel::Search FavouriteSearchModel::searchFromModelIndex(const QModelIndex &index)
{
    if (index.row() >= 0 && index.row() < m_searches.count()) {
        return m_searches.at(index.row());
    }
    return FavouriteSearchModel::Search();
}
