/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackageGroupModel.h"

#include <KIcon>
#include <KLocale>

PackageGroupModel::PackageGroupModel(const Shaman::PackageGroup::List &groups, const Shaman::AbstractPackage::List &list, QObject
*parent)
  : QAbstractItemModel(parent),
    m_groups(groups),
    m_allPackages(0)
{
    setDefaultPackageGroup(list);
    setPackageGroups(groups);
}

PackageGroupModel::~PackageGroupModel()
{
}

int PackageGroupModel::columnCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return 1;
}

QVariant PackageGroupModel::data(const QModelIndex & index, int role) const
{
    if (role == Qt::DisplayRole) {
        return m_groups.at(index.row())->name();
    }
    else if (role == Qt::DecorationRole) {
        return m_groups.at(index.row())->icon().pixmap(32, 32);
    }
    return QVariant();
}

QModelIndex PackageGroupModel::index(int row, int column, const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    if (row >= 0 && row < m_groups.count())
        return createIndex(row, column, m_groups.at(row));
    return QModelIndex();
}

QModelIndex PackageGroupModel::parent(const QModelIndex & index) const
{
    Q_UNUSED(index);
    return QModelIndex();
}

int PackageGroupModel::rowCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return m_groups.count();
}

Shaman::PackageGroup * PackageGroupModel::packageGroupFromModelIndex(const QModelIndex &index)
{
    if (index.row() >= 0 && index.row() < m_groups.count())
        return m_groups.at(index.row());
    return 0;
}

void PackageGroupModel::setPackageGroups(const Shaman::PackageGroup::List &groupList)
{
    m_groups = groupList;
    m_groups.prepend(m_allPackages);
    reset();
    return;
}

void PackageGroupModel::setDefaultPackageGroup(const Shaman::AbstractPackage::List &packages)
{
    if (!m_allPackages) {
        beginInsertRows(QModelIndex(), 0, 0);
        m_allPackages = new Shaman::PackageGroup(i18n("All Groups"), KIcon("bookmark-new-list"), this);
        m_groups.prepend(m_allPackages);
        endInsertRows();
    }
    m_allPackages->clear();
    *m_allPackages << packages;
}
