/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackageStateModel.h"

#include <KIcon>
#include <KLocale>

PackageStateModel::PackageStateModel(QObject *parent)
  : QAbstractItemModel(parent)
{
    m_states << AllPackages << InstalledPackages << NotInstalledPackages << UpgradeablePackages << QueuedPackages;
}

PackageStateModel::~PackageStateModel()
{
}

int PackageStateModel::columnCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return 1;
}

QVariant PackageStateModel::data(const QModelIndex & index, int role) const
{
    if (role == Qt::DisplayRole)
    {
        switch (m_states.at(index.row()))
        {
            case AllPackages:
                return i18n("All Packages");
            case InstalledPackages:
                return i18n("Installed Packages");
            case NotInstalledPackages:
                return i18n("Not installed Packages");
            case UpgradeablePackages:
                return i18n("Upgradeable Packages");
            case QueuedPackages:
                return i18n("Packages in Queue");
        }
    }
    else if (role == Qt::DecorationRole) {
        switch (m_states.at(index.row()))
        {
            case AllPackages:
                return KIcon("view-process-all-tree").pixmap(16, 16);
            case InstalledPackages:
                return KIcon("user-online").pixmap(16, 16);
            case NotInstalledPackages:
                return KIcon("user-offline").pixmap(16, 16);
            case UpgradeablePackages:
                return KIcon("user-invisible").pixmap(16, 16);
            case QueuedPackages:
                return KIcon("user-away-extended").pixmap(16, 16);
        }
    }
    return QVariant();
}

QModelIndex PackageStateModel::index(int row, int column, const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    if (row >= 0 && row < m_states.count())
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex PackageStateModel::parent(const QModelIndex & index) const
{
    Q_UNUSED(index);
    return QModelIndex();
}

int PackageStateModel::rowCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return m_states.count();
}

PackageStateModel::State PackageStateModel::stateFromIndex(int index)
{
    if (index >= 0 && index < m_states.count()) {
        return m_states.at(index);
    }
    return AllPackages;
}
