/* This file is part of the KDE project

   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef PACKAGESMODEL_H
#define PACKAGESMODEL_H

#include <QAbstractItemModel>
#include <QPointer>
#include <shamanmetapackage.h>
#include <shamanpackagewatcher.h>

class Item;
class AbstractItem
{
    public:
        AbstractItem() {}
        virtual ~AbstractItem() {}

        virtual bool isPackageItem() {return false;}
};

class PackageItem : public AbstractItem
{
    public:
        PackageItem(Shaman::AbstractPackage * package, Item * parent);

        virtual bool isPackageItem();
        Shaman::AbstractPackage * package();
        Item * parent();

    private:
        Shaman::AbstractPackage * m_package;
        Item * m_parent;
};
class Item : public AbstractItem
{
    public:
        Item(Shaman::MetaPackage * package);
        virtual ~Item();

        Shaman::MetaPackage * metaPackage();
        void append(PackageItem * item);
        QList<PackageItem*> children();
        int count();
        void sort();

    private:
        Shaman::MetaPackage * m_package;
        QList<PackageItem*> m_children;
};

class PackagesModel : public QAbstractItemModel
{
    Q_OBJECT
    public:
        PackagesModel(Shaman::MetaPackage::List packages, QObject *parent);
        ~PackagesModel();

        virtual int columnCount(const QModelIndex & parent = QModelIndex()) const;
        virtual QVariant data (const QModelIndex & index, int role = Qt::DisplayRole) const;
        virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
        virtual QModelIndex index(int row, int column, const QModelIndex & parent = QModelIndex()) const;
        virtual QModelIndex parent(const QModelIndex & index) const;
        virtual int rowCount(const QModelIndex & parent = QModelIndex()) const;
        Shaman::AbstractPackage * packageFromModelIndex(const QModelIndex &index);
        Shaman::MetaPackage * metaPackageFromModelIndex(const QModelIndex &index);

    public slots:
        void setPackages(const Shaman::MetaPackage::List &pkgList);
        void watch(Shaman::AbstractPackage::List &pkgList);

    private slots:
        void reloadPackage();
        
    private:
        QList<Item*> m_packages;
        QList< QPointer<Shaman::PackageWatcher> > m_watchers;
};

#endif
