/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef PACKAGESSORTFILTERPROXYMODEL_H
#define PACKAGESSORTFILTERPROXYMODEL_H

#include "PackageStateModel.h"
#include "SearchModel.h"

#include <shamanbackend.h>

#include <QSortFilterProxyModel>
#include <QString>
#include <shamanmetapackage.h>

namespace Shaman {
    class Repository;
    class PackageGroup;
}

using namespace Shaman;

class PackagesSortFilterProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
    public:
        PackagesSortFilterProxyModel(QObject *parent);
        ~PackagesSortFilterProxyModel();

        PackageStateModel::State packageState() {return m_state;}
        SearchModel::Type searchType() {return m_searchType;}
        QString searchText() const {return m_searchText;}
        Repository * repositoryFilter() const {return m_repo;}
        PackageGroup * packageGroupFilter() const {return m_group;}

        bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;

    public slots:
        void setPackageState(PackageStateModel::State state);
        void setSearchType(SearchModel::Type searchType);
        void setSearchText(const QString searchText = QString());
        void setRepositoryFilter(Repository * repo = 0);//TODO: Change to Repository/Group-class??
        void setPackageGroupFilter(PackageGroup * group = 0);
        void searchCompleted(const Shaman::MetaPackage::List &packages);

    signals:
        void textSearchCompleted(bool dummy = true);

    private:
        Repository * m_repo;
        PackageGroup * m_group;
        PackageStateModel::State m_state;
        QString m_searchText;
        SearchModel::Type m_searchType;
        BackendCapabilities::SearchCapabilities m_searchCapabilities;
        QList<Shaman::MetaPackage*> m_packages;
};

#endif
