/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef PACKAGESVIEW_H
#define PACKAGESVIEW_H

#include <QTreeView>
#include <QEventLoop>

#include <kdeversion.h>

#include <shamanbackend.h>

#if KDE_IS_VERSION(4,3,67)
#include <KPixmapSequence>
#else
#include <QPixmap>
#endif

class QTimeLine;
class KAction;
class KMenu;

namespace Shaman {
    class BackendPackage;
}

class CacheEventLoop : public QEventLoop
{
    Q_OBJECT
    public:
        explicit CacheEventLoop(Shaman::AbstractPackage::CachedValue value, QObject* parent = 0)
         : QEventLoop(parent), m_value(value), m_cached(false) {}
         virtual ~CacheEventLoop() {}

         int exec(ProcessEventsFlags flags = AllEvents) {
             if (m_cached) {
                 return 0;
             } else {
                 return QEventLoop::exec(flags);
             }
         }

    public slots:
        void requestQuit(Shaman::AbstractPackage::CachedValue value) {
            if (m_value == value) {
                m_cached = true;
                quit();
            }
        }

    private:
        Shaman::AbstractPackage::CachedValue m_value;
        bool m_cached;
};

class PackagesView : public QTreeView
{
    Q_OBJECT
    public:
        PackagesView(QWidget * parent = 0);
        ~PackagesView();

        void init();

        QList<KAction*> actions();

    public slots:
        void setEnabled(bool enabled);
        void backendStateChanged(Shaman::Backend::State state);

    signals:
        void currentPackageChanged(Shaman::AbstractPackage * package);

    protected:
        void contextMenuEvent(QContextMenuEvent * e);
        void paintEvent(QPaintEvent* event);

    private slots:
        void currentRowChanged(const QModelIndex &current, const QModelIndex &previous);
        void installCurrentPackages();
        void updateCurrentPackages();
        void uninstallCurrentPackages();
        void cancelCurrentPackages();
        void stopRetainingPaintEvent();

    private:
        QPointer<KAction> m_installAction;
        QPointer<KAction> m_updateAction;
        QPointer<KAction> m_uninstallAction;
        KAction * m_cancelAction;
        KMenu * m_contextMenu;
        bool m_retainPaintEvent;
        QTimeLine * m_fadeTimeLine;
        QTimeLine * m_spinnerTimeLine;
#if KDE_IS_VERSION(4,3,67)
        KPixmapSequence m_sequence;
#else
        void loadSpinnerFrames();
        QVector<QPixmap> m_spinner;
#endif
        QPixmap m_backgroundPixmap;
        QString m_message;
        Shaman::Backend::State m_cachedState;
        bool m_addingToQueue;
};

#endif 
