/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef PACKAGESVIEWDOCKVIEW_H
#define PACKAGESVIEWDOCKVIEW_H

#include <QListView>

#include <kdeversion.h>

#if KDE_IS_VERSION(4,3,67)
#include <KPixmapSequence>
#else
#include <QPixmap>
#endif

class QTimeLine;
class QPaintEvent;

class PackagesViewDockView : public QListView
{
    Q_OBJECT
    Q_ENUMS(ViewType)

    public:
        enum ViewType {
            NoView = 0,
            RepositoriesView = 1,
            PackageGroupsView = 2,
            FavoriteSearchesView = 3
        };

        PackagesViewDockView(QWidget* parent = 0);
        virtual ~PackagesViewDockView();

        void setViewType(ViewType type);

    public slots:
        void setEnabled(bool enabled);
        void slotRowsInserted();

    private slots:
        void stopRetainingPaintEvent();

    protected:
        virtual void paintEvent(QPaintEvent* e);

    private:
        ViewType m_viewType;
        bool m_retainPaintEvent;
        QTimeLine * m_fadeTimeLine;
        QTimeLine * m_spinnerTimeLine;
#if KDE_IS_VERSION(4,3,67)
        KPixmapSequence m_sequence;
#else
        void loadSpinnerFrames();
        QVector<QPixmap> m_spinner;
#endif
        QPixmap m_backgroundPixmap;
        QString m_message;
};

#endif // PACKAGESVIEWDOCKVIEW_H
