/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef PACKAGESVIEWDOCKWIDGET_H
#define PACKAGESVIEWDOCKWIDGET_H

#include "ui_PackagesViewDockWidget.h"

#include "SearchModel.h"

#include <QDockWidget>

namespace Shaman {
    class Backend;
    class PackageGroup;
    class Repository;
}

class RepositoryModel;
class PackageGroupModel;
class FavouriteSearchModel;
class QSortFilterProxyModel;
class PackagesViewPlugin;

class PackagesViewDockWidget : public QDockWidget, public Ui::PackagesViewDockWidget
{
    Q_OBJECT
    public:
        PackagesViewDockWidget(Shaman::Backend * backend, PackagesViewPlugin *plugin, QWidget * parent = 0);
        ~PackagesViewDockWidget();

    public slots:
        void addFavouriteSearch(const QString &search, SearchModel::Type type);

    signals:
        void packageGroupFilterChanged(Shaman::PackageGroup * group);
        void repositoryFilterChanged(Shaman::Repository * repository);
        void changeSearch(const QString &search, SearchModel::Type type);

    private slots:
        void changeModel();
        void currentRowChanged(const QModelIndex &current, const QModelIndex &previous);

    private:
        Shaman::Backend * m_backend;
        QSortFilterProxyModel * m_proxyModel;
        RepositoryModel * m_repoModel;
        PackageGroupModel * m_groupModel;
        FavouriteSearchModel * m_favouriteSearchModel;
        PackagesViewPlugin *m_plugin;
};

#endif
