/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackagesViewPlugin.h"

#include "PackagesViewWidget.h"
#include "PackagesViewDockWidget.h"

#include <shamanguiinterface.h>
#include <shamancoreinterface.h>
#include <shamanstatusbarinterface.h>

#include <KAction>

SHAMAN_PLUGIN_EXPORT(PackagesViewPlugin)

PackagesViewPlugin::PackagesViewPlugin(QObject *parent, const QVariantList&)
  : Shaman::GuiPlugin(parent)
{
}

PackagesViewPlugin::~PackagesViewPlugin()
{
    //delete m_dockWidget;
    //delete m_widget;
}

void PackagesViewPlugin::init()
{
    statusBar()->setStandardMessage(i18n("%1 Packages", core()->metaPackages().count()), core()->backend()->properties()->packageIcon().pixmap(22, 22));
    m_dockWidget = new PackagesViewDockWidget(core()->backend(), this);
    gui()->addDockWidget(Qt::LeftDockWidgetArea, m_dockWidget);
    m_widget = new PackagesViewWidget(core()->backend());
    connect(m_widget, SIGNAL(currentPackageChanged(Shaman::AbstractPackage *)),
            this, SLOT(setCurrentPackage(Shaman::AbstractPackage *)));
    connect(m_widget, SIGNAL(favouriteSearchAdded(const QString &, SearchModel::Type)), 
            m_dockWidget, SLOT(addFavouriteSearch(const QString &, SearchModel::Type)));
    connect(m_dockWidget, SIGNAL(changeSearch(const QString &, SearchModel::Type)), m_widget, SLOT(setSearch(const QString &, SearchModel::Type)));
    connect(m_dockWidget, SIGNAL(repositoryFilterChanged(Shaman::Repository *)), m_widget, SLOT(setRepositoryFilter(Shaman::Repository *)));
    connect(m_dockWidget, SIGNAL(packageGroupFilterChanged(Shaman::PackageGroup *)), m_widget, SLOT(setPackageGroupFilter(Shaman::PackageGroup *)));
    gui()->setMainWidget(m_widget);
    foreach (KAction * action, m_widget->packagesViewActions())
        gui()->addAction(action);
}

void PackagesViewPlugin::setCurrentPackage(Shaman::AbstractPackage * package)
{
    core()->setCurrentPackage(package);
}
