/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackagesViewWidget.h"
#include "PackagesModel.h"
#include "PackagesSortFilterProxyModel.h"
#include "PackageStateModel.h"

#include <shamanbackend.h>

#include <KIcon>
#include <QTimer>
#include <QDebug>
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <KDebug>

PackagesViewWidget::PackagesViewWidget(Shaman::Backend * backend, QWidget * parent)
  : QWidget(parent),
    m_backend(backend),
    m_searchTimer(new QTimer(this))
{
    setupUi(this);
    addFavouriteSearchButton->setIcon(KIcon("rating"));

    m_searchTimer->setInterval(1000);
    m_searchTimer->setSingleShot(true);
    connect(m_searchTimer, SIGNAL(timeout()), this, SLOT(startSearch()));

    m_model = new PackagesModel(Shaman::Base::instance()->core()->metaPackages(), this);
    m_proxyModel = new PackagesSortFilterProxyModel(this);
    m_proxyModel->setSourceModel(m_model);
    treeView->setModel(m_proxyModel);
    treeView->init();

    m_stateModel = new PackageStateModel(this);
    stateCombo->setModel(m_stateModel);

    m_searchModel = new SearchModel(this);
    searchCombo->setModel(m_searchModel);

    connect(searchLine, SIGNAL(textChanged(const QString &)), m_searchTimer, SLOT(start()));
    connect(m_proxyModel, SIGNAL(textSearchCompleted(bool)), searchLine, SLOT(setEnabled(bool)));
    connect(m_proxyModel, SIGNAL(textSearchCompleted(bool)), treeView, SLOT(setEnabled(bool)));
    connect(stateCombo, SIGNAL(currentIndexChanged(int)), SLOT(stateComboIndexChanged(int)));
    connect(searchCombo, SIGNAL(currentIndexChanged(int)), SLOT(searchComboIndexChanged(int)));
    connect(addFavouriteSearchButton, SIGNAL(clicked()), SLOT(addFavouriteSearch()));
    connect(treeView, SIGNAL(currentPackageChanged(Shaman::AbstractPackage *)),
            SIGNAL(currentPackageChanged(Shaman::AbstractPackage*)));
    connect(Shaman::Base::instance()->core(), SIGNAL(metaPackagesChanged(Shaman::MetaPackage::List)),
            m_model, SLOT(setPackages(Shaman::MetaPackage::List)));
}

PackagesViewWidget::~PackagesViewWidget()
{
}

void PackagesViewWidget::setRepositoryFilter(Shaman::Repository * repository)
{
    m_proxyModel->setRepositoryFilter(repository);
}

void PackagesViewWidget::setPackageGroupFilter(Shaman::PackageGroup * group)
{
    m_proxyModel->setPackageGroupFilter(group);
}

void PackagesViewWidget::setSearch(const QString &search, SearchModel::Type type)
{
    searchCombo->setCurrentIndex(m_searchModel->indexFromType(type));
    searchLine->setText(search);
}

void PackagesViewWidget::stateComboIndexChanged(int index)
{
    m_proxyModel->setPackageState(m_stateModel->stateFromIndex(index));
}

void PackagesViewWidget::searchComboIndexChanged(int index)
{
    m_proxyModel->setSearchType(m_searchModel->typeFromIndex(index));
}

QList<KAction*> PackagesViewWidget::packagesViewActions()
{
    return treeView->actions();
}

void PackagesViewWidget::addFavouriteSearch()
{
    if (searchLine->text().isEmpty())
        return;
    emit favouriteSearchAdded(searchLine->text(), m_searchModel->typeFromIndex(searchCombo->currentIndex()));
}

void PackagesViewWidget::startSearch()
{
    searchLine->setEnabled(false);
    treeView->setEnabled(false);
    m_proxyModel->setSearchText(searchLine->text());
}
