/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "RepositoryModel.h"

#include <KIcon>
#include <KLocale>

RepositoryModel::RepositoryModel(const Shaman::Repository::List &repos, const Shaman::AbstractPackage::List &allPackages, QObject
*parent)
  : QAbstractItemModel(parent),
    m_repos(repos),
    m_allPackages(0)
{
    setDefaultRepository(allPackages);
    setRepositories(repos);
}

RepositoryModel::~RepositoryModel()
{
}

int RepositoryModel::columnCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return 1;
}

QVariant RepositoryModel::data(const QModelIndex & index, int role) const
{
    if (role == Qt::DisplayRole) {
        return m_repos.at(index.row())->name();
    }
    else if (role == Qt::DecorationRole) {
        return m_repos.at(index.row())->icon().pixmap(32, 32);
    }
    return QVariant();
}

QModelIndex RepositoryModel::index(int row, int column, const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    if (row >= 0 && row < m_repos.count())
        return createIndex(row, column, m_repos.at(row));
    return QModelIndex();
}

QModelIndex RepositoryModel::parent(const QModelIndex & index) const
{
    Q_UNUSED(index);
    return QModelIndex();
}

int RepositoryModel::rowCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return m_repos.count();
}

Shaman::Repository * RepositoryModel::repositoryFromModelIndex(const QModelIndex &index)
{
    if (index.row() >= 0 && index.row() < m_repos.count())
        return m_repos.at(index.row());
    return 0;
}

void RepositoryModel::setRepositories(const Shaman::Repository::List &repoList)
{
    m_repos = repoList;
    m_repos.prepend(m_allPackages);
    reset();
    return;
}

void RepositoryModel::setDefaultRepository(const Shaman::AbstractPackage::List &packages)
{
    if (!m_allPackages) {
        beginInsertRows(QModelIndex(), 0, 0);
        m_allPackages = new Shaman::Repository(i18n("All Repositories"), KUrl(), this);
        m_repos.prepend(m_allPackages);
        endInsertRows();
    }
    m_allPackages->clear();
    *m_allPackages << packages;
}
