/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "SearchModel.h"

#include <KIcon>
#include <KLocale>

SearchModel::SearchModel(QObject *parent)
  : QAbstractItemModel(parent)
{
    m_types << Basic << Extended;
}

SearchModel::~SearchModel()
{
}

int SearchModel::columnCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return 1;
}

QVariant SearchModel::data(const QModelIndex & index, int role) const
{
    if (role == Qt::DisplayRole)
    {
        switch (m_types.at(index.row()))
        {
            case Basic:
                return i18n("Basic");
            case Extended:
                return i18n("Extended");
        }
    }
    else if (role == Qt::DecorationRole) 
    {
        switch (m_types.at(index.row()))
        {
            case Basic:
                return KIcon("format-text-bold").pixmap(16, 16);//TODO: Better icons!
            case Extended:
                return KIcon("format-justify-fill").pixmap(16, 16);
        }
    }
    return QVariant();
}

QModelIndex SearchModel::index(int row, int column, const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    if (row >= 0 && row < m_types.count())
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex SearchModel::parent(const QModelIndex & index) const
{
    Q_UNUSED(index);
    return QModelIndex();
}

int SearchModel::rowCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return m_types.count();
}

SearchModel::Type SearchModel::typeFromIndex(int index)
{
    if (index >= 0 && index < m_types.count()) {
        return m_types.at(index);
    }
    return Basic;
}

int SearchModel::indexFromType(Type type)
{
    return m_types.indexOf(type);
}
