/* This file is part of the KDE project

   Copyright (C) 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "ProviderPackageModel.h"

#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamanprovider.h>

#include <KLocale>
#include <KDebug>

using namespace Shaman;

ProviderPackageModel::ProviderPackageModel(QObject * parent)
  : QAbstractItemModel(parent)
{
    foreach (Provider *provider, Base::instance()->core()->providers()) {
        connect(provider, SIGNAL(searchCompleted(QString,Shaman::ProviderPackage::List)),
                this, SLOT(slotSearchCompleted(QString,Shaman::ProviderPackage::List)));
    }
}

ProviderPackageModel::~ProviderPackageModel()
{
}

void ProviderPackageModel::search(const QString &name)
{
    m_packages.clear();
    kDebug() << "Search for" << name << "in" << Base::instance()->core()->providers().count() << "plugins";
    foreach (Provider *provider, Base::instance()->core()->providers()) {
        provider->search(name);
    }
}

void ProviderPackageModel::slotSearchCompleted(QString , const ProviderPackage::List &packages)
{
    m_packages << packages;
    reset();
}

int ProviderPackageModel::columnCount(const QModelIndex & parent) const
{
    return 2;
}

QVariant ProviderPackageModel::data(const QModelIndex & index, int role) const
{
    ProviderPackage * package = static_cast<ProviderPackage*>(index.internalPointer());
    if (role == Qt::DisplayRole) {
        switch (index.column()) {
            case 0:
                return package->name();
            case 1:
                return package->version();
        }
    }
    return QVariant();
}

QVariant ProviderPackageModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole) {
        switch (section) {
            case 0:
                return i18n("Name");
            case 1:
                return i18n("Version");
        }
    }
    return QVariant();
}

QModelIndex ProviderPackageModel::index(int row, int column, const QModelIndex & parent) const
{
    if (row >= m_packages.count() || row < 0)
        return QModelIndex();
    return createIndex(row, column, m_packages.at(row));
}

QModelIndex ProviderPackageModel::parent(const QModelIndex & index) const
{
    return QModelIndex();
}

int ProviderPackageModel::rowCount(const QModelIndex & parent) const
{
    if (parent.isValid())
        return 0;
    return m_packages.count();
}

Shaman::ProviderPackage * ProviderPackageModel::packageFromModelIndex(const QModelIndex &index)
{
    if (index.row() > m_packages.count())
        return 0;
    return m_packages.at(index.row());
}
