/* This file is part of the KDE project

   Copyright (C) 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "ProviderUiDialog.h"
#include "ProviderPackageModel.h"
#include "ProcessingHandler.h"

#include <KLocale>
#include <KDebug>

using namespace Shaman;

ProviderUiDialog::ProviderUiDialog(QWidget * parent)
  : KAssistantDialog(parent),
    m_model(0),
    m_handler(0),
    m_select(0),
    m_build(0)
{
    m_handler = new ProcessingHandler(this);
    m_model = new ProviderPackageModel(this);

    QWidget * selectPackages = new QWidget(this);
    selectUi.setupUi(selectPackages);
    selectUi.treeView->setModel(m_model);

    QWidget * building = new QWidget(this);
    buildingUi.setupUi(building);

    m_select = addPage(selectPackages, i18n("Select Packages..."));
    m_build = addPage(building, i18n("Building Packages..."));
    connect(this, SIGNAL(currentPageChanged(KPageWidgetItem*, KPageWidgetItem*)), SLOT(currentPageChanged(KPageWidgetItem*, KPageWidgetItem*)));
    connect(this, SIGNAL(accepted()), SLOT(cleanup()));
    connect(this, SIGNAL(rejected()), SLOT(cleanup()));
    connect(selectUi.searchButton, SIGNAL(clicked()), SLOT(search()));
    connect(m_handler, SIGNAL(newMessage(QString, Provider::MessageLevel)), SLOT(newMessage(QString,Provider::MessageLevel)));
    connect(m_handler, SIGNAL(finished()), SLOT(finished()));
}

ProviderUiDialog::~ProviderUiDialog()
{
}

void ProviderUiDialog::currentPageChanged(KPageWidgetItem * current, KPageWidgetItem * before)
{
    if (current == m_build) {
        kDebug() << "Build";
        QModelIndexList indexes = selectUi.treeView->selectionModel()->selectedRows();
        if (!indexes.isEmpty()) {
            ProviderPackage::List packages;
            foreach (const QModelIndex &idx, indexes) {
                packages << m_model->packageFromModelIndex(idx);
            }
            m_handler->process(packages);
        }
        enableButton(KDialog::User1, false);
    } else if (before == m_build) {
        if (!m_handler->cancelProcessing())
            kDebug() << "Cancelling the processing failed!";//TODO: Show KMessageBox
    }
}

void ProviderUiDialog::finished()
{
    enableButton(KDialog::User1, true);
    enableButton(KDialog::Cancel, false);
}

void ProviderUiDialog::newMessage(const QString &message, Provider::MessageLevel level)
{
    switch (level) {
        case Provider::StatusLevel:
            buildingUi.textEdit->append("<b>" + message + "</b>");
            break;
        case Provider::ProviderMessageLevel:
            buildingUi.textEdit->append("<i>" + message + "</i>");
            break;
        default:
            buildingUi.textEdit->append(message);
            break;
    }
}

void ProviderUiDialog::search()
{
    kDebug() << "Try search";
    m_model->search(selectUi.searchLine->text());
}

void ProviderUiDialog::cleanup()
{
    setCurrentPage(m_select);
}
