/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "TestHooksPlugin.h"
#include <KDebug>
#include <KMessageBox>
#include <klocalizedstring.h>

SHAMAN_PLUGIN_EXPORT(TestHooksPlugin)

TestHooksPlugin::TestHooksPlugin(QObject* parent, const QVariantList&)
    : CorePlugin(parent)
{

}

TestHooksPlugin::~TestHooksPlugin()
{

}

void TestHooksPlugin::init()
{
    kDebug() << "Registering hooks";
    registerHook(PreUpdateRepositoriesHook, "preUpdate");
    registerHook(PostUpdateRepositoriesHook, "postUpdate");
    registerHook(PreTransactionHook, "preTransaction");
    registerHook(PostTransactionHook, "postTransaction");
}

void TestHooksPlugin::preUpdate(const Shaman::Repository::List &repositories)
{
    if (KMessageBox::warningYesNo(0, i18n("Hi! You're about to update your database. Are you sure?")) == KMessageBox::Yes) {
        emit hookCompleted(PreUpdateRepositoriesHook, Success);
    } else {
        emit hookCompleted(PreUpdateRepositoriesHook, SuccessWithWarning);
    }
}

void TestHooksPlugin::postUpdate(const Shaman::Repository::List &repositories)
{
    QString pnames;
    foreach (Shaman::Repository *repository, repositories) {
        pnames.append(repository->name());
        pnames.append(QChar(' '));
    }

    KMessageBox::information(0, i18n("It's always me! You updated %1", pnames));
    emit hookCompleted(PostUpdateRepositoriesHook, Success);
}

void TestHooksPlugin::preTransaction(const Shaman::AbstractPackage::List& packages)
{
    if (KMessageBox::warningYesNo(0, i18n("Hi! You're about to process a queue. Are you sure?")) == KMessageBox::Yes) {
        emit hookCompleted(PreTransactionHook, Success);
    } else {
        KMessageBox::information(0, i18n("Shame. I'm aborting the transaction then"));
        emit hookCompleted(PreTransactionHook, Fatal);
    }
}

void TestHooksPlugin::postTransaction(const Shaman::AbstractPackage::List& packages)
{
    QString pnames;
    foreach (Shaman::AbstractPackage *package, packages) {
        pnames.append(package->name());
        pnames.append(QChar(' '));
    }

    KMessageBox::information(0, i18n("It's always me! You processed: %1", pnames));
    emit hookCompleted(PostTransactionHook, Success);
}

