/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "ShamanGeneralKCM.h"
#include <QGroupBox>
#include <klocalizedstring.h>
#include <QFormLayout>
#include <QCheckBox>
#include <KPluginSelector>
#include <KService>
#include <KServiceTypeTrader>
#include <KPluginInfo>

#include "settings.h"

K_PLUGIN_FACTORY(ShamanKCMFactory,
                 registerPlugin<ShamanGeneralKCM>();
                )
K_EXPORT_PLUGIN(ShamanKCMFactory("kcm_shaman_general"))

ShamanGeneralKCM::ShamanGeneralKCM(QWidget* parent, const QVariantList& args)
    : KCModule(ShamanKCMFactory::componentData(), parent, args)
{
    setButtons(Apply);

    KAboutData *about =
        new KAboutData("kcm_shaman_general", "kcm_shaman_general", ki18n("Shaman Configuration"),
                       "1.0", ki18n("General configuration for Shaman"),
                       KAboutData::License_GPL, ki18n("(c), 2009 Dario Freddi"),
                       ki18n("From this module, you can configure general settings of Shaman and its plugins"));

    about->addAuthor(ki18n("Dario Freddi"), ki18n("Maintainer") , "drf@kde.org",
                     "http://drfav.wordpress.com");

    setAboutData(about);

    QGroupBox *generalBox = new QGroupBox(i18n("General settings"));
    QFormLayout *generalLay = new QFormLayout;
    m_saveQueue = new QCheckBox(i18n("Save queue on exit"));
    generalLay->addWidget(m_saveQueue);
    generalBox->setLayout(generalLay);

    QGroupBox *pluginBox = new QGroupBox(i18n("Plugins"));
    QVBoxLayout *pluginLay = new QVBoxLayout;
    m_selector = new KPluginSelector();

    KService::List offers = KServiceTypeTrader::self()->query("Shaman/Plugin", "[X-KDE-Shaman-plugin-type] == 'Core'");

    m_selector->addPlugins(KPluginInfo::fromServices(offers), KPluginSelector::ReadConfigFile,
            i18n("Core Plugins"), "Service", KGlobal::config());

    offers = KServiceTypeTrader::self()->query("Shaman/Plugin", "[X-KDE-Shaman-plugin-type] == 'GUI'");

    m_selector->addPlugins(KPluginInfo::fromServices(offers), KPluginSelector::ReadConfigFile,
            i18n("GUI Plugins"), "Service", KGlobal::config());

    offers = KServiceTypeTrader::self()->query("Shaman/Plugin", "[X-KDE-Shaman-plugin-type] == 'Provider'");

    m_selector->addPlugins(KPluginInfo::fromServices(offers), KPluginSelector::ReadConfigFile,
            i18n("Provider Plugins"), "Service", KGlobal::config());

    pluginLay->addWidget(m_selector);
    pluginBox->setLayout(pluginLay);

    QVBoxLayout *lay = new QVBoxLayout;
    lay->addWidget(generalBox);
    lay->addWidget(pluginBox);

    setLayout(lay);

    connect(m_selector, SIGNAL(changed(bool)), this, SLOT(changed()));
    connect(m_saveQueue, SIGNAL(stateChanged(int)), this, SLOT(changed()));
}

ShamanGeneralKCM::~ShamanGeneralKCM()
{
}

void ShamanGeneralKCM::load()
{
    m_selector->load();
    m_saveQueue->setChecked(Settings::saveQueueOnExit());
}

void ShamanGeneralKCM::save()
{
    m_selector->save();
    Settings::setSaveQueueOnExit(m_saveQueue->isChecked());
    Settings::self()->writeConfig();
}

void ShamanGeneralKCM::defaults()
{
    m_selector->defaults();
}

