/* This file is part of the KDE project

   Copyright (C) 2010 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "GuiPluginLoader.h"
#include "scripting/ScriptGuiPlugin.h"
#include "shamanmacros.h"
#include <KDebug>

namespace Shaman {

GuiPluginLoader::GuiPluginLoader()
{
    setPluginType("GUI");
}

GuiPluginLoader::~GuiPluginLoader()
{

}

Shaman::AbstractPlugin* GuiPluginLoader::loadInstance(KService::Ptr offer, QObject* parent)
{
    //try to load the specified library
    KPluginFactory *factory = KPluginLoader(offer->library()).factory();

    if (!factory)
    {
        kError(5001) << "KPluginFactory could not load the plugin:" << offer->library();
        return 0;
    }

    GuiPlugin * plugin = factory->create< GuiPlugin >(parent);

    if ( !plugin ) {
        // Troubles...
        kWarning() << "failed to load" << offer->desktopEntryName();
        return 0;
    }

    if (offer->property("X-KDE-Shaman-plugin-script").isValid()) {
        // FIXME: force cast
        ScriptGuiPlugin * scriptPlugin = (ScriptGuiPlugin*)plugin;
        QString scriptname = SHAMAN_SCRIPTED_PLUGINS_INSTALL_DIR;
        if (!scriptname.endsWith('/')) {
            scriptname.append('/');
        }
        scriptname.append(offer->property("X-KDE-Shaman-plugin-script").toString());
        scriptPlugin->loadScript(scriptname);
        kDebug() << "Gui-Plugin script" << scriptname;
    }

    return plugin;
}

}
