/* This file is part of the KDE project

   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "ScriptGuiPlugin.h"
#include "shamancoreinterface.h"
#include "shamanguiinterface.h"
#include "shamanstatusbarinterface.h"
#include "wrapper/PackageGroupWrapper.h"
#include "wrapper/RepositoryWrapper.h"
#include "wrapper/BackendWrapper.h"

#include <kross/core/action.h>
#include <kross/core/manager.h>

class ScriptGuiPlugin::Private
{
    public:
        Private()
          : action(0),
            object(0)
        {}
        ~Private() {}

        Kross::Action * action;
        Kross::Object::Ptr object;
        BackendWrapper * backendWrapper;
};

/*QVariant PackageGroupHandler(void* ptr)
{
    Shaman::PackageGroup * group = static_cast<Shaman::PackageGroup*>(ptr);
    PackageGroupWrapper * wrapper = new PackageGroupWrapper(group, group);
    QVariant v;
    v.setValue((QObject*)wrapper);
    return v;
}

QVariant RepositoryHandler(void* ptr)
{
    Shaman::Repository * repo = static_cast<Shaman::Repository*>(ptr);
    RepositoryWrapper * wrapper = new RepositoryWrapper(repo, repo);
    QVariant v;
    v.setValue((QObject*)wrapper);
    return v;
}*/

ScriptGuiPlugin::ScriptGuiPlugin(QObject * parent, const QVariantList &)
  : Shaman::GuiPlugin(parent),
    d(new Private())
{
    d->backendWrapper = new BackendWrapper(this);
    registerHook(Shaman::CorePlugin::PreTransactionHook, "preTransactionHook");
    registerHook(Shaman::CorePlugin::PostTransactionHook, "postTransactionHook");
    registerHook(Shaman::CorePlugin::PreUpdateRepositoriesHook, "preUpdateRepositoriesHook");
    registerHook(Shaman::CorePlugin::PostUpdateRepositoriesHook, "postUpdateRepositoriesHook");
    registerHook(Shaman::CorePlugin::PostBackendReloadedHook, "postBackendReloadedHook");
    
    Kross::Manager::self().setStrictTypesEnabled(false);
    //Kross::Manager::self().registerMetaTypeHandler("Shaman::PackageGroup*", PackageGroupHandler);
    //Kross::Manager::self().registerMetaTypeHandler("Shaman::Repository*", RepositoryHandler);
}

ScriptGuiPlugin::~ScriptGuiPlugin()
{
    delete d;
}

void ScriptGuiPlugin::init()
{
    if (d->object && d->object->methodNames().contains("init")) {
        d->object->callMethod("init");
    }
}

void ScriptGuiPlugin::loadScript(const QString &script)
{
    d->action = new Kross::Action(this, script);

    d->action->addObject(this, "GuiPlugin");
    d->action->addObject(core(), "CoreInterface");
    d->action->addObject(gui(), "GuiInterface");
    d->action->addObject(statusBar(), "StatusBar");
    d->action->addObject(d->backendWrapper, "BackendWrapper");

    d->action->setFile(script);
    d->action->trigger();
}

void ScriptGuiPlugin::setGuiPluginClass(Kross::Object::Ptr obj)
{
    d->object = obj;
}

void ScriptGuiPlugin::hookMessage(const QString &message)
{
    GuiPlugin::hookMessage(message);
}

void ScriptGuiPlugin::setHookCompleted(Shaman::CorePlugin::Hook hook, Shaman::CorePlugin::HookResult result)
{
    emit hookCompleted(hook, result);
}

void ScriptGuiPlugin::preTransactionHook(const Shaman::AbstractPackage::List &packages)
{
    if (d->object && d->object->methodNames().contains("preTransactionHook")) {
        //QVariant v;
        //v.setValue(packages);
        d->object->callMethod("preTransactionHook");
    }
    else
        emit hookCompleted(Shaman::CorePlugin::PreTransactionHook, Shaman::CorePlugin::Success);
}

void ScriptGuiPlugin::postTransactionHook(const Shaman::AbstractPackage::List &packages)
{
    if (d->object && d->object->methodNames().contains("postTransactionHook")) {
        //QVariant v;
        //v.setValue(packages);
        d->object->callMethod("postTransactionHook");
    }
    else
        emit hookCompleted(Shaman::CorePlugin::PostTransactionHook, Shaman::CorePlugin::Success);
}

void ScriptGuiPlugin::preUpdateRepositoriesHook(const Shaman::Repository::List &repositories)
{
    if (d->object && d->object->methodNames().contains("preUpdateRepositoriesHook")) {
        /*QList<QObject*> repos;
        foreach (Shaman::Repository * repo, repositories)
            repos << qobject_cast<QObject*>(repo);
        QVariant v;
        v.setValue(repos);*/
        d->object->callMethod("preUpdateRepositoriesHook");
    }    
    else
        emit hookCompleted(Shaman::CorePlugin::PreUpdateRepositoriesHook, Shaman::CorePlugin::Success);
}

void ScriptGuiPlugin::postUpdateRepositoriesHook(const Shaman::Repository::List &repositories)
{
    if (d->object && d->object->methodNames().contains("postUpdateRepositoriesHook")) {
        //QVariant v;
        //v.setValue(repositories);
        d->object->callMethod("postUpdateRepositoriesHook");
    }
    else
        emit hookCompleted(Shaman::CorePlugin::PostUpdateRepositoriesHook, Shaman::CorePlugin::Success);
}

void ScriptGuiPlugin::postBackendReloadedHook()
{
    if (d->object && d->object->methodNames().contains("postBackendReloadedHook")) {
        d->object->callMethod("postBackendReloadedHook");
    }
    else
        emit hookCompleted(Shaman::CorePlugin::PostBackendReloadedHook, Shaman::CorePlugin::Success);
}
