/* This file is part of the KDE project

   Copyright (C) 2008-2010 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanguiinterface_p.h"

#include <QDockWidget>
#include <shamanbase.h>
#include "GuiPluginLoader.h"
#include "shamanstatusbarinterface.h"

namespace Shaman {

class GuiInterfaceHelper
{
  public:
    GuiInterfaceHelper() : q(0) {}
    ~GuiInterfaceHelper() { delete q; }
    GuiInterface *q;
};

K_GLOBAL_STATIC(GuiInterfaceHelper, s_globalGuiInterface)

GuiInterface *GuiInterface::instance()
{
    if (!s_globalGuiInterface->q) {
        new GuiInterface(Shaman::Base::instance());
    }

    return s_globalGuiInterface->q;
}

GuiInterface::GuiInterface(QObject * parent)
  : QObject(parent)
  , d_ptr(new GuiInterfacePrivate)
{
    Q_ASSERT(!s_globalGuiInterface->q);
    s_globalGuiInterface->q = this;
    Shaman::Base::instance()->registerPluginLoader(new GuiPluginLoader);
}

GuiInterface::~GuiInterface()
{
}

void GuiInterface::setMainWidget(QWidget * widget)
{
    emit createMainWidget(widget);
}

void GuiInterface::addDockWidget(Qt::DockWidgetArea area, QDockWidget * widget)
{
    emit createDockWidget(area, widget);
}

void GuiInterface::addAction(QAction * action)
{
    emit createAction(action);
}

void GuiInterface::setStatusBar(QStatusBar* bar)
{
    Q_D(GuiInterface);
    d->statusBar = new StatusBarInterface(bar, this);
}

StatusBarInterface* GuiInterface::statusBar()
{
    Q_D(GuiInterface);
    return d->statusBar;
}

}

#include "shamanguiinterface.moc"
