/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANGUIINTERFACE_H
#define SHAMANGUIINTERFACE_H

#include "shamanmacros.h"
#include <QObject>
class QWidget;
class QDockWidget;
class QAction;
class QStatusBar;

namespace Shaman {

class StatusBarInterface;
class GuiInterfacePrivate;
/** \class GuiInterface shamanguiinterface.h
 * \brief Shaman's GuiInterface-class
 *
 * This class is the connection of the GuiPlugin to the Gui of the Program
 * using libshaman. To access this class the GuiPlugin should use the gui()-function.
 *
 * \code
 * QLabel * widget = new QLabel("Hello World!", 0);
 * gui()->setMainWidget(widget);
 * \endcode
 *
 * This creates a QLabel showing "Hello World!" and sets it as MainWidget afterwards.
 */
class SHAMAN_EXPORT GuiInterface : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(GuiInterface)
    Q_DECLARE_PRIVATE(GuiInterface)
    public:
        static GuiInterface *instance();
        virtual ~GuiInterface();

    public Q_SLOTS:
        /**
         * Sets the MainWidget of the Program
         * @param widget the new MainWidget
         */
        void setMainWidget(QWidget * widget);

        /**
         * This adds a new DockWidget to the Program
         * @param area the QDockWidget area where the Widget should be added
         * @param widget the QDockWidget which should be added
         */
        void addDockWidget(Qt::DockWidgetArea area, QDockWidget * widget);

        /**
         * This adds a new QAction to the program. Most probably this will be
         * visible in the UserInterface as a Button in the ToolBar or as a Menu-Entry.
         * @param action the action to add
         */
        void addAction(QAction * action);

        StatusBarInterface *statusBar();

        void setStatusBar(QStatusBar *bar);

    Q_SIGNALS:
        void createMainWidget(QWidget * widget);
        void createDockWidget(Qt::DockWidgetArea area, QDockWidget * widget);
        void createAction(QAction * action);

    private:
        GuiInterface(QObject * parent);

        GuiInterfacePrivate * const d_ptr;
};
}
#endif
