/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 - 2010 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanstatusbarinterface.h"

#include <QStatusBar>
#include <QTimer>
#include <QProgressBar>
#include <QLabel>

namespace Shaman {
class StatusBarInterface::Private
{
    public:
        Private(QStatusBar * sb)
          : statusBar(sb)
          , resetTimeout(new QTimer)
        {
            textLabel = new QLabel(sb);
            pixmapLabel = new QLabel(sb);
            progressBar = new QProgressBar(sb);
            sb->addWidget(pixmapLabel);
            sb->addWidget(textLabel);
            sb->addWidget(progressBar);
        }
        ~Private()
        {
            resetTimeout->deleteLater();
        }

        QStatusBar * statusBar;
        QLabel * textLabel;
        QLabel * pixmapLabel;
        QProgressBar * progressBar;
        QString standardText;
        QPixmap standardPixmap;
        QTimer *resetTimeout;
};

StatusBarInterface::StatusBarInterface(QStatusBar * statusBar, QObject * parent)
  : QObject(parent),
    d(new Private(statusBar))
{
    d->resetTimeout->setSingleShot(true);
    connect(d->resetTimeout, SIGNAL(timeout()), this, SLOT(reset()));
    reset();
}

StatusBarInterface::~StatusBarInterface()
{
}

void StatusBarInterface::showMessage(const QString &text, const QPixmap &pixmap, int timeout)
{
    d->textLabel->setText(text);
    d->pixmapLabel->setPixmap(pixmap);
    if (timeout > 0) {
        d->resetTimeout->start(timeout * 1000);
    } else {
        d->resetTimeout->stop();
    }
}

void StatusBarInterface::setStandardMessage(const QString &text, const QPixmap &pixmap)
{
    d->standardText = text;
    d->standardPixmap = pixmap;
    if (!d->resetTimeout->isActive()) {
        reset();
    }
}

void StatusBarInterface::setProgress(int percent)
{
    if (percent >= 0) {
        d->progressBar->setRange(0,100);
        d->progressBar->setValue(percent);
    } else {
        d->progressBar->setRange(0,0);
    }

    if (!percent) {
        d->progressBar->hide();
    } else if (d->progressBar->isHidden()) {
        d->progressBar->show();
    }
}

void StatusBarInterface::reset()
{
    setProgress(0);
    d->textLabel->setText(d->standardText);
    d->pixmapLabel->setPixmap(d->standardPixmap);
}
}

#include "shamanstatusbarinterface.moc"
