/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANMAINWINDOW_H
#define SHAMANMAINWINDOW_H

#include <KXmlGuiWindow>
#include <QtCore/QPointer>
#include <shamanbackend.h>

class KAboutData;
class KDialog;
class QDockWidget;
class QWidget;
class KToggleAction;

namespace Shaman
{
    class Base;
}

using namespace Shaman;

class MainWindow : public KXmlGuiWindow
{
    Q_OBJECT
    Q_ENUMS(ConfigSaveType)
    public:
        enum ConfigSaveType {
            NoType = 0,
            GeneralSettingsType = 1,
            BackendSettingsType = 2
        };
        Q_DECLARE_FLAGS(ConfigSaveTypes, ConfigSaveType)

        MainWindow(KAboutData *data);
        ~MainWindow();

    private slots:
        void setUpGui();
        void slotCreateMainWidget(QWidget * widget);
        void slotCreateDockWidget(Qt::DockWidgetArea area, QDockWidget * dockwidget);
        void slotCreateAction(QAction * action);
        void slotQuit();
        void slotPreferences();
        void slotConfigureNotifications();
        void slotConfigureKeys();
        void slotConfigureToolbars();
        void slotShowMenubar();
        void slotSaveMyself();
        void slotNewToolbarConfig();
        void slotAllowedAreasChanged(Qt::DockWidgetAreas);
        void backendStateChanged(Shaman::Backend::State state);
        void aboutToQuit();
        void newTransactionQuestion(Shaman::TransactionQuestion *question);
        void slotActionActivated(uint);
        void slotUpdateSystem();
        void slotUpgradeablePackagesChanged(Shaman::AbstractPackage::List);
        void slotConfigCommitted();
        void slotConfigDialogFinished();

    signals:
        void shamanReady();
        void configChanged();

    private:
        Shaman::Base *m_shaman;
        Shaman::GuiInterface *m_shamanGui;
        KToggleAction *m_menubarAction;
        QMainWindow *m_trickWindow;
        bool m_isReloading;
        KAboutData *m_aboutData;
        QList<QAction*> m_queueActions;
        QList<QAction*> m_reenableActions;
        int m_dockWidgetIndex;
        bool m_isFirstReload;
        QHash<QString, ConfigSaveType> m_configModules;
        ConfigSaveTypes m_saved;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(MainWindow::ConfigSaveTypes)

#endif
