/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "QueueSerializer.h"

#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamanbackend.h>

QHash< QString, int > QueueSerializer::queueSerialized()
{
    Shaman::AbstractPackage::List packages = Shaman::Base::instance()->core()->backend()->packages();
    QHash<QString, int> rethash;

    Shaman::AbstractPackage::List::const_iterator i;
    for (i = packages.constBegin(); i != packages.constEnd(); ++i) {
        if ((*i)->action() != Shaman::AbstractPackage::NoAction) {
            rethash.insert((*i)->name() + "%_sep_%" + (*i)->version(), (int)((*i)->action()));
        }
    }

    return rethash;
}

void QueueSerializer::saveQueueToConfigGroup(KConfigGroup *group)
{
    QHash< QString, int > serialized = queueSerialized();
    // Clean the group first
    group->deleteGroup();

    QHash< QString, int >::const_iterator i;
    for (i = serialized.constBegin(); i != serialized.constEnd(); ++i) {
        group->writeEntry(i.key(), i.value());
    }

    group->config()->sync();
}

void QueueSerializer::saveQueueToFile(const KUrl& path)
{
    KSharedConfig::Ptr config = KSharedConfig::openConfig(path.pathOrUrl());
    KConfigGroup group(config, "QueueSerialized");
    saveQueueToConfigGroup(&group);
    config.data()->sync();
}

void QueueSerializer::loadQueueFromSerialized(const QHash< QString, int >& queue)
{
    Shaman::AbstractPackage::List packages = Shaman::Base::instance()->core()->backend()->packages();

    Shaman::AbstractPackage::List::const_iterator i;
    for (i = packages.constBegin(); i != packages.constEnd(); ++i) {
        QString match = (*i)->name() + "%_sep_%" + (*i)->version();
        if (queue.contains(match)) {
            (*i)->setAction((Shaman::AbstractPackage::Action)(queue[match]));
        }
    }
}

void QueueSerializer::loadQueueFromConfigGroup(const KConfigGroup& group)
{
    QMap<QString, QString> entries = group.entryMap();
    QHash<QString, int> rethash;

    QMap<QString, QString>::const_iterator i;
    for (i = entries.constBegin(); i != entries.constEnd(); ++i) {
        // Just guessing, I wonder why it is a map
        int res = group.readEntry<int>(i.key(), -1);
        if (res != -1) {
            rethash.insert(i.key(), res);
        }

        res = group.readEntry<int>(i.value(), -1);
        if (res != -1) {
            rethash.insert(i.value(), res);
        }
    }

    loadQueueFromSerialized(rethash);
}

void QueueSerializer::loadQueueFromFile(const KUrl& path)
{
    KSharedConfig::Ptr config = KSharedConfig::openConfig(path.pathOrUrl());
    KConfigGroup group(config, "QueueSerialized");
    loadQueueFromConfigGroup(group);
}

