/* This file is part of the KDE project

   Copyright (C) 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/ 
#include "DownloadView.h"

#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamantransactioninterface.h>

#include <QProgressBar>
#include <QLabel>
#include <QFormLayout>

#include <KLocale>

#include <kio/global.h>

DownloadView::DownloadView(QWidget * parent)
  : QScrollArea(parent)
{
    setWidgetResizable(true);
    m_layout = new QFormLayout(this);
    QWidget *widget = new QWidget;
    widget->setLayout(m_layout);

    connect(Shaman::Base::instance()->core()->transactionInterface(),
            SIGNAL(packageDownloadProgressChanged(Shaman::AbstractPackage *, qint64, qint64)),
            this, SLOT(packageDownloadProgressChanged(Shaman::AbstractPackage *, qint64, qint64)));
}

DownloadView::~DownloadView()
{
}

void DownloadView::packageDownloadProgressChanged(Shaman::AbstractPackage *package, qint64 downloadedBytes, qint64 totalBytes)
{
    if (!m_packages.contains(package)) {
        m_packages[package] = new QProgressBar(this);
        m_layout->addRow(i18n("%1:", package->name()), m_packages[package]);

        if (!widget()) {
            // Just don't ask yourself why this one is here. Otherwise it simply won't work
            setWidget(m_layout->parentWidget());
        }
    }

    QProgressBar * progressBar = m_packages[package];
    progressBar->setMaximum(totalBytes);
    progressBar->setValue(downloadedBytes);
    progressBar->setFormat(KIO::convertSize(downloadedBytes) + '/' + KIO::convertSize(totalBytes));
}
