/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "MarkDependenciesDialog.h"

#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamanbackend.h>
#include <QLabel>
#include <klocalizedstring.h>
#include <QProgressBar>
#include <QVBoxLayout>

using namespace Shaman;

MarkDependenciesDialog::MarkDependenciesDialog(QWidget* parent)
    : ErrorQuestionHandler(parent)
    , m_ongoing(false)
{
    // Screw designer.
    QWidget *w = new QWidget;

    m_label = new QLabel(i18n("Please wait, validating transaction..."));
    m_detailsLabel = new QLabel();

    QLabel *pl = new QLabel();
    QPixmap p = KIcon("application-x-archive").pixmap(128,128);
    pl->setPixmap(p);

    m_bar = new QProgressBar;
    m_bar->setRange(0,0);
    m_bar->setFormat(i18n("Marking additional dependencies..."));

    QVBoxLayout *lay = new QVBoxLayout;
    QHBoxLayout *hlay = new QHBoxLayout;

    lay->setSpacing(3);
    lay->addStretch();
    lay->addWidget(m_label);
    lay->addWidget(m_detailsLabel);
    lay->addWidget(m_bar);
    lay->addStretch();

    hlay->addWidget(pl);
    hlay->addLayout(lay);

    w->setLayout(hlay);
    setMainWidget(w);

    setButtons(KDialog::Cancel);
    enableButton(KDialog::Cancel, false);
    setWindowTitle(i18n("Validating targets"));
    setModal(true);

    // Listen to progress
    connect(Base::instance()->core()->backend(), SIGNAL(operationProgress(QString,int)),
            this, SLOT(slotOperationProgress(QString,int)));
}

MarkDependenciesDialog::~MarkDependenciesDialog()
{

}

void MarkDependenciesDialog::markDependencies(const Shaman::ProcessQueueOption::List& options)
{
    connect(Shaman::Base::instance()->core()->backend(), SIGNAL(stateChanged(Shaman::Backend::State)),
            this, SLOT(slotStateChanged(Shaman::Backend::State)));
    show();
    Shaman::Base::instance()->core()->backend()->markAdditionalTargetsForQueue(options);
}

void MarkDependenciesDialog::slotStateChanged(Shaman::Backend::State state)
{
    switch (state) {
        case Shaman::Backend::MarkingAdditionalTargets:
            m_ongoing = true;
            break;
        case Shaman::Backend::Idling:
            if (Shaman::Base::instance()->core()->transactionInterface()->lastError() != TransactionInterface::NoError &&
                m_ongoing) {
                m_bar->setRange(0,1);
                m_bar->setValue(0);
                m_bar->setFormat(i18n("Failed"));
                m_label->setText(i18n("Could not validate the transaction"));
                setAttribute(Qt::WA_DeleteOnClose);
                setButtons(KDialog::Close);
            } else {
                emit dependenciesMarked();
                deleteLater();
            }
            break;
        default:
            break;
    }
}

void MarkDependenciesDialog::slotOperationProgress(const QString &operation, int progress)
{
    m_bar->setRange(0, 100);
    m_bar->setValue(progress);
    m_detailsLabel->setText("<i>" + operation + "</i>");
}

