/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef PROCESSQUEUEDIALOG_H
#define PROCESSQUEUEDIALOG_H

#include <shamantransactioninterface.h>
#include "ErrorQuestionHandler.h"
#include <shamanbackend.h>

class ProcessView;
namespace Shaman {
    class Backend;
}

using namespace Shaman;

class QLabel;
class QProgressBar;
class QScrollArea;

class ProcessQueueDialog : public ErrorQuestionHandler
{
    Q_OBJECT
    public:
        explicit ProcessQueueDialog(Backend * backend, QWidget * parent = 0);
        virtual ~ProcessQueueDialog();

        void processQueue(const Shaman::ProcessQueueOption::List &options);

    private slots:
        void stateChanged();
        void downloadProgressChanged(qint64,qint64);
        void downloadProgressReliabilityChanged(bool);
        void errorTriggered(Shaman::TransactionInterface::Error,QString);

    protected:
        virtual void closeEvent(QCloseEvent* e);

    private:
        enum DetailsWidget {
            DownloadViewWidget,
            ProcessViewWidget
        };
        void setDetailsWidget(DetailsWidget widget);

        Backend * m_backend;
        QLabel* m_preHooksLabel;
        QLabel * m_downloadLabel;
        QLabel * m_installLabel;
        QLabel * m_cleanupLabel;
        QLabel* m_postHooksLabel;
        QProgressBar * m_progressBar;
        QWidget * m_containerWidget;
        TransactionInterface* m_transactionInterface;
        bool m_running;
        QPointer<ProcessView> m_processView;
};

#endif
