/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "ProcessQueueHandler.h"

#include "ProcessQueueDialog.h"
#include "ReviewQueueDialog.h"

#include <shamanmacros.h>
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamanbackend.h>

#include <KAction>
#include <KLocale>
#include "MarkDependenciesDialog.h"

ProcessQueueHandler::ProcessQueueHandler(QWidget * parent)
  : QObject(parent)
{
}

ProcessQueueHandler::~ProcessQueueHandler()
{
}

void ProcessQueueHandler::processQueue()
{
    // We're starting brand new, clear the options
    m_options.clear();
    markTargets();
}

void ProcessQueueHandler::markTargets()
{
    if (m_markDependenciesDialog.isNull()) {
        m_markDependenciesDialog = new MarkDependenciesDialog(qobject_cast<QWidget*>(parent()));
    }

    connect(m_markDependenciesDialog.data(), SIGNAL(dependenciesMarked()), this, SLOT(reviewQueue()));
    m_markDependenciesDialog.data()->markDependencies(m_options);
}

void ProcessQueueHandler::unmarkTargets()
{
    Shaman::Base::instance()->core()->backend()->unmarkAdditionalTargets();
}

void ProcessQueueHandler::reviewQueue()
{
    if (m_reviewDialog.isNull()) {
        m_reviewDialog = new ReviewQueueDialog(this, qobject_cast<QWidget*>(parent()));
    }

    //connect(m_reviewDialog.data(), SIGNAL(finished(int)), this, SLOT(processQueueInternal(int)));
    connect(m_reviewDialog.data(), SIGNAL(processQueue(Shaman::AbstractPackage::List)),
SLOT(processQueueInternal(Shaman::AbstractPackage::List)));
    m_reviewDialog.data()->execute(Shaman::Base::instance()->core()->backend()->packages());
}

void ProcessQueueHandler::processQueueInternal(Shaman::AbstractPackage::List packages)
{
    foreach (Shaman::AbstractPackage * package, Shaman::Base::instance()->core()->backend()->packages()) {
        if (package->action() != Shaman::AbstractPackage::NoAction && !packages.contains(package))
            package->setAction(Shaman::AbstractPackage::NoAction);
    }
    if (m_dialog.isNull()) {
        m_dialog = new ProcessQueueDialog(Shaman::Base::instance()->core()->backend(), qobject_cast<QWidget*>(parent()));
    }
    m_dialog.data()->processQueue(m_options);
}
