/* This file is part of the KDE project

   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "ReviewQueueDialog.h"

#include <KLocale>
#include <KDebug>

#include <QHeaderView>
#include <QPainter>
#include <QStyleOptionViewItemV4>
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamanbackend.h>
#include <shamanbackendpackage.h>
#include <QCheckBox>
#include "ProcessQueueHandler.h"

const int SPACING = 2;

ReviewQueueDelegate::ReviewQueueDelegate(QObject * parent)
  : QStyledItemDelegate(parent)
{
}

ReviewQueueDelegate::~ReviewQueueDelegate()
{
}

QWidget * ReviewQueueDelegate::createEditor(QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index) const
{
    Q_UNUSED(option)
    Q_UNUSED(index)
    QCheckBox *checkbox = new QCheckBox(parent);
    connect(checkbox, SIGNAL(stateChanged(int)), SLOT(commitData()));
    
    return checkbox;
}

void ReviewQueueDelegate::commitData()
{
    QStyledItemDelegate::commitData(qobject_cast<QWidget*>(QObject::sender()));
}

void ReviewQueueDelegate::setEditorData(QWidget * editor, const QModelIndex & index) const
{
    qobject_cast<QCheckBox*>(editor)->setChecked(index.data(Qt::CheckStateRole).toBool());
}

void ReviewQueueDelegate::setModelData(QWidget * editor, QAbstractItemModel * model, const QModelIndex & index) const
{
    model->setData(index, qobject_cast<QCheckBox*>(editor)->isChecked() ? 2 : 0, Qt::CheckStateRole);
}

void ReviewQueueDelegate::updateEditorGeometry(QWidget * editor, const QStyleOptionViewItem & option, const QModelIndex & index) const
{
    QCheckBox *checkBox = qobject_cast<QCheckBox*>(editor);
    checkBox->resize(QSize(option.rect.width() - SPACING * 2, option.rect.height() - SPACING * 2));
    checkBox->move(QPoint(option.rect.x() + SPACING, option.rect.y() + SPACING));
}

void ReviewQueueDelegate::paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & idx) const
{
    QStyleOptionViewItemV4 optV4 = option;
    initStyleOption(&optV4, idx);

    Shaman::AbstractPackage * package = m_packages[optV4.text];
    if (!package) {
        QStyledItemDelegate::paint(painter, option, idx);
        return;
    }
    
    painter->save();
    
    painter->setClipRect(optV4.rect);
    
    QStyle *style = QApplication::style();
    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter);
    
    QRect iconRect = optV4.rect;
    iconRect.setSize(QSize(optV4.rect.height() - SPACING * 2, optV4.rect.height() - SPACING * 2));
    iconRect.moveTo(QPoint(optV4.rect.height() / 1.7 + iconRect.x() + SPACING, iconRect.y() + SPACING));

    painter->drawPixmap(iconRect, optV4.icon.pixmap(iconRect.size()));
    
    painter->save();
    
    QFont nameFont = painter->font();
    if (package->action() != Shaman::AbstractPackage::TransactionDependencyUninstall &&
        package->action() != Shaman::AbstractPackage::TransactionDependencyInstall &&
        package->action() != Shaman::AbstractPackage::TransactionDependencyUpgrade) {
        if (idx.data(Qt::CheckStateRole).toInt() == Qt::Checked)
            nameFont.setWeight(QFont::Bold);
    } else {
        bool needed = false;
        const QAbstractItemModel * model = idx.model();
        /*foreach (Shaman::AbstractPackage * p, m_packages.values()) {
            if (p->dependencies().contains(package)) {
                for (int i = 0; i != model->rowCount(); i++) {
                    if (model->index(i, 0).data(Qt::DisplayRole) == p->name() && model->index(i,
0).data(Qt::CheckStateRole).toInt() == Qt::Checked) {
                        needed = true;
                        nameFont.setWeight(QFont::DemiBold);
                        break;
                    }
                }
                if (needed)
                    break;
            }
        }*/
        nameFont.setItalic(true);
    }
    painter->setFont(nameFont);
    
    QRect textRect = optV4.rect;
    textRect.setX(iconRect.x() + iconRect.width() + SPACING);
    textRect = painter->boundingRect(textRect, Qt::AlignLeft | Qt::AlignVCenter, package->name());
    //textRect.setWidth(optV4.rect.width() / 2);

    painter->drawText(textRect, package->name());

    painter->restore();

    QRect typeRect;

    if ((package->action() == Shaman::AbstractPackage::Upgrade || package->action() ==
Shaman::AbstractPackage::TransactionDependencyUpgrade) &&
        Shaman::Base::instance()->core()->backend()->capabilities()->informationCapabilities() &
        Shaman::BackendCapabilities::ShowUpdateType) {
        QString typeText;
        switch (package->updateType()) {
            case Shaman::AbstractPackage::LowPriorityUpdate:
                typeText = i18n("Low Priority Update");
                break;
            case Shaman::AbstractPackage::FeatureEnhancementUpdate:
                typeText = i18n("Feature Enhancement");
                break;
            case Shaman::AbstractPackage::StandardUpdate:
                typeText = i18n("Standard Update");
                break;
            case Shaman::AbstractPackage::BugFixUpdate:
                typeText = i18n("Bug Fix");
                break;
            case Shaman::AbstractPackage::ImportantUpdate:
                typeText = i18n("Important Update");
                break;
            case Shaman::AbstractPackage::SecurityUpdate:
                typeText = i18n("Security Update");
                break;
        }
        typeRect = painter->boundingRect(optV4.rect, Qt::AlignRight | Qt::AlignTop, typeText);
        typeRect.moveTo(QPoint(typeRect.x() - SPACING, typeRect.y() + SPACING));
        painter->drawText(typeRect, typeText);
    }

    QRect versionRect = painter->boundingRect(optV4.rect, Qt::AlignRight | Qt::AlignBottom, i18n("Version: %1",
package->version()));
    versionRect.moveTo(QPoint(versionRect.x() - SPACING, versionRect.y() - SPACING));
    painter->drawText(versionRect, i18n("Version: %1", package->version()));
    
    painter->restore();
}

QSize ReviewQueueDelegate::sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const
{
    return QSize(0, QFontMetrics(option.font).height() * 2 + 4 * SPACING);
}

void ReviewQueueDelegate::addPackage(Shaman::AbstractPackage * package)
{
    m_packages.insert(package->name(), package);
}

void ReviewQueueDelegate::clear()
{
    m_packages.clear();
}

ReviewQueueDialog::ReviewQueueDialog(ProcessQueueHandler *handler, QWidget * parent)
  : KDialog(parent)
  , m_handler(handler)
{
    QWidget * widget = new QWidget(this);
    ui.setupUi(widget);
    setMainWidget(widget);
    setAttribute(Qt::WA_DeleteOnClose);
    setWindowTitle(i18n("Review queue"));

    ui.treeWidget->header()->hide();
    ui.treeWidget->setRootIsDecorated(false);

    m_delegate = new ReviewQueueDelegate(this);

    ui.treeWidget->setItemDelegate(m_delegate);

    connect(ui.changeProcessOptions, SIGNAL(clicked(bool)), this, SLOT(slotChangeProcessingOptions()));
}

ReviewQueueDialog::~ReviewQueueDialog()
{
}

void ReviewQueueDialog::execute(const Shaman::AbstractPackage::List &packages)
{
    //Cleanup
    m_delegate->clear();
    while (ui.treeWidget->topLevelItem(0)) {
        delete ui.treeWidget->takeTopLevelItem(0);
    }

    bool empty = true;
    int installCount = 0;
    int removeCount = 0;
    int upgradeCount = 0;
    int downloadSize = 0;
    int installSize = 0;
    foreach (Shaman::AbstractPackage * package, packages) {
        if (empty && package->action() != Shaman::AbstractPackage::NoAction)
            empty = false;
        if (package->action() == Shaman::AbstractPackage::NoAction)
            continue;
        m_delegate->addPackage(package);
        QTreeWidgetItem * item = new QTreeWidgetItem(ui.treeWidget, QStringList() << package->name());
        switch (package->action()) {
            case Shaman::AbstractPackage::Install:
                item->setIcon(0, KIcon("list-add"));
                if (package->type() == Shaman::AbstractPackage::BackendPackageType)
                    downloadSize += static_cast<Shaman::BackendPackage*>(package)->downloadSize();
                installCount++;
                installSize += package->size();
                break;
            case Shaman::AbstractPackage::Uninstall:
                item->setIcon(0, KIcon("list-remove"));
                removeCount++;
                installSize -= package->size();
                break;
            case Shaman::AbstractPackage::Upgrade:
                if (package->type() == Shaman::AbstractPackage::BackendPackageType)
                    downloadSize += static_cast<Shaman::BackendPackage*>(package)->downloadSize();
            case Shaman::AbstractPackage::TransactionDependencyUpgrade:
                if (Shaman::Base::instance()->core()->backend()->capabilities()->informationCapabilities() &
                    Shaman::BackendCapabilities::ShowUpdateType) {
                    switch (package->updateType()) {
                        case Shaman::AbstractPackage::BugFixUpdate:
                            item->setIcon(0, KIcon("user-invisible", 0, QStringList() << "tools-report-bug"));
                            break;
                        case Shaman::AbstractPackage::FeatureEnhancementUpdate:
                            item->setIcon(0, KIcon("user-invisible", 0, QStringList() << "view-task-add"));
                            break;
                        case Shaman::AbstractPackage::ImportantUpdate:
                            item->setIcon(0, KIcon("user-invisible", 0, QStringList() << "emblem-important"));
                            break;
                        case Shaman::AbstractPackage::SecurityUpdate:
                            item->setIcon(0, KIcon("user-invisible", 0, QStringList() << "security-medium"));
                            break;
                        case Shaman::AbstractPackage::LowPriorityUpdate:
                            item->setIcon(0, KIcon("user-invisible", 0, QStringList() << "download-later"));
                            break;
                        case Shaman::AbstractPackage::StandardUpdate:
                            default:
                            item->setIcon(0, KIcon("user-invisible", 0, QStringList() << "applications-other"));
                            break;
                    }
                } else {
                    item->setIcon(0, KIcon("user-invisible"));
                }
                upgradeCount++;
                break;
            case Shaman::AbstractPackage::TransactionDependencyInstall:
                item->setIcon(0, KIcon("list-add", 0, QStringList() << "emblem-symbolic-link"));
                if (package->type() == Shaman::AbstractPackage::BackendPackageType)
                    downloadSize += static_cast<Shaman::BackendPackage*>(package)->downloadSize();
                installCount++;
                installSize += package->size();
                break;
            case Shaman::AbstractPackage::TransactionDependencyUninstall:
                item->setIcon(0, KIcon("list-remove", 0, QStringList() << "emblem-symbolic-link"));
                removeCount++;
                installSize -= package->size();
                break;
            default:
                break;
        }
        item->setCheckState(0, Qt::Checked);
        if (package->action() != Shaman::AbstractPackage::TransactionDependencyUninstall &&
            package->action() != Shaman::AbstractPackage::TransactionDependencyInstall &&
            package->action() != Shaman::AbstractPackage::TransactionDependencyUpgrade)
        ui.treeWidget->openPersistentEditor(item);
    }
    if (empty) {
        reject();
    } else {
        QString labelText;
        if (installCount)
            labelText.append(i18np("%1 Package to be installed", "%1 Packages to be installed", installCount));
        if (removeCount && installCount)
            labelText.append(", ");
        if (removeCount)
            labelText.append(i18np("%1 Package to be removed", "%1 Packages to be removed", installCount));
        if ((installCount && upgradeCount) || (removeCount && upgradeCount))
            labelText.append(", ");
        if (upgradeCount)
            labelText.append(i18np("%1 Package to be upgraded", "%1 Packages to be upgraded", installCount));
        ui.numberLabel->setText(labelText);
        if (installSize > 0) {
            ui.sizeLabel->setText(i18n("%1 of disk space will be used", KGlobal::locale()->formatByteSize(installSize)));
        } else {
            // Since we're showing how much space is freed we want this number to be positive
            ui.sizeLabel->setText(i18n("%1 of disk space will be freed", KGlobal::locale()->formatByteSize((0 - installSize))));
        }
        ui.downloadLabel->setText(i18n("%1 will be downloaded", KGlobal::locale()->formatByteSize(downloadSize)));
    }

    m_processQueueOptions.clear();
    QVBoxLayout *lay = new QVBoxLayout;

    foreach (Shaman::ProcessQueueOption * const option,
             Shaman::Base::instance()->core()->backend()->availableProcessQueueOptions()) {
        QCheckBox *box = new QCheckBox(option->name());
        box->setWhatsThis(option->description());
        box->setToolTip(option->description());
        lay->addWidget(box);
        m_processQueueOptions.insert(option, box);
    }

    ui.processingOptionsWidget->setLayout(lay);
    ui.processingOptionsWidget->hide();

    setModal(true);

    connect(this, SIGNAL(accepted()), SLOT(slotAccepted()));
    connect(this, SIGNAL(rejected()), SLOT(slotRejected()));

    show();
}

void ReviewQueueDialog::slotAccepted()
{
    Shaman::AbstractPackage::List returnPackages;
    QHash<QString, Shaman::AbstractPackage*> packages = m_delegate->packages();
    while (ui.treeWidget->topLevelItem(0)) {
        QTreeWidgetItem * item = ui.treeWidget->takeTopLevelItem(0);
        if (item->checkState(0) == Qt::Checked)
            returnPackages << packages[item->text(0)];
    }

    Shaman::ProcessQueueOption::List newopts;
    QHash<Shaman::ProcessQueueOption*, QCheckBox*>::const_iterator i;
    for (i = m_processQueueOptions.constBegin(); i != m_processQueueOptions.constEnd(); ++i) {
        if (i.value()->isChecked()) {
            kDebug() << "Option " << i.key()->name() << " enabled";
            newopts.append(i.key());
        }
    }

    m_handler->m_options = newopts;
    m_handler->unmarkTargets();
    m_handler->markTargets();

    emit processQueue(returnPackages);
}

void ReviewQueueDialog::slotRejected()
{
    m_handler->unmarkTargets();
}

void ReviewQueueDialog::slotChangeProcessingOptions()
{
    ui.processingOptionsWidget->setVisible(ui.changeProcessOptions->isChecked());
}
