/* This file is part of the KDE project

   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef REVIEWQUEUEDIALOG_H
#define REVIEWQUEUEDIALOG_H

#include "ui_ReviewQueueDialog.h"

#include <shamanabstractpackage.h>

#include <KDialog>

#include <QStyledItemDelegate>

class ProcessQueueHandler;
class QTreeWidget;
class QTreeWidgetItem;
class QCheckBox;

namespace Shaman {
    class ProcessQueueOption;
}

class ReviewQueueDelegate : public QStyledItemDelegate
{
    Q_OBJECT
    public:
        ReviewQueueDelegate(QObject * parent = 0);
        ~ReviewQueueDelegate();
 
        virtual QWidget * createEditor(QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index) const;
        virtual void setEditorData(QWidget * editor, const QModelIndex & index) const;
        virtual void setModelData(QWidget * editor, QAbstractItemModel * model, const QModelIndex & index) const;
        virtual void updateEditorGeometry(QWidget * editor, const QStyleOptionViewItem & option, const QModelIndex & index) const;

        virtual void paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const;

        virtual QSize sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const;  

        void addPackage(Shaman::AbstractPackage * package);

        void clear();

        QHash<QString, Shaman::AbstractPackage*> packages() { return m_packages; }

    private slots:
        void commitData();

    private:
        QHash<QString, Shaman::AbstractPackage*> m_packages;
};

class ReviewQueueDialog : public KDialog
{
    Q_OBJECT
    public:
        explicit ReviewQueueDialog(ProcessQueueHandler *handler, QWidget * parent = 0);
        ~ReviewQueueDialog();

        void execute(const Shaman::AbstractPackage::List &packages);

    private slots:
        void slotAccepted();
        void slotRejected();
        void slotChangeProcessingOptions();

    signals:
        void processQueue(const Shaman::AbstractPackage::List &packages);

    private:
        Ui::ReviewQueueDialog ui;
        ReviewQueueDelegate * m_delegate;
        ProcessQueueHandler * m_handler;
        QHash<Shaman::ProcessQueueOption*, QCheckBox*> m_processQueueOptions;
};

#endif
